/**
  * Copyright (c) 2004, Telecooperation Office (TecO),
  * Universitaet Karlsruhe (TH), Germany.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
  *
  *   * Redistributions of source code must retain the above copyright
  *     notice, this list of conditions and the following disclaimer.
  *   * Redistributions in binary form must reproduce the above
  *     copyright notice, this list of conditions and the following
  *     disclaimer in the documentation and/or other materials provided
  *     with the distribution.
  *   * Neither the name of the Universitaet Karlsruhe (TH) nor the names
  *     of its contributors may be used to endorse or promote products
  *     derived from this software without specific prior written
  *     permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
  * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
  * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
  * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
  * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 **/


/**
* Processor file for 18f6720.
* Contains processor specific funtions like ADC control
*
* file version: 001 (2004-04-08, sabin (wendhack@teco.edu))
* 				002 2004,06,20 albert, including spi
*				003
*				004 2204 07 07 albert, spi modified to speed up
*				005 2004-10-10 uwe, DelayUs renew
*				006 2004-11-07 sabin, DelayUs new. Inserted global mue with reserved register.
*				007	2004_11_10 sabin, DelayUsLong new. Added global mue_hi with reserved register.
*
* Date:
*  2005_02_01 Daniel & Raphael
*  new Particle Base System organization
*
*/



/*
 * Keep in mind: This file is included by the otap loader.
 * To reduce its code size, place code not needed by the loader
 * in
 *   #ifndef __LOADER_ONLY__
 *   ...
 *   #endif
 */


#include <tools.h>

#include "18f6720.h"


// to make DelayUs as independent as possible from ccs compiler........
uint8_t at 0x53 mue_hi;
uint8_t at 0x54 mue_lo;



//#pragma inline
void DelayMs(uint16_t Minizekunden) wparam  //ZEKUNDEN ZZZZEKUNDEN
{
	for (;Minizekunden;Minizekunden--)
	{
		DelayUs(250);
		DelayUs(250);
		DelayUs(250);
		DelayUs(250);
	}
}


// works only with 3 microsecs and more. absolutely 0,2 microsecs too much.
void DelayUs( uint8_t muesekunden ) wparam
{
		_asm
		movwf	ASM(mue_lo) /* argument muesekunden is passed via wreg */
		movlw	2
		subwf	ASM(mue_lo), F
		nop
		nop
		nop
		nop

	du_HERE:
		nop
		nop
		decfsz 	ASM(mue_lo), F
		bra	du_HERE
		_endasm;
}


#ifndef __LOADER_ONLY__

// 0,2 microsecs too much per muesekunden_hi.
//#pragma inline
void DelayUsLong( uint16_t muesekunden ) wparam
{
		muesekunden -= 3;
		mue_lo = ( uint8_t )muesekunden;
		mue_hi = ( uint8_t )( muesekunden >> 8 );
		++mue_hi;

		_asm
	dul_LOW:
		nop
		nop

	dul_HIGH:
		decfsz 	ASM(mue_lo), F
		bra 	dul_LOW
		decfsz	ASM(mue_hi), F
		bra 	dul_HIGH
		_endasm;
}

void WriteIntEeprom(uint8_t address, uint8_t __PIC_data) wparam
{
	bit_clear(PIC_EECON1, CFGS); 					//access prog or data memory
	bit_clear(PIC_PIR2, EEIF); 					//make sure that the int flag is cleared
	PIC_EEADR = address; 						//load address
	PIC_EEDATA = __PIC_data; 						//load data
	bit_clear(PIC_EECON1, EEPGD); 					//point to data memory
	bit_set(PIC_EECON1, WREN); 					//enable write

	PIC_EECON2 = 0x55; 						//write 0x55
	PIC_EECON2 = 0xAA; 						//write 0xAA
	bit_set(PIC_EECON1, WR); 						//set WR to begin write


	while (!bit_test(PIC_PIR2, EEIF));				//wait for the write to complete
	bit_clear(PIC_EECON1, WREN);					//disable writes
	bit_clear(PIC_PIR2, EEIF);
}


uint8_t ReadIntEeprom(uint8_t address) wparam
{
	bit_clear(PIC_EECON1, CFGS); 					//access data memory
	PIC_EEADR = address; 						//load address
	bit_clear(PIC_EECON1, EEPGD); 					//point to data memory
	bit_set(PIC_EECON1, RD); 						//enable read

	return(PIC_EEDATA); 							//return data
}



void ADCDisable()
{
        bit_clear(PIC_ADCON0,0);                // disable A/D
}

void ADCInit()
{
        PIC_ADCON0 = 0x00; // AN0 channel selected
        PIC_ADCON1 = 0x0D; // AN0 and AN1 are analog
        PIC_ADCON2 = 0x06; // left-justified, fosc/64 highest sampling speed fro 20 MHz
}

void ADCConfig(analog_config conf) wparam
{

	switch(conf)
	{
		case 0:
			PIC_ADCON1=0x0F;
		break;
		case AN0:
			PIC_ADCON1=0x0E;
		break;
		case AN0_AN1:
			PIC_ADCON1=0x0D;
		break;
		case AN0_AN1_AN2:
			PIC_ADCON1=0x0C;
		break;
		case AN0_AN1_AN2_AN3:
			PIC_ADCON1=0x0B;
		break;
		case AN0_AN1_AN2_AN3_AN4:
			PIC_ADCON1=0x0A;
		break;
		case AN0_AN1_AN2_AN3_AN4_AN5:
			PIC_ADCON1=0x09;
		break;
	}

}


void ADCEnable()
{
        bit_set(PIC_ADCON0,0);          // enable A/D
}


uint8_t ADCIsChSet(uint8_t channel) wparam
{
        uint8_t actual;

        actual=((PIC_ADCON0&0x3C)>>channel);
        return (actual!=channel)?0:1;
}


void ADCSetCh(uint8_t channel) wparam
{
        PIC_ADCON0&=0x00;
        PIC_ADCON0|=(channel<<2);

        #ifdef SAVETIMINGS
        DelayUs(50);                                           //find out correct time?????
        #endif
}

void ADCSetClock(uint8_t speed) wparam
{
        PIC_ADCON2&=0xF8;                                 // fosc/64 fastest sampling rate for 20 Mhz
        PIC_ADCON2|=0x06;
}

uint8_t ADCRead8()
{
        uint8_t value;

        PIC_ADCON2&=0x7F;                         // set result to left-justified

        bit_set(PIC_ADCON0,1);                          // set GO/~DONE - start conversion
        while (bit_test(PIC_ADCON0,1)) ;        // wait for ready
        value=PIC_ADRESH;
        return value;
}

uint16_t ADCRead10()
{
        uint16_t value;
        uint8_t temp;

        PIC_ADCON2|=0x80;                         // set to right-justified

        bit_set(PIC_ADCON0,1);                          // set GO/~DONE - start conversion
        while (bit_test(PIC_ADCON0,1)) ;                // wait for ready

        value = 0;
        value=PIC_ADRESH;
        value = value << 8;
        temp = PIC_ADRESL;
        value |= ((uint16_t)temp) & 0x00FF;
        return value;
}




//UART1


//void RPCSerialSendByte(uint8_t data)
void Uart1SendByte(uint8_t __PIC_data) wparam
{
	while (!(bit_test(PIC_PIR1,4))) ; 		// wait for free	TX1IF
	PIC_TXREG1=__PIC_data;
}

uint8_t Uart1GetByte()
{
	return PIC_RCREG1;
}


//uint8_t RPCSerialDataIsThere()
uint8_t Uart1DataIsThere()
{
	if (bit_test(PIC_PIR1,5)) return 1; else return 0;
}


//void RPCSerialWaitForTxEnd()
void Uart1WaitForTxEnd()
{
	while (!bit_test(PIC_TXSTA1,1));
}



// deletes all crap in the buffer
//void RPCSerialFlushUart()
void Uart1FlushReceive()
{
	uint8_t nirvana;
	if (Uart1DataIsThere()) nirvana=PIC_RCREG1;
	if (Uart1DataIsThere()) nirvana=PIC_RCREG1;

}



//uint8_t RPCSerialUartCheckError()
uint8_t Uart1CheckError()
{
	uint8_t error;
	uint8_t buff;

	error=0;
	//check Framing error, if so, read receivebuffer
	if (bit_test(PIC_RCSTA1,2))
	{
		buff=PIC_RCREG1;
		bit_set(error,0);
	}

	//check Overrun error, if so, clear CREN bit (this is for asynchronous mode)
	if (bit_test(PIC_RCSTA1,1))
	{
		bit_clear(PIC_RCSTA1,4);
		bit_set(PIC_RCSTA1,4);
		bit_set(error,1);
	}

	return error;
}


//void RPCSerialInitUart()
// inits and turns on
void Uart1Init(serial_speed speed, serial_mode sync_mode, uint8_t bits) wparam
{
//	output_high( pin_led_1 );

	if ((speed==SERIAL_625000) && (sync_mode==SERIAL_ASYNCHRONOUS) && (bits==8))
	{
		PIC_SPBRG1=0x01;			// 625kbit
		bit_set(PIC_TRISC,6);		// pins are input!, uart controls them
		bit_set(PIC_TRISC,7);
		bit_clear(PIC_TXSTA1,6);	// 8 bit transmit
		bit_set(PIC_TXSTA1,2);		// set high speed
		bit_clear(PIC_TXSTA1,4);	// set asynchronous mode
		bit_set(PIC_TXSTA1,5);		// transmit enabled
		bit_clear(PIC_RCSTA1,6);	// 8 bit reception
		bit_set(PIC_RCSTA1,4);		// enable receiver
		bit_set(PIC_RCSTA1,7);		// enable serial port (power on)
	}

}



//void RPCSerialDisableUart()
void Uart1Disable()
{
	bit_clear(PIC_RCSTA1,7);		// disable serial port (power off)  //DONT ever play with trisc6,7 during uart operation!!!
	bit_set(PIC_TRISC,6);		//tx in input (to make sure)
	bit_set(PIC_TRISC,7);		//rx is input
	bit_clear(PIC_TXSTA1,5);		//disable tx
	bit_clear(PIC_RCSTA1,4);		// disable rx

}


// checks everything , clears errors and makes the port ready
void Uart1Enable()
{
	Uart1CheckError();
	Uart1FlushReceive();
	Uart1WaitForTxEnd();
	bit_set(PIC_TXSTA1,5);		// transmit enabled
	bit_set(PIC_RCSTA1,4);		// enable receiver
	bit_set(PIC_RCSTA1,7);		// enable serial port (power on)
}



// returns 1 if byte is there in waittime
//uint8_t RPCLookForRx(uint16_t waittime)
uint8_t Uart1WaitForRx(uint16_t wait_time) wparam
{
	while (wait_time--)
	{
		Uart1CheckError();
		if (Uart1DataIsThere() ) return 1;
		//if (!input(PIN_RPC_SERIAL_COMM));
	}
	return 0;
}


///TIMER THINGS
//--------------------------------------------------------------------------------------------------------
void Timer3Init(timer_config conf) wparam
{
	switch (conf)
	{
		case CLOCK_32kHz:
			bit_set( PIC_T1CON, 3 );	//TIMER1 oszillator enable (is for timer3 as well)
			PIC_T3CON = 0x77;		//timer3: 2*8bit acces,1:8 prescale,no external sync, input from timer 1 oszi, enable timer
			//setting of tris c0,1 for timer oszi is not necessary, oszillator forces them to input
		break;
	}
}

uint8_t Timer3GetHi()
{
	return PIC_TMR3H;
}
uint8_t Timer3GetLo()
{
	return PIC_TMR3L;
}
void Timer3SetHi(uint8_t val) wparam
{
	PIC_TMR3H=val;
}
void Timer3SetLo(uint8_t val) wparam
{
	PIC_TMR3L=val;
}




#endif // __LOADER_ONLY__



//----------------------------------------------------
// the spi
//only master modes are supported

void SPISetMode(SPI_MODE mode) wparam
{
	//first set the pins correctly

	if (mode==SPI_MODE_TIMER2)
	{
		// this is the mode for rf
		PIC_SSPSTAT=0;
		PIC_SSPCON1=0x23;
	}

	if (mode==SPI_MODE_FAST)
	{
		//PIC_SSPCON1=0x32;	// idle is high, power on, speed in fosc/64
		PIC_SSPCON1=0x30;	// idle is high, power on, speed in fosc/4
		PIC_SSPSTAT=0x80;	// data is transmitted on rising edge, received on falling edge
	}



}

void SPIInit(SPI_MODE mode) wparam
{

	//first set the pins correctly

	bit_clear(PIC_TRISC,5);		//the sdo
	bit_set(PIC_TRISC,4);		//the sdi
	bit_clear(PIC_TRISC,3);		//the clock

	SPISetMode(mode);

}





// shifts one byte out and the rx in at the same time (typical spi process)
uint8_t SPIShift(uint8_t out) wparam
{
  uint8_t in,dummy;

	dummy=PIC_SSPBUF;
	PIC_SSPBUF=out;
	_asm
			LOOP3_1:
				BTFSS 	ASM(RF_SSPSTAT_BF)		// sspstat<bf> wait for free
				BRA    LOOP3_1
	_endasm;

	in=PIC_SSPBUF;
	return in;
}


