/**
  * Copyright (c) 2004, Telecooperation Office (TecO),
  * Universitaet Karlsruhe (TH), Germany.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
  *
  *   * Redistributions of source code must retain the above copyright
  *     notice, this list of conditions and the following disclaimer.
  *   * Redistributions in binary form must reproduce the above
  *     copyright notice, this list of conditions and the following
  *     disclaimer in the documentation and/or other materials provided
  *     with the distribution.
  *   * Neither the name of the Universitaet Karlsruhe (TH) nor the names
  *     of its contributors may be used to endorse or promote products
  *     derived from this software without specific prior written
  *     permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
  * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
  * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
  * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
  * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 **/


/*
*
*	particle flash driver for the atmel AT45DB041B working on the 220 pc
*	todo: define selfttest loopback test pins
*/


/**
* file version: 001 (2004-0707 krohn)
*
*
* Date:
*  2005_02_01 Daniel & Raphael
*  new Particle Base System organization
*
*/


/*
 * Keep in mind: This file is included by the otap loader.
 * To reduce its code size, place code not needed by the loader
 * in
 *   #ifndef __LOADER_ONLY__
 *   ...
 *   #endif
 */


#include "at45db041.h"
#include "coreboard.h"


#define FlashEnable() do { SPIInit(SPI_MODE_FAST); output_low(PIN_FLASH_CS); } while(0)

#define FlashDisable() output_high(PIN_FLASH_CS) 

#define FlashOff() output_low(PIN_FLASH_POWER)

#define FlashOn() output_high(PIN_FLASH_POWER)



/*

	data is clocked into flash with rising edge
	data is clocked out with falling edge

*/


//OPCODES
#define FLASH_BUFFER_1_WRITE					0x84
#define FLASH_BUFFER_2_WRITE					0x87
#define FLASH_BUFFER_1_READ   					0x54
#define FLASH_BUFFER_2_READ						0x56

#define FLASH_STATUS_READ						0x57
#define	FLASH_FLASH_CONTINUOUS_READ				0x68
#define FLASH_PAGE_READ								0x52		// doesn't stop automatically, starts again from beginning of page...
#define FLASH_BUFFER_1_TO_PAGE_ERASE_PROGRAM	0x83

#define FLASH_BUFFER_2_TO_PAGE_ERASE_PROGRAM	0x86
#define FLASH_BUFFER_1_TO_PAGE_PROGRAM			0x88		// has to be erased before.
#define FLASH_BUFFER_2_TO_PAGE_PROGRAM			0x89		// has to be erased before.
#define FLASH_PAGE_ERASE						0x81
#define FLASH_BLOCK_ERASE						0x50		// erases 8 pages, addresses from 0 to 255.

#define FLASH_PAGE_PROGRAM_THRU_BUFFER_1		0x82
#define FLASH_PAGE_PROGRAM_THRU_BUFFER_2		0x85

#define FLASH_PAGE_TO_BUFFER_1_TRANSFER			0x53
#define FLASH_PAGE_TO_BUFFER_2_TRANSFER			0x55
#define FLASH_PAGE_TO_BUFFER_1_COMPARE			0x53
#define FLASH_PAGE_TO_BUFFER_2_COMPARE			0x55//??? opcode
#define FLASH_PAGE_REWRITE_THRU_BUFFER_1		0x58
#define FLASH_PAGE_REWRITE_THRU_BUFFER_2		0x59



// some addresses
#define FLASH_ID_ADDRESS 540408		//start address for ID in external flash: last page, first byte


uint32_t flash_old_write_address;
uint8_t	flash_write_done;
uint8_t flash_active_write_buffer;


void FlashInit()
{
	//set the states correctly
	//set all pins correctly


	bit_clear(TRIS_FLASH_POWER);

	bit_clear(TRIS_POTI_POWER);
	output_high(PIN_POTI_POWER);

	FlashOn();

	bit_clear(TRIS_FLASH_CS);
	FlashDisable();

	DelayMs(100);	//wait time according spec after power on


	flash_old_write_address=0xFFFFFFFF;		//set the old address to a value out of range to force a page load at first access
 	flash_active_write_buffer=1;
	flash_write_done=true;
}

void FlashReadSequenceFromBuffer1(uint16_t pos,uint8_t*__PIC_data, uint16_t len) wparam
{
	uint16_t i;

	FlashEnable();

	SPIShift(FLASH_BUFFER_1_READ);
	SPIShift(FLASH_DONT_CARE);
	SPIShift(pos >> 8);
	SPIShift(pos & 0xFF);
	SPIShift(FLASH_DONT_CARE);

	for (i=0;i<len;i++)
	{
		*__PIC_data=SPIShift(FLASH_DONT_CARE);//one read instruction
		__PIC_data++;
	}


	FlashDisable();
	return;
}

void FlashReadSequenceFromBuffer2(uint16_t pos,uint8_t*__PIC_data, uint16_t len) wparam
{
	uint16_t i;

	FlashEnable();

	SPIShift(FLASH_BUFFER_2_READ);
	SPIShift(FLASH_DONT_CARE);
	SPIShift(pos >> 8);
	SPIShift(pos & 0xFF);
	SPIShift(FLASH_DONT_CARE);

	for (i=0;i<len;i++)
	{
		*__PIC_data=SPIShift(FLASH_DONT_CARE);//one read instruction
		__PIC_data++;
	}


	FlashDisable();
	return;
}

#ifndef __LOADER_ONLY__

uint8_t FlashReadFromBuffer1(uint16_t pos) wparam
{
	uint8_t buff;

	FlashEnable();

	SPIShift(FLASH_BUFFER_1_READ);
	SPIShift(FLASH_DONT_CARE);
	SPIShift(pos >> 8);
	SPIShift(pos & 0xFF);
	SPIShift(FLASH_DONT_CARE);

	buff	=SPIShift(FLASH_DONT_CARE);//one read instruction


	FlashDisable();
	return buff;
}

// nine bit are valid from addr, remains is dont care
uint8_t FlashWriteToBuffer1(uint16_t addr, uint8_t b) wparam
{
	uint8_t buff;

	FlashEnable();

	SPIShift(FLASH_BUFFER_1_WRITE);
	SPIShift(0);
	SPIShift(addr >> 8);
	SPIShift(addr & 0xFF);

	buff=SPIShift(b);

	FlashDisable();
	return buff;
}

// nine bit are valid from addr, remains is dont care
uint8_t FlashWriteToBuffer2(uint16_t addr, uint8_t b) wparam
{
	uint8_t buff;

	FlashEnable();

	SPIShift(FLASH_BUFFER_2_WRITE);
	SPIShift(0);
	SPIShift(addr >> 8);
	SPIShift(addr & 0xFF);

	buff=SPIShift(b);

	FlashDisable();
	return buff;
}

#endif // __LOADER_ONLY__

uint8_t FlashReady()
{
	uint8_t status;

	FlashEnable();
	SPIShift( FLASH_STATUS_READ );
	status = SPIShift( FLASH_DONT_CARE );
	FlashDisable();
	if( bit_test( status, 7 )) return true; else return false;
}

#ifndef __LOADER_ONLY__

void FlashWaitForReady()
{
	while (!FlashReady()) ;  //DelayMs(1);	//let the flash time to do its stuff, wait a ms
}

//returns true if two adresses are in the same page
uint8_t FlashInSamePageOld(uint32_t addr1, uint32_t addr2) wparam
{

	if (  ((uint16_t)(addr1/264)) == ((uint16_t)(addr2/264))  )
	{

		return true;
	}
	else
	{

		return false;
	}

}
#endif // __LOADER_ONLY__


uint8_t FlashInSamePage(uint32_t addr1, uint32_t addr2) wparam
{
	if (addr2>addr1)
	{
		if ( ((addr1%264)+(addr2-addr1))>=264) return false;
	}
	else
	{
		if ( ((addr2%264)+(addr1-addr2)) >=264) return false;
	}
	return true;
}

uint8_t FlashSendAddress(uint32_t flash_address) wparam
{
	uint16_t pagenr;
	uint16_t bytenr;


	pagenr=(uint16_t) (flash_address/264);
	bytenr=(uint16_t) (flash_address-( (uint32_t)(pagenr)*264 )     );

	pagenr<<=1;
	if (bit_test(bytenr,8)) bit_set(pagenr,0);

	//res[0]=hi(pagenr);
	//res[1]=lo(pagenr);
	//res[2]=hi(bytenr);
	//res[2]=lo(bytenr);


	SPIShift(pagenr >> 8);
	SPIShift(pagenr & 0xFF);
	SPIShift(bytenr & 0xFF);


/*
		SPIShift(0);
		SPIShift(0);
		SPIShift(0);
*/
	return 0;
}

#ifndef __LOADER_ONLY__

uint16_t FlashGetBytePos_old(uint32_t flash_addr) wparam
{
	uint16_t pagenr;
	uint16_t bytenr;

	pagenr=(uint16_t) (flash_addr/264);
	bytenr=(uint16_t) (flash_addr-( (uint32_t)(pagenr)*264 )     );
	return bytenr;
}

#endif // __LOADER_ONLY__

uint16_t FlashGetBytePos(uint32_t flash_addr) wparam
{
	return (uint16_t) (flash_addr % 264);
}


// this function positions the read pointer to a certain position and then the user can get continuous data by using SPIShift(),
// no flash disable is processed!!!
uint8_t FlashSetReadPointer(uint32_t flash_start_address) wparam
{
		if (!FlashReady()) return 0;
		FlashEnable();
		SPIShift(FLASH_FLASH_CONTINUOUS_READ);
		FlashSendAddress(flash_start_address );
		SPIShift(FLASH_DONT_CARE);
		SPIShift(FLASH_DONT_CARE);
		SPIShift(FLASH_DONT_CARE);
		SPIShift(FLASH_DONT_CARE);
		return 0;
}

#ifndef __LOADER_ONLY__

// this funciton store the RAM into the page (and erases before) where flash_address belongs to
uint8_t FlashStoreBuffer1(uint32_t flash_address) wparam
{
	FlashEnable();
	//if (!FlashReady()) return 0;
	SPIShift(FLASH_BUFFER_1_TO_PAGE_ERASE_PROGRAM);

	FlashSendAddress(flash_address);
	FlashDisable();
	return 0;
}

// this funciton store the RAM into the page (and erases before) where flash_address belongs to
uint8_t FlashStoreBuffer2(uint32_t flash_address) wparam
{
	FlashEnable();
	//if (!FlashReady()) return 0;
	SPIShift(FLASH_BUFFER_2_TO_PAGE_ERASE_PROGRAM);
	FlashSendAddress(flash_address);
	FlashDisable();
	return 0;
}

// this funciton loads a page (flash_address belongs to) into the buffer1 RAM
uint8_t FlashLoadToBuffer1(uint32_t flash_address) wparam
{
	FlashEnable();
	//if (!FlashReady()) return 0;
	SPIShift(FLASH_PAGE_TO_BUFFER_1_TRANSFER);
	FlashSendAddress(flash_address);
	FlashDisable();
	return 0;
}

// this funciton loads a page (flash_address belongs to) into the buffer1 RAM
uint8_t FlashLoadToBuffer2(uint32_t flash_address) wparam
{
	FlashEnable();
	//if (!FlashReady()) return 0;
	SPIShift(FLASH_PAGE_TO_BUFFER_2_TRANSFER);
	FlashSendAddress(flash_address);
	FlashDisable();
	return 0;
}


//stores the active RAM buff in a page if necessary
uint8_t FlashFlush()
{

	FlashWaitForReady();



	if (flash_write_done==false)
	{

		if (flash_active_write_buffer==1)
		{
			FlashStoreBuffer1(flash_old_write_address);
			flash_write_done=true;
		}
		else
		{
			FlashStoreBuffer2(flash_old_write_address);
			flash_write_done=true;
		}
	}
	FlashWaitForReady();
	return 0;
}



uint8_t FlashChangeWritePage(uint32_t old_addr,uint32_t new_addr) wparam
{
	//the active page has changed and the content of RAM has not yet been saved
	// so first load/save the buffers
	if (!FlashReady()) return 0;
	{
		if (flash_active_write_buffer==1)
		{
			//load new page to buffer2
			FlashLoadToBuffer2(new_addr);
			FlashWaitForReady();
			if (flash_write_done==false)
			{
				FlashStoreBuffer1(old_addr);
				flash_write_done=true;
			}
			flash_active_write_buffer=2;
			flash_old_write_address=new_addr;
		}
		else
		{
			//load new page to buffer1
			FlashLoadToBuffer1(new_addr);
			FlashWaitForReady();
			if (flash_write_done==false)
			{
				FlashStoreBuffer2(old_addr);
				flash_write_done=true;
			}

			flash_active_write_buffer=1;
			flash_old_write_address=new_addr;
		}
	}
	return 0;
}

//this function uses RAM buffer 1
// return = 0 for bad, 1 for good
// if cannot handle 2 page switches!!! (buffer longer 264)
uint8_t FlashWriteSequence(uint32_t flash_start_address, uint8_t* __PIC_data, uint16_t count) wparam
{
	uint16_t i,buffer_pos;
	uint32_t write_address;

	if (count>264) return 0;


	if (!FlashInSamePage(flash_start_address,flash_old_write_address))
	{

		if (!FlashReady()) return 0;
		FlashChangeWritePage(flash_old_write_address,flash_start_address);//the active page has changed and the content of RAM has not yet been saved so first load/save the buffers
	}


	// now check if a page switch will be neccessary during writing, if so, abort if the last write is still in progess
	if ((!(FlashInSamePage(flash_start_address,flash_start_address+(uint32_t)count))) && (!FlashReady())) return 0;

	buffer_pos=FlashGetBytePos(flash_start_address);
	write_address=flash_start_address;

	FlashEnable();
	if (flash_active_write_buffer==1) SPIShift(FLASH_BUFFER_1_WRITE); else SPIShift(FLASH_BUFFER_2_WRITE);
	FlashSendAddress(buffer_pos);

	for (i=0;i<count;i++)
	{
		if (buffer_pos==264)
		{

			buffer_pos=0;
			FlashDisable();
			if (!FlashReady()) return 0;
			//do a page switch
			FlashChangeWritePage(write_address-1,write_address);
			//buffer_pos=buffer_pos-264;
			FlashEnable();
			buffer_pos=0;
			if (flash_active_write_buffer==1) SPIShift(FLASH_BUFFER_1_WRITE); else SPIShift(FLASH_BUFFER_2_WRITE);
			FlashSendAddress(buffer_pos);

		}

		SPIShift(*__PIC_data);
		flash_write_done=false;
		flash_old_write_address=write_address;
		buffer_pos++;
		write_address++;
		__PIC_data++;


	}
	FlashDisable();

	return 1;


}
#endif // __LOADER_ONLY__


uint8_t FlashReadContinuous(uint32_t flash_start_address, uint8_t* __PIC_data, uint16_t count) wparam
{

	uint16_t pos;

	if (!FlashReady()) return 0;
	FlashEnable();
	SPIShift(FLASH_FLASH_CONTINUOUS_READ);
	FlashSendAddress(flash_start_address );
	SPIShift(FLASH_DONT_CARE);
	SPIShift(FLASH_DONT_CARE);
	SPIShift(FLASH_DONT_CARE);
	SPIShift(FLASH_DONT_CARE);

	for (pos=0;pos<count;pos++)
	{
		*__PIC_data=SPIShift(FLASH_DONT_CARE);
		__PIC_data++;
	}
	FlashDisable();
	return 1;

}


uint8_t FlashReadSequence(uint32_t flash_start_address, uint8_t* __PIC_data, uint16_t count) wparam
{
	uint16_t pos,saveCount,flashReadBeforeRAMCount;

	//
	// A write before a read may create a true dependency and therefore result
	// in reading on old data from the flash because the recent data weren't
	// written back to the flash.
	//
	// Solution 1:
	// Explictit write back by calling FlashFlush
	// This costs tiiiiiiiiiiiime!!!!!!!
	//
	// Solution 2:
	// Read from RAM buffer the last recent data
	// Fast, but more complex.
	//
	// Here: we go for solution 2.
	//
	// ToDo: write this function better: FLASH/RAM/FLASH pattern


	if (!FlashReady()) return 0;

	if (  FlashInSamePage(flash_start_address,flash_old_write_address) )
	{

		//
		// first data portion is already in RAM buffer,
		// afterwards read continously from flash
		//

		pos = FlashGetBytePos(flash_start_address); // start address in RAM buffer
		saveCount = count;

		// read RAM buffer, there is the last recent version of page
		FlashEnable();
		if ( flash_active_write_buffer == 1 )
		{
			SPIShift(FLASH_BUFFER_1_READ);
			FlashSendAddress(flash_start_address);
			SPIShift(FLASH_DONT_CARE);
		}

		if ( flash_active_write_buffer == 2 )
		{
			SPIShift(FLASH_BUFFER_2_READ);
			FlashSendAddress(flash_start_address);
			SPIShift(FLASH_DONT_CARE);
		}


		while ( count )
		{

			if (pos == 264) break; // end of page, but no end of read

			*__PIC_data = SPIShift(FLASH_DONT_CARE); // from RAM buffer
			__PIC_data++;
			pos++;
			count--;
		}
		FlashDisable();

		if (count)
		{
			// from now read continuously
			FlashReadContinuous(flash_start_address + (saveCount - count), __PIC_data, count);
		}

		return 1;  // done, return with ok

	}

	// check whether we encounter the buffer
	if ( (flash_start_address <= flash_old_write_address) && (flash_start_address + count) >= flash_old_write_address )
	{
		// during read, we will reach one of the buffers containing
		// the last recent portion of data

		// determine address borders for reading FLASH/RAM/FLASH
		saveCount = count;
		flashReadBeforeRAMCount = flash_old_write_address - flash_start_address - FlashGetBytePos(flash_old_write_address);
		pos = 0;

		//
		// read FLASH
		//
//		if ( flashReadCountBeforeRAM )
//		{
			FlashReadContinuous( flash_start_address, __PIC_data, flashReadBeforeRAMCount );
			count -= flashReadBeforeRAMCount;
			__PIC_data += flashReadBeforeRAMCount;
//		}

		//
		// read RAM
		//
		FlashEnable();
		if ( flash_active_write_buffer == 1 )
		{
			SPIShift(FLASH_BUFFER_1_READ);
			FlashSendAddress(flash_start_address+flashReadBeforeRAMCount);
			SPIShift(FLASH_DONT_CARE);
		}

		if ( flash_active_write_buffer == 2 )
		{
			SPIShift(FLASH_BUFFER_2_READ);
			FlashSendAddress(flash_start_address+flashReadBeforeRAMCount);
			SPIShift(FLASH_DONT_CARE);
		}

		while ( count )
		{

			if (pos == 264) break; // end of page, but no end of read

			*__PIC_data = SPIShift(FLASH_DONT_CARE); // from RAM buffer
			__PIC_data++;
			pos++;
			count--;
		}
		FlashDisable();

		//
		// read FLASH
		//
		if (count)
		{
			// from now read continously
			FlashReadContinuous( flash_start_address + (saveCount-count), __PIC_data, count);
		}

		return 1;  // done, return with ok
	}

	//
	// we do not encounter a buffer, so read continously from flash
	//
	FlashReadContinuous(flash_start_address, __PIC_data, count);

	return 1;  // done, return with ok
}

