/**
  * Copyright (c) 2004, Telecooperation Office (TecO),
  * Universitaet Karlsruhe (TH), Germany.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
  *
  *   * Redistributions of source code must retain the above copyright
  *     notice, this list of conditions and the following disclaimer.
  *   * Redistributions in binary form must reproduce the above
  *     copyright notice, this list of conditions and the following
  *     disclaimer in the documentation and/or other materials provided
  *     with the distribution.
  *   * Neither the name of the Universitaet Karlsruhe (TH) nor the names
  *     of its contributors may be used to endorse or promote products
  *     derived from this software without specific prior written
  *     permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
  * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
  * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
  * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
  * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 **/

#ifndef __AT45DB041_H__
#define __AT45DB041_H__
/*
*
*	particle flash driver for the atmel AT45DB041B working on the 220 pc
*	todo: define selfttest loopback test pins
*/


/**
* file version: 001 (2004-0707 krohn)
*
*
* Date:
*  2005_02_01 Daniel & Raphael
*  new Particle Base System organization
*
*/


/*

	data is clocked into flash with rising edge
	data is clocked out with falling edge

*/

#include <tools.h>

extern uint32_t flash_old_write_address;
extern uint8_t	flash_write_done;
extern uint8_t flash_active_write_buffer;


#ifndef __LOADER_ONLY__

void FlashInit();

void FlashReadSequenceFromBuffer1(uint16_t pos,uint8_t*__PIC_data, uint16_t len) wparam;
void FlashReadSequenceFromBuffer2(uint16_t pos,uint8_t*__PIC_data, uint16_t len) wparam;
uint8_t FlashReadFromBuffer1(uint16_t pos) wparam;

// nine bit are valid from addr, remains is dont care
uint8_t FlashWriteToBuffer1(uint16_t addr, uint8_t b) wparam;

// nine bit are valid from addr, remains is dont care
uint8_t FlashWriteToBuffer2(uint16_t addr, uint8_t b) wparam;

uint8_t FlashReady();

void FlashWaitForReady();

//returns true if two adresses are in the same page
uint8_t FlashInSamePageOld(uint32_t addr1, uint32_t addr2) wparam;
uint8_t FlashInSamePage(uint32_t addr1, uint32_t addr2) wparam;
uint8_t FlashSendAddress(uint32_t flash_address) wparam;
uint16_t FlashGetBytePos_old(uint32_t flash_addr) wparam;
uint16_t FlashGetBytePos(uint32_t flash_addr) wparam;


// this function positions the read pointer to a certain position and then the user can get continuous data by using SPIShift(),
// no flash disable is processed!!!
uint8_t FlashSetReadPointer(uint32_t flash_start_address) wparam;


// this funciton store the RAM into the page (and erases before) where flash_address belongs to
uint8_t FlashStoreBuffer1(uint32_t flash_address) wparam;

// this funciton store the RAM into the page (and erases before) where flash_address belongs to
uint8_t FlashStoreBuffer2(uint32_t flash_address) wparam;

// this funciton loads a page (flash_address belongs to) into the buffer1 RAM
uint8_t FlashLoadToBuffer1(uint32_t flash_address) wparam;

// this funciton loads a page (flash_address belongs to) into the buffer1 RAM
uint8_t FlashLoadToBuffer2(uint32_t flash_address) wparam;


//stores the active RAM buff in a page if necessary
uint8_t FlashFlush();

uint8_t FlashChangeWritePage(uint32_t old_addr,uint32_t new_addr) wparam;

//this function uses RAM buffer 1
// return = 0 for bad, 1 for good
// if cannot handle 2 page switches!!! (buffer longer 264)
uint8_t FlashWriteSequence(uint32_t flash_start_address, uint8_t* __PIC_data, uint16_t count) wparam;
uint8_t FlashReadContinuous(uint32_t flash_start_address, uint8_t* __PIC_data, uint16_t count) wparam;
uint8_t FlashReadSequence(uint32_t flash_start_address, uint8_t* __PIC_data, uint16_t count) wparam;

#endif // __LOADER_ONLY__

#define FLASH_DONT_CARE							0x00



#endif
