/**
  * Copyright (c) 2004, Telecooperation Office (TecO),
  * Universitaet Karlsruhe (TH), Germany.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
  *
  *   * Redistributions of source code must retain the above copyright
  *     notice, this list of conditions and the following disclaimer.
  *   * Redistributions in binary form must reproduce the above
  *     copyright notice, this list of conditions and the following
  *     disclaimer in the documentation and/or other materials provided
  *     with the distribution.
  *   * Neither the name of the Universitaet Karlsruhe (TH) nor the names
  *     of its contributors may be used to endorse or promote products
  *     derived from this software without specific prior written
  *     permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
  * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
  * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
  * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
  * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 **/


/**
* This is the board file for particle computer communication board 210
*
* file version: 001 (2004-11-03, sabin)
* 				identical to pc202.c 005
*				changed PCGetID as to always return production_id as the first two bytes.
*
*
* Date:
*  2005_02_01 Daniel & Raphael
*  new Particle Base System organization
*
*/

#include "board210.h"

#include "../system/rf.h"

#include "pci2c.h"

#include "pceeprom.h"

/* CLOCK_SPEED is defined in the header file */
#include <delay.c>



void PCInit()
{


	ADCInit();

	PCSetDigitalPorts(NORMAL);
	PCSetAnalogPorts(ANALOG_ALL_DIGITAL);

	PCI2CInit();
	PCEepromInit();

	selftest_active=false;
}



uint8_t PCSetDigitalPorts(port_settings choice) wparam
{
	switch(choice)
	{
		case NORMAL:

			PIC_TRISA=0xEF;	PIC_PORTA=0x00;
			PIC_TRISB=0xFF;	PIC_PORTB=0x00;
			PIC_TRISC=0xDB;	PIC_PORTC=0x00;
			PIC_TRISD=0x7A;	PIC_PORTD=0x04;
			PIC_TRISE=0x0C;	PIC_PORTE=0x00;
			PIC_TRISF=0x1B;	PIC_PORTF=0x00;
			PIC_TRISG=0xE4;	PIC_PORTG=0x00;
			// the defaults for A/D Port
			//PIC_ADCON0=0x01;		// Fosc/64;ch=AN0;stop;turnon
			//PIC_ADCON1=0x0B;		// Fosc/64;Aport is all analog ref=Vdd; left jsutified
			//PIC_ADCON2=0x06;
			break;

		case SELFTEST:

			PIC_TRISA=0xD3;	PIC_PORTA=0x00;
			PIC_TRISB=0xFF;	PIC_PORTB=0x00;
			PIC_TRISC=0xDB;	PIC_PORTC=0x00;
			PIC_TRISD=0x02;	PIC_PORTD=0x04;
			PIC_TRISE=0x0C;	PIC_PORTE=0x00;
			PIC_TRISF=0x1A;	PIC_PORTF=0x00;
			PIC_TRISG=0xE4;	PIC_PORTG=0x00;
			//set all digital
			//PIC_ADCON0=0x00;		// Fosc/64;ch=AN0;stop;turnoff
			//PIC_ADCON1=0x0F;		// all digital; vrefs are supply
			//PIC_ADCON2=0x06;		// whatever
			break;

		case ALLIN:
			PIC_TRISA=0xFF;	PIC_PORTA=0xFF;
			PIC_TRISB=0xFF;	PIC_PORTB=0xFF;
			PIC_TRISC=0xFF;	PIC_PORTC=0xFF;
			PIC_TRISD=0xFF;	PIC_PORTD=0xFF;
			PIC_TRISE=0xFF;	PIC_PORTE=0xFF;
			PIC_TRISF=0xFF;	PIC_PORTF=0xFF;
			PIC_TRISG=0xFF;	PIC_PORTG=0xFF;
			//set all digital
			//PIC_ADCON0=0x00;		// Fosc/64;ch=AN0;stop;turnoff
			//PIC_ADCON1=0x0F;		// all digital; vrefs are supply
			//PIC_ADCON2=0x06;		// whatever

			break;
	}
}




//Attention: this function sets tris to different values
uint8_t PCSelfTestBoardIsThere()
{
	uint8_t remember_TRISD;
	uint8_t remember_TRISB;
	uint8_t remember_PORTB;
	uint8_t remember_PORTD;

	boolean isthere;

	remember_TRISD=PIC_TRISD;		//store old values
	remember_TRISB=PIC_TRISB;
	remember_PORTD=PIC_PORTD;		//store old values
	remember_PORTB=PIC_PORTB;


	bit_set(PIC_TRISB,7);
	bit_clear(PIC_TRISD,5);

	//check to see if selftestboard is there

	isthere=true;

	output_high(PIN_D5);
	DelayUs(10);
	if (!input(PIN_B7)) isthere=false;
	output_low(PIN_D5);
	DelayUs(10);
	if (input(PIN_B7)) isthere=false;


	PIC_TRISD=remember_TRISD;		//return to old values
	PIC_TRISB=remember_TRISB;
	PIC_PORTD=remember_PORTD;		//return to old values
	PIC_PORTB=remember_PORTB;

	if (isthere==true)
	{
		selftest_active=true;
		return 1;
	}
	else
	{
		selftest_active=false;
		return 0;
	}
}





//return value 0: no selftest board found, 1: selftest board is there
//
//  uint8_t*result: a 10 byte array is expected

uint8_t PCSelfTest(uint8_t* result) wparam
{

	uint8_t buff,nirvana,i;


	PCSetDigitalPorts(SELFTEST);		//set ports to selftest
	PCSetAnalogPorts(ANALOG_ALL_DIGITAL);

	PCLedRedOff();

	PCLedBlueOn();
 	DelayMs(100);
 	PCLedBlueOff();
	DelayMs(100);
	PCLedBlueOn();
 	DelayMs(100);
 	PCLedBlueOff();
	DelayMs(100);


	PCLedBlueOn();		//start selftest



//new:
// B7-D5,B6-D4,B5-D3,B4-D6,B3-F0,B2-A3,B1-A2,B0-A5  B2-TX,B1-RX
// 1     0     1     0     1     0     1     0 (result0)
// 0     1     0     1     0     1     0     1 (result1)
//----------------------------------------------------

output_high(PIN_D5);
output_low(PIN_D4);
output_high(PIN_D3);
output_low(PIN_D6);
output_high(PIN_F0);
output_low(PIN_A3);
output_high(PIN_A2);
output_low(PIN_A5);

result[0]=PIC_PORTB; //expected: 10101010

output_low(PIN_D5);
output_high(PIN_D4);
output_low(PIN_D3);
output_high(PIN_D6);
output_low(PIN_F0);
output_high(PIN_A3);
output_low(PIN_A2);
output_high(PIN_A5);

result[1]=PIC_PORTB;  //expected: 0x55

//----------------------------------------------------
//now serial
PIC_TRISA=255;
PIC_TRISB=0xFD;
output_high(PIN_B1);			// idle for serial

// uart init
_asm
	movlw	0x40			//129: "19200"	 //movlw 	0x15			// 0d21 =56.818 kbit ="57600"
	movwf 	ASM(PIC_SPBRG1)		//

	bcf		ASM(PIC_TXSTA1),6		// 8 bit transmit
	bsf		ASM(PIC_TXSTA1),2		// set high  speed
	bcf		ASM(PIC_TXSTA1),4		// set asynchronous mode
	bsf		ASM(PIC_TXSTA1),5		// transmit enabled


	bsf		ASM(PIC_RCSTA1),7		// enable serial port (power on)
	bcf		ASM(PIC_RCSTA1),6		// 8 bit reception
	bsf     ASM(PIC_RCSTA1),4		// enable receiver


	bcf		ASM(PIC_TRISC),6		//c6 is output for tx
	bsf		ASM(PIC_TRISC),7		//c7 is input for rx

_endasm;



//check Framing error, if so, read receivebuffer
if (bit_test(PIC_RCSTA1,2))
{
	buff=PIC_RCREG1;
}

//check Overrun error, if so, clear CREN bit
if (bit_test(PIC_RCSTA1,1))
{
	bit_clear(PIC_RCSTA1,4);
}

// set CREN (Continous Receive ENable)
bit_set(PIC_RCSTA1,4);

PIC_TXREG1=255;							// just any data

//FLUSH rx
if (bit_test(PIC_PIR1,5))  nirvana=PIC_RCREG1;
if (bit_test(PIC_PIR1,5))  nirvana=PIC_RCREG1;

//send one byte hardware -> software serial
//while (!(bit_test(PIC_PIR1,4))) ; 		// wait for free
while (!(bit_test(PIC_TXSTA1,1))) ; 		// wait for free TRMT
DelayUs(250);
DelayUs(250);
//PCLedRedOn();
//DelayMs(100);

buff=255;
PIC_TXREG1=139;							// just any data

while ((buff--) && !kbhit());

if (kbhit()) {buff=getch();result[2]=buff;} else result[2]=255;

//PCLedRedOff();
//DelayMs(100);


//send one byte sortware -> hardware serial
putc(117);		// now send on software serial
DelayUs(100);
if (bit_test(PIC_PIR1,5)) result[3]=PIC_RCREG1; else result[3]=255;

//
//----------------------------------------------------
// now analog
//PIC_ADCON0=PIC_ADCON0_SYSTEM_VALUE;
//PIC_ADCON1=PIC_ADCON1_SYSTEM_VALUE;
//PIC_ADCON2=PIC_ADCON2_SYSTEM_VALUE

result[4]=0;//AppGetSupplyVoltage();

//
//----------------------------------------------------
// now I2C, FRAM
result[5]=0;
PIC_TRISD=0x07;
PIC_TRISB=255;


PCEepromInit();
PCEepromOn();

//PCLedRedOn();

if (PCEepromTest()==0) result[5]=28; else result[5]=0;

//PCLedRedOff();

//
//----------------------------------------------------
// now show result

	PIC_TRISB=0xBF;
	buff=0;
	if (result[0]==0xAA) buff++;	//170

	//debuggiveout(result[0]);

	if (result[1]==0x55) buff++;	//85

//		debuggiveout(result[1]);

	if (result[2]==139) buff++;

//		debuggiveout(result[2]);

	if (result[3]==117) buff++;

//		debuggiveout(result[3]);

	if (result[5]==28) buff++;

	//debuggiveout(result[5]);


	if (buff==5) result[9]=0; else result[9]=255;

	DelayMs(800);
	PCLedBlueOff();
	DelayMs(800);

	//PCLedBlueOn();DelayMs(200);PCLedBlueOff();DelayMs(200);
	//PCLedBlueOn();DelayMs(200);PCLedBlueOff();DelayMs(200);
	//PCLedBlueOn();DelayMs(200);PCLedBlueOff();DelayMs(200);

	if (result[9]==0) PCLedBlueOn(); else PCLedRedOn();
	DelayMs(800);
	PCLedBlueOff();
	PCLedRedOff();




return 1;

}



void PCSetAnalogPorts(analog_settings config) wparam
{
        uint8_t temp;

        // delete bits 0-3
        temp = PIC_ADCON1 & 0xF0;

        // set bits 0-3


        switch(config)
        {
                case ANALOG_ALL_DIGITAL:
                      ADCConfig(AN0_AN1);  //PIC_ADCON1 = temp | 0x0D;
                break;
                case ANALOG_CONN_11:
                      ADCConfig(AN0_AN1_AN2);//  PIC_ADCON1 = temp | 0x0C;
                break;
                case ANALOG_CONN_11_12:
                      ADCConfig(AN0_AN1_AN2_AN3);  //PIC_ADCON1 = temp | 0x0B;
                break;
                case ANALOG_CONN_11_12_14:
                       ADCConfig(AN0_AN1_AN2_AN3_AN4);  //PIC_ADCON1 = temp | 0x0A;	//an0-an4 analog
                break;
                case ANALOG_CONN_11_12_13_14:
                        ADCConfig(AN0_AN1_AN2_AN3_AN4_AN5); //PIC_ADCON1 = temp | 0x09;	//an0-AN5 analog
                break;
        }
}





/// for RPC Server:: match UART to RPC


uint8_t RPCServerSerialWaitForRx(uint16_t wait_time) wparam
{
	return Uart1WaitForRx(wait_time);
}

void RPCServerSerialEnable()
{
	Uart1Enable();
}
void RPCServerSerialDisable()
{
	Uart1Disable();
}
void RPCServerSerialInit(serial_speed speed, serial_mode sync_mode, uint8_t bits) wparam
{
	Uart1Init(speed, sync_mode, bits);
}
uint8_t RPCServerSerialCheckError()
{
	return Uart1CheckError();
}
void RPCServerSerialFlushReceive()
{
	Uart1FlushReceive();
}
void RPCServerSerialWaitForTxEnd()
{
	Uart1WaitForTxEnd();
}
uint8_t RPCServerSerialDataIsThere()
{
	return Uart1DataIsThere();
}
void RPCServerSerialSendByte(uint8_t __PIC_data) wparam
{
	Uart1SendByte(__PIC_data);
}
uint8_t RPCServerSerialGetByte()
{
	return Uart1GetByte();
}



void ClockTimerInit(timer_config conf) wparam
{
	 Timer3Init(conf);
}

uint8_t ClockTimerGetHi()
{
	return Timer3GetHi();
}
uint8_t ClockTimerGetLo()
{
	return Timer3GetLo();
}

void ClockTimerSetHi(uint8_t val) wparam
{
	Timer3SetHi(val);
}
void ClockTimerSetLo(uint8_t val) wparam
{
	Timer3SetLo(val);
}


/*
 *
 *	Power RF Module
 *
 */
void RFPowerModuleOn()
{
	output_low(PIN_RF_POWER);	//inverse logic!!
	output_high(PIN_POTI_POWER);
}

void RFPowerModuleOff()
{
	output_high(PIN_RF_POWER);	//inverse logic!!
	output_low(PIN_POTI_POWER);
}




// this is more or less the old LLGetIdFromHardware from the awarecon#084 times
uint8_t PCGetID(uint8_t* p_id) wparam
{
	uint8_t i,buff,result;

		//check if valid ID in Eeprom:
		result=0;
		for (i=0;i<=7;i++)
		{
		  buff=ReadIntEeprom(EEPROM_ADDRESS_ID+i);
		  p_id[i]=buff;
		  if (buff==255) result++;
		}
		if (result<8)
		{
			// ID in internal EEProm was valid!
			// now write it to external eeprom and exit
			PCEepromWriteSequence(EEPROM_EXT_ADDRESS_ID, p_id,8);
			return 1;
		}

		// id in internal EEPROM was invalid; try external eeprom
		PCEepromReadSequence(EEPROM_EXT_ADDRESS_ID,p_id,8);
		result=0;
		for(i=0;i<=7;i++)
		{
			buff=p_id[i];
			if (buff==255) result++;
		}
		if (result<8)
		{
			//external was ok, write it to internal
			for(i=0;i<=7;i++)
			{
				WriteIntEeprom(EEPROM_ADDRESS_ID+i,p_id[i]);
			}
			return 1;
		}

		return 0;		//both were invalid: there's no valid id

}
