/**
  * Copyright (c) 2004, Telecooperation Office (TecO),
  * Universitaet Karlsruhe (TH), Germany.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
  *
  *   * Redistributions of source code must retain the above copyright
  *     notice, this list of conditions and the following disclaimer.
  *   * Redistributions in binary form must reproduce the above
  *     copyright notice, this list of conditions and the following
  *     disclaimer in the documentation and/or other materials provided
  *     with the distribution.
  *   * Neither the name of the Universitaet Karlsruhe (TH) nor the names
  *     of its contributors may be used to endorse or promote products
  *     derived from this software without specific prior written
  *     permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
  * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
  * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
  * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
  * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 **/


/**
* driver for 1-wire ID chip DS 2431 on the particle 224
*
* Author: krohn@teco.edu
* Date: 2004-08-12
*
* 002: some minor things like return vals
* 003: corrected the readbyte one wire to lsb first
* 004: timings corrected uwe
*
* 005: 	Inserted CRC-check (new: IDChipCheckCRC & CRC_TABLE) of ID (in: IDChipGetId).
*		IDChipGetId returns 0 if problem with OneWire stuff or wrong CRC, 1 if everything ok.
*		2004-11-03 by sabin
*
* 006: 	Inserted IDChipReadEeprom, IDChipWriteEeprom, IDChipCheckCRCScratchpad, IDChipWriteProductionID
*		whereas: IDChipCheckCRCScratchpad doesn't work at all, and IDChipWriteProductionID & IDChipWriteEeprom
*		only work without checking results......... preliminary version.....
*		2004-11-07 by sabin
*/


#ifndef __DS2431_H__
#define	__DS2431_H__					006

#include <tools.h>

//functions
void IDChipInit();
void IDChipOn();
void IDChipOff();

uint8_t IDChipGetId(uint8_t *__PIC_data) wparam;
uint8_t IDChipCheckCRCID( uint8_t* __PIC_data ) wparam;
uint8_t IDChipReadEeprom( uint8_t address, uint8_t number_bytes, uint8_t* buff ) wparam;
uint8_t IDChipWriteEeprom( uint8_t address, uint8_t* buff ) wparam;
uint8_t IDChipCheckCRCScratchpad( uint8_t* crc, uint8_t address, uint8_t* buff ) wparam;
uint8_t IDChipWriteProductionID( uint8_t production_id_hi, uint8_t production_id_lo ) wparam;

//#pragma inline
void OneWireHigh();
//#pragma inline
void OneWireLow();
void OneWireInit();

//#pragma inline
uint8_t OneWireRead();
uint8_t OneWirePresencePulse();
void OneWireWriteByte(uint8_t __PIC_data) wparam;
uint8_t OneWireReadByte();

#endif // __DS2431_H__
