/**
  * Copyright (c) 2004, Telecooperation Office (TecO),
  * Universitaet Karlsruhe (TH), Germany.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
  *
  *   * Redistributions of source code must retain the above copyright
  *     notice, this list of conditions and the following disclaimer.
  *   * Redistributions in binary form must reproduce the above
  *     copyright notice, this list of conditions and the following
  *     disclaimer in the documentation and/or other materials provided
  *     with the distribution.
  *   * Neither the name of the Universitaet Karlsruhe (TH) nor the names
  *     of its contributors may be used to endorse or promote products
  *     derived from this software without specific prior written
  *     permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
  * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
  * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
  * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
  * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 **/


/**
* driver for magnetic buzzer owmb 505025
* MAKE SURE THAT NO INTERRUPTS OCCURR DURING BUZZERS!!!!
*
*
* Author: Albert Krohn krohn@teco.edu
* Date: 2004-07-30 (yyyy-mm-dd)
*
* Current state: ok
*
* version 001: 	2004-11-10 by sabin (wendhack@teco.edu)
*
*
* Date:
*  2005_02_01 Daniel & Raphael
*  new Particle Base System organization
*
*
*/

#include "owmb.h"
#include "coreboard.h"

uint8_t sound_volume;

void BuzzerOff()
{
	output_low(PIN_BUZZER);
}


void BuzzerSetVolume(uint8_t vol) wparam
{
	if (vol>20) vol=20;
	sound_volume=vol;
}

uint8_t BuzzerGetVolume()
{
	return sound_volume;
}


/**
* Initializes the Buzzer
*/
uint8_t BuzzerInit()
{
	bit_clear(TRIS_BUZZER);				// set sound pin to output
	BuzzerOff();
	sound_volume=20;						//max
	return 0;								// indicate no error
}


//frequency in hz, duration in ms, duty in us (the high time)( for high=0=50%duty)
void BuzzerTone(uint16_t frequency,uint16_t duration) wparam
{
	uint16_t i,T,cycles;
#define USE_FLOAT 1
#if defined(USE_FLOAT) && USE_FLOAT > 0
	float f,d,tt;
	
	f=(float) frequency;
	d=(float) duration;
	tt=(1000000/f);
	cycles=(uint16_t)( ((float)duration)*f/1000);
#else
	uint16_t tt;
	uint32_t prod = (uint32_t)duration * (uint32_t)frequency;

	tt = (1000000UL/(uint32_t)frequency);
	cycles = (uint16_t)(prod/1000UL);
#endif
	T=(uint16_t)tt;
	T=T-(uint16_t)sound_volume;


	for(i=0;i<cycles;i++)
	{
		output_high(PIN_BUZZER);
		delay_us(sound_volume);
		output_low(PIN_BUZZER);
		delay_us(T);
	}
	BuzzerOff();		//just to make sure
}


void BuzzerSweepUp()
{
	uint16_t T;

	for (T=2000;T>166;T=T-5)
	{
		output_high(PIN_BUZZER);
		DelayUs(10);
		output_low(PIN_BUZZER);
		DelayUsLong(T);
	}
}


void BuzzerSweepDown()
{
	uint16_t T;

	for (T=166;T<2000;T=T+5)
	{
		output_high(PIN_BUZZER);
		DelayUs(10);
		output_low(PIN_BUZZER);
		DelayUsLong(T);
	}
}


void BuzzerPlayASong(uint8_t song) wparam
{
	switch(song)

	{
		case 1:	// particle startup
				BuzzerTone(TONE_G7,70);
				BuzzerTone(TONE_C7,80);
				DelayMs(20);
				BuzzerTone(TONE_C7,100);
				BuzzerTone(TONE_D7,200);
				DelayMs(100);
				BuzzerTone(TONE_D7,400);
				break;




		case 2:
				//alle meine Entchen, tief
				BuzzerTone(TONE_C6,100);
				BuzzerTone(TONE_D6,100);
				BuzzerTone(TONE_E6,100);
				BuzzerTone(TONE_F6,100);
				BuzzerTone(TONE_G6,150);DelayMs(50);
				BuzzerTone(TONE_G6,150);DelayMs(50);
				BuzzerTone(TONE_A6,50);DelayMs(50);
				BuzzerTone(TONE_A6,50);DelayMs(50);
				BuzzerTone(TONE_A6,50);DelayMs(50);
				BuzzerTone(TONE_A6,50);DelayMs(50);
				BuzzerTone(TONE_G6,400);
				BuzzerTone(TONE_A6,50);DelayMs(50);
				BuzzerTone(TONE_A6,50);DelayMs(50);
				BuzzerTone(TONE_A6,50);DelayMs(50);
				BuzzerTone(TONE_A6,50);DelayMs(50);
				BuzzerTone(TONE_G6,400);
				BuzzerTone(TONE_F6,50);DelayMs(50);
				BuzzerTone(TONE_F6,50);DelayMs(50);
				BuzzerTone(TONE_F6,50);DelayMs(50);
				BuzzerTone(TONE_F6,50);DelayMs(50);
				BuzzerTone(TONE_E6,150);DelayMs(50);
				BuzzerTone(TONE_E6,150);DelayMs(50);
				BuzzerTone(TONE_D6,50);DelayMs(50);
				BuzzerTone(TONE_D6,50);DelayMs(50);
				BuzzerTone(TONE_D6,50);DelayMs(50);
				BuzzerTone(TONE_D6,50);DelayMs(50);
				BuzzerTone(TONE_C6,400);
				break;
		case 3:
				//alle meine Entchen super hoch
				BuzzerTone(TONE_C8,100);
				BuzzerTone(TONE_D8,100);
				BuzzerTone(TONE_E8,100);
				BuzzerTone(TONE_F8,100);
				BuzzerTone(TONE_G8,150);DelayMs(50);
				BuzzerTone(TONE_G8,150);DelayMs(50);
				BuzzerTone(TONE_A8,50);DelayMs(50);
				BuzzerTone(TONE_A8,50);DelayMs(50);
				BuzzerTone(TONE_A8,50);DelayMs(50);
				BuzzerTone(TONE_A8,50);DelayMs(50);
				BuzzerTone(TONE_G8,400);
				BuzzerTone(TONE_A8,50);DelayMs(50);
				BuzzerTone(TONE_A8,50);DelayMs(50);
				BuzzerTone(TONE_A8,50);DelayMs(50);
				BuzzerTone(TONE_A8,50);DelayMs(50);
				BuzzerTone(TONE_G8,400);
				BuzzerTone(TONE_F8,50);DelayMs(50);
				BuzzerTone(TONE_F8,50);DelayMs(50);
				BuzzerTone(TONE_F8,50);DelayMs(50);
				BuzzerTone(TONE_F8,50);DelayMs(50);
				BuzzerTone(TONE_E8,150);DelayMs(50);
				BuzzerTone(TONE_E8,150);DelayMs(50);
				BuzzerTone(TONE_D8,50);DelayMs(50);
				BuzzerTone(TONE_D8,50);DelayMs(50);
				BuzzerTone(TONE_D8,50);DelayMs(50);
				BuzzerTone(TONE_D8,50);DelayMs(50);
				BuzzerTone(TONE_C8,400);
				break;
		case 4:
				//fanfare
				BuzzerTone(TONE_C6,100);
				BuzzerTone(TONE_E6,100);
				BuzzerTone(TONE_G6,100);
				BuzzerTone(TONE_C7,200);
				BuzzerTone(TONE_G6,100);
				BuzzerTone(TONE_C7,500);
				break;
		case 5:
				//Deutschland Nationalhymne
				BuzzerTone(TONE_C5,600);
				BuzzerTone(TONE_D5,200);
				BuzzerTone(TONE_E5,400);
				BuzzerTone(TONE_D5,400);
				BuzzerTone(TONE_F5,400);
				BuzzerTone(TONE_E5,400);
				BuzzerTone(TONE_D5,200);
				BuzzerTone(TONE_H4,200);
				BuzzerTone(TONE_C5,400);

				BuzzerTone(TONE_A5,400);
				BuzzerTone(TONE_G5,400);
				BuzzerTone(TONE_F5,400);
				BuzzerTone(TONE_E5,400);
				BuzzerTone(TONE_D5,400);

				BuzzerTone(TONE_E5,200);
				BuzzerTone(TONE_C5,200);
				BuzzerTone(TONE_G5,800);

				break;
	}
}


