/**
  * Copyright (c) 2004, Telecooperation Office (TecO),
  * Universitaet Karlsruhe (TH), Germany.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
  *
  *   * Redistributions of source code must retain the above copyright
  *     notice, this list of conditions and the following disclaimer.
  *   * Redistributions in binary form must reproduce the above
  *     copyright notice, this list of conditions and the following
  *     disclaimer in the documentation and/or other materials provided
  *     with the distribution.
  *   * Neither the name of the Universitaet Karlsruhe (TH) nor the names
  *     of its contributors may be used to endorse or promote products
  *     derived from this software without specific prior written
  *     permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
  * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
  * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
  * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
  * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 **/


/**
* driver for PCEeprom
*
*
* Author: Albert Krohn  (krohn@teco.edu)
* Date: 2003-07-06 (yyyy-mm-dd)
*
* Current state: template
*
* Date:
*  2005_02_01 Daniel & Raphael
*  new Particle Base System organization
*
*/


#include "pceeprom.h"
#include "coreboard.h"
//I2C Device!... needs the PCi2c driver
#include "pci2c.h"

/**
* Switches the PCEeprom on
*/
void PCEepromInit()
{
	PCI2CInit();
}

void PCEepromOn()
{
	PCI2COn();
}

void PCEepromOff()
{
	PCI2COff();
}

uint8_t PCEepromPrepare()
{
	PCEepromOn();
}

//returns the state of the PCEeprom: 0:not ready (e.g. writing), 1: ready
uint8_t PCEepromReady()
{

	PCI2CStart();
	if (!PCI2CWrite(0xa0+XROM_ID)) {PCI2CStop();return 1;}
	return 0;
}

//returns the number of bytes that fit into page, counted from addr
uint8_t PCEepromBytesLeftInPage(uint16_t addr) wparam
{
	uint16_t current_page;
	current_page=addr&0xFFC0;
	return (64-(uint8_t)(addr-current_page));
}

//returns 0 if successful
//1-byte modus, needs 5 ms wait time till next write process starts
uint8_t PCEepromWriteByte(uint16_t addr,uint8_t __PIC_data) wparam
{
	PCI2CStart();
	if (PCI2CWrite(0xa0+XROM_ID)) {PCI2CStop();return 1;}
	if (PCI2CWrite(addr>>8)) {PCI2CStop();return 2;}
	if (PCI2CWrite(addr&0xFF))  {PCI2CStop();return 2;}
	if (PCI2CWrite(__PIC_data))  {PCI2CStop();return 3;}
	PCI2CStop();
	return 0;
}

//reads one single byte from addr, returns 0 if successful
uint8_t PCEepromReadByte(uint16_t addr, uint8_t *dataout) wparam
{
	PCI2CStart();
	if (PCI2CWrite(0xa0+XROM_ID)) {PCI2CStop();return 1;}
	if (PCI2CWrite(addr>>8)) {PCI2CStop();return 2;}
	if (PCI2CWrite(addr&0xFF))  {PCI2CStop();return 2;}
	PCI2CStart();
	if (PCI2CWrite(0xa1+XROM_ID)) {PCI2CStop(); return 3;}
	*dataout=PCI2CRead(0);
	PCI2CStop();

	return 0;
}


//returns 0 if successful
//sequential write modus, will only write,if data fits into current page.
uint8_t PCEepromWriteSequence(uint16_t addr, uint8_t* buff, uint16_t count) wparam
{
	uint16_t i;
	i=0;
	if(PCEepromBytesLeftInPage(addr)<count) return 4;		//data does not fit into page
	PCI2CStart();
	if (PCI2CWrite(0xa0+XROM_ID)) {PCI2CStop();return 1;}
	if (PCI2CWrite(addr>>8)) {PCI2CStop();return 2;}
	if (PCI2CWrite(addr&0xFF))  {PCI2CStop();return 2;}
	while(i<count)
	{
		if (PCI2CWrite(buff[i]))  {PCI2CStop();return 3;}
		i++;
	}
	PCI2CStop();
	return 0;
}


//returns 0 if successful
uint8_t PCEepromReadSequence(uint16_t addr,uint8_t* buff,uint16_t count) wparam
{
	uint16_t i;
	i=0;
	PCI2CStart();
	if (PCI2CWrite(0xa0+XROM_ID)) {PCI2CStop(); return 1;}
	if (PCI2CWrite(addr>>8)) {PCI2CStop(); return 2;}
	if (PCI2CWrite(addr&0xFF)) {PCI2CStop(); return 2;}
	PCI2CStart();
	if (PCI2CWrite(0xa1+XROM_ID)) {PCI2CStop(); return 3;}
	for(i=0;i<count-1;i++)	buff[i]=PCI2CRead(1);
	buff[count-1]=PCI2CRead(0);
	PCI2CStop();
	return 0;
}


//return 0 if succesful, returns 1 if value is false, returns 2,3 or 4 if
//address isn't acknowledged
uint8_t PCEepromTest()
{
	uint8_t __PIC_data;

	PCI2CStart();
	if (PCI2CWrite(0xa0+XROM_ID))
		{PCI2CStop(); return 2;}
	PCI2CWrite(0x79);	//address is 31110
	PCI2CWrite(0x86);

	PCI2CWrite(44);
	PCI2CStop();
	DelayMs(5);	//wait during PCEeprom write cycle

	PCI2CStart();
	if (PCI2CWrite(0xa0+XROM_ID))
		{PCI2CStop(); return 3;}
	PCI2CWrite(0x79);
	PCI2CWrite(0x86);
	PCI2CStart();
	if (PCI2CWrite(0xa1+XROM_ID))
		{PCI2CStop(); return 4;}
	__PIC_data=PCI2CRead(0);
	PCI2CStop();
	if (__PIC_data==44) return 0;
	else return 1;
}



