/**
  * Copyright (c) 2004, Telecooperation Office (TecO),
  * Universitaet Karlsruhe (TH), Germany.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
  *
  *   * Redistributions of source code must retain the above copyright
  *     notice, this list of conditions and the following disclaimer.
  *   * Redistributions in binary form must reproduce the above
  *     copyright notice, this list of conditions and the following
  *     disclaimer in the documentation and/or other materials provided
  *     with the distribution.
  *   * Neither the name of the Universitaet Karlsruhe (TH) nor the names
  *     of its contributors may be used to endorse or promote products
  *     derived from this software without specific prior written
  *     permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
  * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
  * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
  * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
  * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 **/


#ifndef __PCEEPROM_H__
#define __PCEEPROM_H__
/**
* driver for PCEeprom
*
*
* Author: Albert Krohn  (krohn@teco.edu)
* Date: 2003-07-06 (yyyy-mm-dd)
*
* Current state: template
*
* Date:
*  2005_02_01 Daniel & Raphael
*  new Particle Base System organization
*
*/


//I2C Device!... needs the PCi2c driver

#define PCEeprom_DRIVER	001




// EEprom adresses

#define EEPROM_EXT_ADDRESS_ID_HI    121		//start address for ID in external EEPROM:31112
#define EEPROM_EXT_ADDRESS_ID_LO	136		// from there on until 32767 is free (another 1655 byte)
#define EEPROM_EXT_ADDRESS_ID		31112	//same as uint16_t


#include <tools.h>

/**
* Switches the PCEeprom on
*/
void PCEepromInit();

void PCEepromOn();

void PCEepromOff();

uint8_t PCEepromPrepare();

//returns the state of the PCEeprom: 0:not ready (e.g. writing), 1: ready
uint8_t PCEepromReady();

//returns the number of bytes that fit into page, counted from addr
uint8_t PCEepromBytesLeftInPage(uint16_t addr) wparam;

//returns 0 if successful
//1-byte modus, needs 5 ms wait time till next write process starts
uint8_t PCEepromWriteByte(uint16_t addr,uint8_t __PIC_data) wparam;

//reads one single byte from addr, returns 0 if successful
uint8_t PCEepromReadByte(uint16_t addr, uint8_t *dataout) wparam;


//returns 0 if successful
//sequential write modus, will only write,if data fits into current page.
uint8_t PCEepromWriteSequence(uint16_t addr, uint8_t* buff, uint16_t count) wparam;


//returns 0 if successful
uint8_t PCEepromReadSequence(uint16_t addr,uint8_t* buff,uint16_t count) wparam;

//return 0 if succesful, returns 1 if value is false, returns 2,3 or 4 if
//address isn't acknowledged
uint8_t PCEepromTest();

#endif // __PCEEPROM_H__
