/**
  * Copyright (c) 2004, Telecooperation Office (TecO),
  * Universitaet Karlsruhe (TH), Germany.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
  *
  *   * Redistributions of source code must retain the above copyright
  *     notice, this list of conditions and the following disclaimer.
  *   * Redistributions in binary form must reproduce the above
  *     copyright notice, this list of conditions and the following
  *     disclaimer in the documentation and/or other materials provided
  *     with the distribution.
  *   * Neither the name of the Universitaet Karlsruhe (TH) nor the names
  *     of its contributors may be used to endorse or promote products
  *     derived from this software without specific prior written
  *     permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
  * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
  * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
  * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
  * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 **/

/**
* board file for the simple sensor board 192
*
* Author: krohn@teco.edu
* Date: 2003-08-21
*
* changed by cdecker@teco.edu
* Date: 2003-11-25
*
* version: 001 (2003-11-27, cdecker)
*   - first defs
*
* version: 002 (2003-11-28, cdecker)
*   - defs are re-newed for architecture reasons
*
* version: 003 (2003-12-01, cdecker)
*   - basic functions for ssimp 192
*   - no defs anymore
*
* version: 004 (2003-12-02, cdecker)
*   - complete initialization
*
* Date:
*  2005_02_01 Daniel & Raphael
*  new Particle Base System organization
*
*/


#include "ssimp192.h"
#include "coreboard.h"

void SSimpInit()
{

	// number of analog pins on the connector [0..4]
	// 0: CONN_11..CONN_14 are digital
	// 1: CONN_11 is analog, CONN_12..CONN_14 are digital
	// 2: CONN_11..CONN_12 are analog, CONN_13..CONN_14 are digital
	// 3: CONN_11..CONN_13 are analog, CONN_14 is digital
	// 4: CONN_11..CONN_14 are analog

	// set analog pins
	PCSetAnalogPorts(ANALOG_CONN_11_12_14);

	// set digital pins

	// comment in to test the memory consumption of the gigantic switch case function #-)
	// preliminary result: 8% more memory consumption
	//PCSetDigitalPin(PIN_ACCL_ADXL_POWER, high);

	bit_clear(TRIS_ACCL_ADXL_POWER);
	output_high(PIN_ACCL_ADXL_POWER);
	bit_set(TRIS_ACCL_ADXL_XOUT);
	bit_set(TRIS_ACCL_ADXL_YOUT);

	bit_clear(TRIS_ACCL_ADXL_POWER_Z);
	output_high(PIN_ACCL_ADXL_POWER_Z);
	bit_set(TRIS_ACCL_ADXL_ZOUT);

	bit_clear(TRIS_LIGHT_TSL250_POWER);
	output_high(PIN_LIGHT_TSL250_POWER);

	bit_clear(TRIS_LIGHT_TSL2550_POWER);
	output_high(PIN_LIGHT_TSL2550_POWER);

	bit_set(TRIS_LIGHT_TSL2550_SDA);
	bit_set(TRIS_LIGHT_TSL2550_SCLK);

	bit_clear(TRIS_TEMP_TC74_POWER);
	output_high(PIN_TEMP_TC74_POWER);

	bit_set(TRIS_TEMP_TC74_SDA);
	bit_set(TRIS_TEMP_TC74_SCLK);

	bit_clear(TRIS_FORCE_FSR_POWER);
	output_high(PIN_FORCE_FSR_POWER);

	bit_set(TRIS_FORCE_FSR_OUT);

	bit_clear(TRIS_AUDIO_MIC_POWER);
	output_high(PIN_AUDIO_MIC_POWER);

	bit_set(TRIS_AUDIO_MIC_OUT);

	bit_clear(TRIS_LED_AMBER_POWER);
	output_high(PIN_LED_AMBER_POWER);


	bit_clear(TRIS_SENSOR_I2C_POWER);
	output_high(PIN_SENSOR_I2C_POWER);

	bit_set(TRIS_SENSOR_I2C_SDA);
	bit_set(TRIS_SENSOR_I2C_SCL);


}

void SSimpLEDAmberOn()
{
	output_high(PIN_LED_AMBER_POWER);
}


void SSimpLEDAmberOff()
{
	output_low(PIN_LED_AMBER_POWER);
}

void SSimpSensorsInit()
{

	#ifdef LIGHT_SENSOR
		LightSensorInit();		//inits the light and sets the tris etc..
	#endif

	#ifdef AUDIO_SENSOR
		AudioSensorInit();		//inits the light and sets the tris etc..
	#endif

	#ifdef FORCE_SENSOR
		ForceSensorInit();		//inits the light and sets the tris etc..
	#endif

	#ifdef ACCL_SENSOR
		AcclSensorInit();		//inits the light and sets the tris etc..
	#endif

	#ifdef ACCL_Z_SENSOR
		AcclZSensorInit();		//inits the light and sets the tris etc..
	#endif

	#ifdef TEMPERATURE_SENSOR
		TemperatureSensorInit();		//inits the light and sets the tris etc..
	#endif

	#ifdef AMBIENT_LIGHT_SENSOR
		AmbientLightSensorInit();
	#endif

}


void SSimpSensorsOn()
{


	#ifdef LIGHT_SENSOR
		LightSensorOn();
	#endif

	#ifdef AUDIO_SENSOR
		AudioSensorOn();
	#endif

	#ifdef FORCE_SENSOR
		ForceSensorOn();
	#endif

	#ifdef ACCL_SENSOR
		AcclSensorOn();
	#endif

	#ifdef ACCL_Z_SENSOR
		AcclZSensorOn();
	#endif

	#ifdef TEMPERATURE_SENSOR
		TemperatureSensorOn();
	#endif

	#ifdef AMBIENT_LIGHT_SENSOR
		AmbientLightSensorOn();
	#endif
}

