/**
  * Copyright (c) 2004, Telecooperation Office (TecO),
  * Universitaet Karlsruhe (TH), Germany.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
  *
  *   * Redistributions of source code must retain the above copyright
  *     notice, this list of conditions and the following disclaimer.
  *   * Redistributions in binary form must reproduce the above
  *     copyright notice, this list of conditions and the following
  *     disclaimer in the documentation and/or other materials provided
  *     with the distribution.
  *   * Neither the name of the Universitaet Karlsruhe (TH) nor the names
  *     of its contributors may be used to endorse or promote products
  *     derived from this software without specific prior written
  *     permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
  * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
  * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
  * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
  * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 **/


/**
* header file for the simple sensor board 192
*
* Author: krohn@teco.edu
* Date: 2003-08-21
*
* changed by cdecker@teco.edu
* Date: 2003-11-25
*
* version: 001 (2003-11-27, cdecker)
*   - sensor pin defs
*
*
* Date:
*  2005_02_01 Daniel & Raphael
*  new Particle Base System organization
*
*/


#ifndef __SSIMP192_H__
#define	__SSIMP192_H__					001


/*
 * The sensordrivers used for this board can be found at the end of
 * this file.
 */


// pin defs for sensors

// accelerator ADXL210
#define PIN_ACCL_ADXL_POWER				PIN_CONN_04
#define PIN_ACCL_ADXL_XOUT				PIN_CONN_05
#define PIN_ACCL_ADXL_YOUT				PIN_CONN_06

#define TRIS_ACCL_ADXL_POWER			TRIS_CONN_04
#define TRIS_ACCL_ADXL_XOUT				TRIS_CONN_05
#define TRIS_ACCL_ADXL_YOUT				TRIS_CONN_06

//extra accl
#define PIN_ACCL_ADXL_POWER_Z			PIN_CONN_04
#define PIN_ACCL_ADXL_ZOUT				PIN_CONN_07

#define TRIS_ACCL_ADXL_POWER_Z			TRIS_CONN_04
#define TRIS_ACCL_ADXL_ZOUT				TRIS_CONN_07


// light sensor TSL250
#define PIN_LIGHT_TSL250_POWER			PIN_CONN_08
#define PIN_LIGHT_TSL250_OUT			PIN_CONN_14

#define TRIS_LIGHT_TSL250_POWER			TRIS_CONN_08
#define TRIS_LIGHT_TSL250_OUT			TRIS_CONN_14

#define ADCH_LIGHT_TSL250				ADCH_CONN_14


// light sensor TSL2550
#define PIN_LIGHT_TSL2550_POWER			PIN_CONN_04
#define PIN_LIGHT_TSL2550_SDA			PIN_CONN_13
#define PIN_LIGHT_TSL2550_SCLK			PIN_CONN_09

#define TRIS_LIGHT_TSL2550_POWER		TRIS_CONN_04
#define TRIS_LIGHT_TSL2550_SDA			TRIS_CONN_13
#define TRIS_LIGHT_TSL2550_SCLK			TRIS_CONN_09


// temperature sensor TC74
#define PIN_TEMP_TC74_POWER				PIN_CONN_04
#define PIN_TEMP_TC74_SDA				PIN_CONN_13
#define PIN_TEMP_TC74_SCLK				PIN_CONN_09

#define TRIS_TEMP_TC74_POWER			TRIS_CONN_04
#define TRIS_TEMP_TC74_SDA				TRIS_CONN_13
#define TRIS_TEMP_TC74_SCLK				TRIS_CONN_09


// force sensor
#define PIN_FORCE_FSR_POWER				PIN_CONN_17
#define PIN_FORCE_FSR_OUT				PIN_CONN_12

#define TRIS_FORCE_FSR_POWER			TRIS_CONN_17
#define TRIS_FORCE_FSR_OUT				TRIS_CONN_12

#define ADCH_FORCE_FSR					ADCH_CONN_12	//channel AN2


// audio
#define PIN_AUDIO_MIC_POWER				PIN_CONN_17
#define PIN_AUDIO_MIC_OUT				PIN_CONN_11

#define TRIS_AUDIO_MIC_POWER			TRIS_CONN_17
#define TRIS_AUDIO_MIC_OUT				TRIS_CONN_11

#define ADCH_AUDIO_MIC					ADCH_CONN_11


// generic sensor
#define PIN_GENERIC_POWER				PIN_CONN_08
#define PIN_GENERIC_OUT					PIN_CONN_14

#define TRIS_GENERIC_POWER				TRIS_CONN_08
#define TRIS_GENERIC_OUT				TRIS_CONN_14

#define ADCH_GENERIC_OUT				ADCH_CONN_14



// LED
#define PIN_LED_AMBER_POWER				PIN_CONN_03
#define TRIS_LED_AMBER_POWER			TRIS_CONN_03


// i2c sensors:


//pc i2c for eeprom etc
#define PIN_SENSOR_I2C_SDA          	PIN_CONN_13
#define TRIS_SENSOR_I2C_SDA         	TRIS_CONN_13

#define PIN_SENSOR_I2C_SCL          	PIN_CONN_09
#define TRIS_SENSOR_I2C_SCL         	TRIS_CONN_09

#define PIN_SENSOR_I2C_POWER        	PIN_CONN_04
#define TRIS_SENSOR_I2C_POWER       	TRIS_CONN_04




/**
* Initializes the SSimp
*/
void SSimpInit();

/**
* Switches the amber LED on
*/
void SSimpLEDAmberOn();

/**
* Switches the amber LED off
*/
void SSimpLEDAmberOff();

/**
* Initializes all sensors on SSimp
*/
void SSimpSensorsInit();

/**
* Turn on all sensors on SSimp
*/
void SSimpSensorsOn();


//+++++ include the sensors your want to use ++++++++++++++++++++++++++++++++++
#include "../sensors/fsr.h"
#include "../sensors/sp0101nc1.h"
#include "../sensors/adxl210_xy.h"
#include "../sensors/adxl210_z.h"
#include "../sensors/tc74.h"
#include "../sensors/tsl2550.h"
#include "../sensors/voltage.h"


#endif // __SSIMP192_H__
