/**
  * Copyright (c) 2004, Telecooperation Office (TecO),
  * Universitaet Karlsruhe (TH), Germany.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
  *
  *   * Redistributions of source code must retain the above copyright
  *     notice, this list of conditions and the following disclaimer.
  *   * Redistributions in binary form must reproduce the above
  *     copyright notice, this list of conditions and the following
  *     disclaimer in the documentation and/or other materials provided
  *     with the distribution.
  *   * Neither the name of the Universitaet Karlsruhe (TH) nor the names
  *     of its contributors may be used to endorse or promote products
  *     derived from this software without specific prior written
  *     permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
  * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
  * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
  * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
  * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 **/


#ifndef __DISPTAB_C__
#define __DISPTAB_C__

#include <tools.h>
#include <initprio.c>

/***********************************************
 * Dispatch table for low priority interrupts. *
 ***********************************************/
void __irqhandler_low () interrupt 2 {
  disable_interrupts(INT_LOWPRIO);

#if defined (ad_handler) && !defined (ad_fast)
  if (PIR1bits.ADIF) {
    ad_handler();
#ifndef ad_noclear
    PIR1bits.ADIF = 0;
#endif
  }
#endif

#if defined (buscol_handler) && !defined (buscol_fast)
  if (PIR2bits.BCLIF) {
    buscol_handler();
#ifndef buscol_noclear
    PIR2bits.BCLIF = 0;
#endif
  }
#endif

#if defined (ccp1_handler) && !defined (ccp1_fast)
  if (PIR1bits.CCP1IF) {
    ccp1_handler();
#ifndef ccp1_noclear
    PIR1bits.CCP1IF = 0;
#endif
  }
#endif

#if defined (ccp2_handler) && !defined (ccp2_fast)
  if (PIR2bits.CCP2IF) {
    ccp2_handler();
#ifndef ccp2_noclear
    PIR2bits.CCP2IF = 0;
#endif
  }
#endif

#if defined (comp_handler) && !defined (comp_fast)
  if (PIR2bits.CMIF) {
    comp_handler();
#ifndef comp_noclear
    PIR2bits.CMIF = 0;
#endif
  }
#endif

#if defined (eeprom_handler) && !defined (eeprom_fast)
  if (PIR2bits.EEIF) {
    eeprom_handler();
#ifndef eeprom_noclear
    PIR2bits.EEIF = 0;
#endif
  }
#endif

#if defined (ext_handler) && !defined (ext_fast)
  if (INTCONbits.INT0F) {
    ext_handler();
#ifndef ext_noclear
    INTCONbits.INT0F = 0;
#endif
  }
#endif

#if defined (ext1_handler) && !defined (ext1_fast)
  if (INTCON3bits.INT1F) {
    ext1_handler();
#ifndef ext1_noclear
    INTCON3bits.INT1F = 0;
#endif
  }
#endif

#if defined (ext2_handler) && !defined (ext2_fast)
  if (INTCON3bits.INT2F) {
    ext2_handler();
#ifndef ext2_noclear
    INTCON3bits.INT2F = 0;
#endif
  }
#endif

#if defined (ext3_handler) && !defined (ext3_fast)
  if (INTCON3bits.INT3F) {
    ext3_handler();
#ifndef ext3_noclear
    INTCON3bits.INT3F = 0;
#endif
  }
#endif

#if defined (lowvolt_handler) && !defined (lowvolt_fast)
  if (PIR2bits.LVDIF) {
    lowvolt_handler();
#ifndef lowvolt_noclear
    PIR2bits.LVDIF = 0;
#endif
  }
#endif

#if defined (psp_handler) && !defined (psp_fast)
  if (PIR1bits.PSPIF) {
    psp_handler();
#ifndef psp_noclear
    PIR1bits.PSPIF = 0;
#endif
  }
#endif

#if defined (rb_handler) && !defined (rb_fast)
  if (INTCONbits.RBIF) {
    rb_handler();
#ifndef rb_noclear
    INTCONbits.RBIF = 0;
#endif
  }
#endif

#if defined (rc_handler) && !defined (rc_fast)
  if (PIR1bits.RCIF) {
    rc_handler();
#ifndef rc_noclear
    PIR1bits.RCIF = 0;
#endif
  }
#endif

#if defined (rda_handler) && !defined (rda_fast)
  if (PIR1bits.RCIF) {
    rda_handler();
#ifndef rda_noclear
    PIR1bits.RCIF = 0;
#endif
  }
#endif

#if defined (rtcc_handler) && !defined (rtcc_fast)
  if (INTCONbits.T0IF) {
    rtcc_handler();
#ifndef rtcc_noclear
    INTCONbits.T0IF = 0;
#endif
  }
#endif

#if defined (ssp_handler) && !defined (ssp_fast)
  if (PIR1bits.SSPIF) {
    ssp_handler();
#ifndef ssp_noclear
    PIR1bits.SSPIF = 0;
#endif
  }
#endif

#if defined (tbe_handler) && !defined (tbe_fast)
  if (PIR1bits.TXIF) {
    tbe_handler();
#ifndef tbe_noclear
    PIR1bits.TXIF = 0;
#endif
  }
#endif

#if defined (timer0_handler) && !defined (timer0_fast)
  if (INTCONbits.T0IF) {
    timer0_handler();
#ifndef timer0_noclear
    INTCONbits.T0IF = 0;
#endif
  }
#endif

#if defined (timer1_handler) && !defined (timer1_fast)
  if (PIR1bits.TMR1IF) {
    timer1_handler();
#ifndef timer1_noclear
    PIR1bits.TMR1IF = 0;
#endif
  }
#endif

#if defined (timer2_handler) && !defined (timer2_fast)
  if (PIR1bits.TMR2IF) {
    timer2_handler();
#ifndef timer2_noclear
    PIR1bits.TMR2IF = 0;
#endif
  }
#endif

#if defined (timer3_handler) && !defined (timer3_fast)
  if (PIR2bits.TMR3IF) {
    timer3_handler();
#ifndef timer3_noclear
    PIR2bits.TMR3IF = 0;
#endif
  }
#endif

#if defined (timer4_handler) && !defined (timer4_fast)
  if (PIR3bits.TMR4IF) {
    timer4_handler();
#ifndef timer4_noclear
    PIR3bits.TMR4IF = 0;
#endif
  }
#endif

  enable_interrupts(INT_LOWPRIO);

}

/************************************************
 * Dispatch table for high priority interrupts. *
 ************************************************/
void __irqhandler_high () interrupt 1 {
  disable_interrupts(GLOBAL);

#if defined (ad_handler) && defined (ad_fast)
  if (PIR1bits.ADIF) {
    ad_handler();
#ifndef ad_noclear
    PIR1bits.ADIF = 0;
#endif
  }
#endif

#if defined (buscol_handler) && defined (buscol_fast)
  if (PIR2bits.BCLIF) {
    buscol_handler();
#ifndef buscol_noclear
    PIR2bits.BCLIF = 0;
#endif
  }
#endif

#if defined (ccp1_handler) && defined (ccp1_fast)
  if (PIR1bits.CCP1IF) {
    ccp1_handler();
#ifndef ccp1_noclear
    PIR1bits.CCP1IF = 0;
#endif
  }
#endif

#if defined (ccp2_handler) && defined (ccp2_fast)
  if (PIR2bits.CCP2IF) {
    ccp2_handler();
#ifndef ccp2_noclear
    PIR2bits.CCP2IF = 0;
#endif
  }
#endif

#if defined (comp_handler) && defined (comp_fast)
  if (PIR2bits.CMIF) {
    comp_handler();
#ifndef comp_noclear
    PIR2bits.CMIF = 0;
#endif
  }
#endif

#if defined (eeprom_handler) && defined (eeprom_fast)
  if (PIR2bits.EEIF) {
    eeprom_handler();
#ifndef eeprom_noclear
    PIR2bits.EEIF = 0;
#endif
  }
#endif

#if defined (ext_handler) && defined (ext_fast)
  if (INTCONbits.INT0F) {
    ext_handler();
#ifndef ext_noclear
    INTCONbits.INT0F = 0;
#endif
  }
#endif

#if defined (ext1_handler) && defined (ext1_fast)
  if (INTCON3bits.INT1F) {
    ext1_handler();
#ifndef ext1_noclear
    INTCON3bits.INT1F = 0;
#endif
  }
#endif

#if defined (ext2_handler) && defined (ext2_fast)
  if (INTCON3bits.INT2F) {
    ext2_handler();
#ifndef ext2_noclear
    INTCON3bits.INT2F = 0;
#endif
  }
#endif

#if defined (ext3_handler) && defined (ext3_fast)
  if (INTCON3bits.INT3F) {
    ext3_handler();
#ifndef ext3_noclear
    INTCON3bits.INT3F = 0;
#endif
  }
#endif

#if defined (lowvolt_handler) && defined (lowvolt_fast)
  if (PIR2bits.LVDIF) {
    lowvolt_handler();
#ifndef lowvolt_noclear
    PIR2bits.LVDIF = 0;
#endif
  }
#endif

#if defined (psp_handler) && defined (psp_fast)
  if (PIR1bits.PSPIF) {
    psp_handler();
#ifndef psp_noclear
    PIR1bits.PSPIF = 0;
#endif
  }
#endif

#if defined (rb_handler) && defined (rb_fast)
  if (INTCONbits.RBIF) {
    rb_handler();
#ifndef rb_noclear
    INTCONbits.RBIF = 0;
#endif
  }
#endif

#if defined (rc_handler) && defined (rc_fast)
  if (PIR1bits.RCIF) {
    rc_handler();
#ifndef rc_noclear
    PIR1bits.RCIF = 0;
#endif
  }
#endif

#if defined (rda_handler) && defined (rda_fast)
  if (PIR1bits.RCIF) {
    rda_handler();
#ifndef rda_noclear
    PIR1bits.RCIF = 0;
#endif
  }
#endif

#if defined (rtcc_handler) && defined (rtcc_fast)
  if (INTCONbits.T0IF) {
    rtcc_handler();
#ifndef rtcc_noclear
    INTCONbits.T0IF = 0;
#endif
  }
#endif

#if defined (ssp_handler) && defined (ssp_fast)
  if (PIR1bits.SSPIF) {
    ssp_handler();
#ifndef ssp_noclear
    PIR1bits.SSPIF = 0;
#endif
  }
#endif

#if defined (tbe_handler) && defined (tbe_fast)
  if (PIR1bits.TXIF) {
    tbe_handler();
#ifndef tbe_noclear
    PIR1bits.TXIF = 0;
#endif
  }
#endif

#if defined (timer0_handler) && defined (timer0_fast)
  if (INTCONbits.T0IF) {
    timer0_handler();
#ifndef timer0_noclear
    INTCONbits.T0IF = 0;
#endif
  }
#endif

#if defined (timer1_handler) && defined (timer1_fast)
  if (PIR1bits.TMR1IF) {
    timer1_handler();
#ifndef timer1_noclear
    PIR1bits.TMR1IF = 0;
#endif
  }
#endif

#if defined (timer2_handler) && defined (timer2_fast)
  if (PIR1bits.TMR2IF) {
    timer2_handler();
#ifndef timer2_noclear
    PIR1bits.TMR2IF = 0;
#endif
  }
#endif

#if defined (timer3_handler) && defined (timer3_fast)
  if (PIR2bits.TMR3IF) {
    timer3_handler();
#ifndef timer3_noclear
    PIR2bits.TMR3IF = 0;
#endif
  }
#endif

#if defined (timer4_handler) && defined (timer4_fast)
  if (PIR3bits.TMR4IF) {
    timer4_handler();
#ifndef timer4_noclear
    PIR3bits.TMR4IF = 0;
#endif
  }
#endif

  enable_interrupts (GLOBAL);
}

#else
  #warning Interrupt dispatch table has been included more than once!
#endif // __DISPTAB_C__
