/**
  * Copyright (c) 2004, Telecooperation Office (TecO),
  * Universitaet Karlsruhe (TH), Germany.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
  *
  *   * Redistributions of source code must retain the above copyright
  *     notice, this list of conditions and the following disclaimer.
  *   * Redistributions in binary form must reproduce the above
  *     copyright notice, this list of conditions and the following
  *     disclaimer in the documentation and/or other materials provided
  *     with the distribution.
  *   * Neither the name of the Universitaet Karlsruhe (TH) nor the names
  *     of its contributors may be used to endorse or promote products
  *     derived from this software without specific prior written
  *     permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
  * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
  * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
  * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
  * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 **/

/*
*
* Date:
*  2005_02_01 Daniel & Raphael
*  CCS compatibility functions
*
*/

#ifndef __TOOLS_H__
#define __TOOLS_H__

#include <pic18fregs.h>
#include <stdint.h>
#include <stddef.h>
#include <math.h>
#include <float.h>
#include <malloc.h>
#include <ctype.h>
#include <string.h>

/* needed in sdcc_defs.h */
#define  _concat2(a,...)	a##__VA_ARGS__
#define  _concat(a,...)		_concat2(a,__VA_ARGS__)
#define  PORT2(...)		_concat(PORT,__VA_ARGS__)
#define  PORT(...)		PORT2(PIN(__VA_ARGS__))
#define  TRIS2(...)		_concat(TRIS,__VA_ARGS__)
#define  TRIS(...)		TRIS2(PIN(__VA_ARGS__))
#define  LAT2(...)		_concat(LAT,__VA_ARGS__)
#define  LAT(...)		LAT2(PIN(__VA_ARGS__))

#include "sdcc_defs.h"
#include "tools_defs.h"

#define ALL_PINS	0xFF


/***********************
 * P R O T O T Y P E N *
 ***********************/

// for now...
#ifndef OWN_IRQ_HANDLERS
#define OWN_IRQ_HANDLERS
void __irqhandler_high () interrupt 1;
void __irqhandler_low () interrupt 2;
void init_irq_priorities();
#endif

#define expand_makro(x)         x
#define ASM2(...)    _##__VA_ARGS__
#define ASM(...)     ASM2(__VA_ARGS__)

/* 
 * I/O related helpers
 */
#define set_pin_to_output(...)			bit_clear(TRIS(__VA_ARGS__))

#define portmode_fast(tris, input, pin)		0/* nop */
#define portmode_fixed(tris, input, pin)	tris = FIXEDMASK_TRIS##tris
#define portmode_standard(tris, input, pin)	TRIS##tris = (pin == ALL_PINS) ? input*0xFF : TRIS##tris, TRIS##tris##bits.TRIS##tris##pin = input
#define set_mode_A(input, pin)			portmode_standard(A, input, pin)
#define set_mode_B(input, pin)			portmode_standard(B, input, pin)
#define set_mode_C(input, pin)			portmode_standard(C, input, pin)
#define set_mode_D(input, pin)			portmode_standard(D, input, pin)
#define set_mode_E(input, pin)			portmode_standard(E, input, pin)
#define set_mode_F(input, pin)			portmode_standard(F, input, pin)
#define set_mode_G(input, pin)			portmode_standard(G, input, pin)
#define set_mode_H(input, pin)			portmode_standard(H, input, pin)
#define set_mode_J(input, pin)			portmode_standard(J, input, pin)

#define set_mode_for2(letter, input, pin)	set_mode_##letter (input, pin)
#define set_mode_for(letter, input, pin)	set_mode_for2(letter, input, pin)

/* 
 * some routines that CCS offers
 */

#define abs(x)					(((x) > 0) ? (x) : (-(x)))
#define acos(x)					acosf(x)
#define asin(x)					asinf(x)
#define atan(x)					atanf(x)
#define atan2(x,y)				atan2f(x,y)

#define bit_clear(...)				bit_clear2(__VA_ARGS__)
#define bit_set(...)				bit_set2(__VA_ARGS__)
#define bit_test(...)				bit_test2(__VA_ARGS__)
// the argument "bit" must be a "number" according to CCS manual; we require a literal
// or produce really inefficient code...
#define bit_set2(var,bit)			((var)|=(1U << (bit)))
#define bit_clear2(var,bit)			do {if (sizeof (var) <= 1) { ((var)&=0xFF ^ (1U << (bit))); } else	\
                                        	if (sizeof (var) <= 2) { ((var)&=0xFFFF ^ (1U << (bit))); } else	\
                                        	{ ((var)&=0xFFFFFFFF ^ (1UL << (bit))); }} while(0)

#define bit_test_byte(var,bit)			((unsigned char)(((unsigned char)(var)) & ((unsigned char)(1U << (bit)))))

#if 0
#define bit_test2(var,bit)			(((bit) < 8) ? bit_test_byte(var,bit)					\
                                                  : ((bit) < 16) ? bit_test_byte(((unsigned int)(var))>>8, (bit)-8)	\
                                                  : ((bit) < 24) ? bit_test_byte(((unsigned int)(var))>>16, (bit)-16)	\
                                                  : bit_test_byte(((unsigned int)(var))>>24, (bit)-24))
#else
#define bit_test2(var,bit)			((sizeof (var) == 1) ? bit_test_byte(var,bit)				\
                                                  : (sizeof(var) <= 2) ? (((uint16_t)var) & (uint16_t)(1U << (bit)))	\
                                                  : (var & (1UL << (bit))))
#endif

#define ceil(x)					ceilf(x)
#define cos(x)					cosf(x)
#define cosh(x)					coshf(x)

#define delay_cycles_rem(x)			do {if (x>10) _asm nop _endasm;				\
						if (x>9) _asm nop _endasm; if (x>8) _asm nop _endasm;	\
						if (x>7) _asm nop _endasm; if (x>6) _asm nop _endasm;	\
						if (x>5) _asm nop _endasm; if (x>4) _asm nop _endasm;	\
						if (x>3) _asm nop _endasm; if (x>2) _asm nop _endasm;	\
						if (x>1) _asm nop _endasm; if (x>0) _asm nop _endasm;} while(0)
#define delay_cycles(x)				if (x < 12) { delay_cycles_rem(x); } \
						else { delay_cycles_loop((x / 3) - 3); delay_cycles_rem((x % 3) + 1); }
void delay_cycles_loop(unsigned char i) _naked wparam;

#define delay_us_short(x)			delay_cycles((CLOCK_SPEED/4000000)*x)
#define disable_interrupts(level)		INT(level) = 0
int div_compute(char num, char denom) wparam;
#define div(num, denom)				((div_t)div_compute((num), (denom)))

#define enable_interrupts(level)		INT(level) = 1
#define erase_program_eeprom(address) do { TBLPTRU = ((address) >> 16); \
   TBLPTRH = (((address) & 0xFF00) >> 8); TBLPTRL = ((address) & 0xFF); \
 _asm bsf _EECON1, 7 _endasm; _asm bcf _EECON1, 6 _endasm; _asm bsf _EECON1, 2 _endasm; \
 _asm bsf _EECON1, 4 _endasm; disable_interrupts(GLOBAL); EECON2 = 0x55; EECON2 = 0xAA; \
 _asm bsf _EECON1, 1 _endasm; _asm nop _endasm; enable_interrupts(GLOBAL); } while(0)
#define exp(x)					expf(x)
#define ext_int_edge1(edge)			ext_int_edge(0, edge)
#define ext_int_edge(source, edge)		do {if (source == 1) INTCON2bits.INTEDG1 = edge else	\
						 if (source == 2) INTCON2bits.INTEDG2 = edge else	\
						 INTCON2bits.INTEDG0 = edge;} while(0)

#define fabs(value)				fabsf(value)
#define floor(value)				floorf(value)
#define fmod(val1, val2)			modff(val1, val2)

#define get_rtcc()				get_timer0()
#define get_timer0()				((unsigned int16) TMR0L | (TMR0H << 8))
#define get_timer1()				((unsigned int16) TMR1L | (TMR1H << 8))
#define get_timer2()				((unsigned int8)  TMR2)
#define get_timer3()				((unsigned int16) TMR3L | (TMR3H << 8))
#define getch()					1 /* todo: rs232 -- needed by PCSelfTest() */
#define goto_address(location)			_asm goto location _endasm; /* CHECK ME */

#define input_from_pin2(port, pin)		((set_mode_##port(TRUE, pin), PORT##port) & (1 << pin))
#define input_from_pin(pin)			input_from_pin2(pin)
#define input(pin)				input_from_pin(PIN(pin))
#define input_a()				(set_mode_A(TRUE, ALL_PINS), PORTA)
#define input_b()				(set_mode_B(TRUE, ALL_PINS), PORTB)
#define input_c()				(set_mode_C(TRUE, ALL_PINS), PORTC)
#define input_d()				(set_mode_D(TRUE, ALL_PINS), PORTD)
#define input_e()				(set_mode_E(TRUE, ALL_PINS), PORTE)

#define kbhit()					TRUE /* todo: rs232 -- needed by PCSelfTest() */

#define labs(x)						(((x) > 0) ? (x) : (-(x))) /* difference to abs() ? */
#define ldexp(value, exp)				ldexpf(value, exp)
long ldiv_compute(int num, int denom) wparam;
#define ldiv(num, denom)			((ldiv_t)ldiv_compute((num), (denom)))
#define log(value)				logf(value)
#define log10(value)				log10f(value)

#define make8(var, offset)			(((var) >> ((offset)<<3)) & 0xff)
#define make16(varhigh, varlow)			((unsigned int)((((varhigh) & 0xff) << 8)+((varlow) & 0xff)))
/* overloading not solvable, needs manual source modification */
#define make32_1(var1)				((unsigned long) var1)
#define make32_2(var1,var2)			( (sizeof(var2)+sizeof(var1) = 4) ? (var1 << 16 | var2 ) : (var1 << (sizeof(var2) << 3) | var2) )
#define make32_3(var1,var2,var3)		( (sizeof(var1)+sizeof(var2)+sizeof(var3) > 4) ? ( (var1 << ((4-sizeof(var1)) << 3)) | (var2 << ((4-sizeof(var1)-sizeof(var2)) << 3)) | ( var3 >> ((sizeof(var1)+sizeof(var2)+sizeof(var3)-4) << 3) )) :  (((make32_2(var1, var2)) << sizeof (var3)) | var3) )
#define make32_4(var1,var2,var3,var4)		( (sizeof(var1)+sizeof(var2)+sizeof(var3)+sizeof(var4) = 4) ? ( (var1 << 24) | (var2 << 16) | (var3 << 8) | var4 )  : (make32_3(var1, var2, var3)))

#define output_a(value)				do {LATA = value; set_mode_A(FALSE, ALL_PINS);} while(0)
#define output_b(value)				do {LATB = value; set_mode_B(FALSE, ALL_PINS);} while(0)
#define output_c(value)				do {LATC = value; set_mode_C(FALSE, ALL_PINS);} while(0)
#define output_d(value)				do {LATD = value; set_mode_D(FALSE, ALL_PINS);} while(0)
#define output_e(value)				do {LATE = value; set_mode_E(FALSE, ALL_PINS);} while(0)
#define output_bit3(port, pin, value)		do {LAT##port##bits.LAT##port##pin = value; set_mode_##port(FALSE, pin); } while(0)
#define output_bit2(pin, value)		        output_bit3(pin, value)
#define output_bit(dest, value)			output_bit2(PIN(dest),value)
#define output_float_set(port, pin)		bit_set(port, pin)
#define output_float2(pin)			output_float_set(pin)
#define output_float(pin)			output_float2(TRIS(pin))
#define output_high(...)			output_bit2(PIN(__VA_ARGS__), 1)
#define output_low(...)				output_bit2(PIN(__VA_ARGS__), 0)

#define port_b_pullups(value)			INTCON2bits.RBPU = (1 - value);
#define pow(x, y)				powf(x, y)
#define psp_input_full()			PSPCONbits.IBF
#define psp_output_full()			PSPCONbits.OBF
#define psp_overflow()				PSPCONbits.IBOV
#ifndef putc
#define putc(cdata)				/* todo: rs232 */
#endif
#ifndef putchar
#define putchar(cdata)				/* todo: rs232 */
#endif

#define read_bank(bank, offset)				(*((char*)(((bank) << 8) + (offset))))
#define read_program_memory(address, dataptr, count)	do {unsigned char i = 0; TBLPTRU = ((address) >> 16); \
							TBLPTRH = (((address) & 0xFF00) >> 8); TBLPTRL = ((address) & 0xFF); \
							for (;i<count;i++) {_asm TBLRD *+ _endasm; dataptr[i] = TABLAT;}} while(0)
#define reset_cpu()				_asm RESET _endasm
#define restart_cause()				(RCON & 0x001F)
#define restart_wdt()				_asm CLRWDT _endasm
void rotate_left (unsigned char *adr, unsigned char n_bytes) wparam;
void rotate_right (unsigned char *adr, unsigned char n_bytes) wparam;

#define set_rtcc(value)				set_timer_0(value)
#define set_timer0(value)			do {TMR0L=(value) & 0x00FF; TMR0H=((value) >> 8);} while(0)
#define set_timer1(value)			do {TMR1L=(value) & 0x00FF; TMR1H=((value) >> 8);} while(0)
#define set_timer2(value)			TMR2=(value)
#define set_timer3(value)			do {TMR3L=(value) & 0x00FF; TMR3H=((value) >> 8);} while(0)
#define set_timer4(value)			do {TMR4L=(value) & 0x00FF; TMR4H=((value) >> 8);} while(0)
#define set_tris_a(value)			TRISA=value 
#define set_tris_b(value)			TRISB=value
#define set_tris_c(value)			TRISC=value
#define set_tris_d(value)			TRISD=value
#define set_tris_e(value)			TRISE=value
#define setup_ccp1(mode)			CCP1CON = mode
#define setup_ccp2(mode)			CCP2CON = mode
#define setup_ccp3(mode)			CCP3CON = mode
#define setup_ccp4(mode)			CCP4CON = mode
#define setup_ccp5(mode)			CCP5CON = mode
#define setup_psp(mode)				PSPCON = mode
#define setup_spi(mode)				SSPCON1 = mode
#define setup_timer_0(mode)			T0CON = (mode) ^ 0x80
#define setup_timer_1(mode)			T1CON = mode
#define setup_timer_2(mode,period,postscale)	/* postscale must be 0..15, not 1..16! */ \
						do {T2CON = ((mode) | (((postscale-1) & 0x0F) << 3)); PR2 = period;} while(0)
#define setup_timer_3(mode)			T3CON = mode
#define setup_timer_4(mode)			T4CON = mode
#define setup_wdt(mode)				do {if (mode == WDT_ON) _asm bsf _WDTCON, 0 _endasm; else _asm bcf _WDTCON, 0 _endasm;} while(0)
unsigned char shift_left (unsigned char *adr, unsigned char n_bytes, unsigned char value) wparam;
unsigned char shift_right (unsigned char *adr, unsigned char n_bytes, unsigned char value) wparam;
#define sin(rad)				sinf(rad)
#define sinh(value)				sinhf(value)
#define sleep()					_asm sleep _endasm
#define spi_data_is_in()			SSPSTATbits.BF
#define spi_write(value)			SSPBUF = value
#define sqrt(value)				sqrtf(value)
#define swap(x)					x=((((x) << 4) & 0x0F0) | (((x) >> 4) & 0x0F))

#define tan(rad)				tanf(rad)
#define tanh(value)				tanhf(value)

#define write_bank(bank, offset, value)				*(char*)(((bank)<<8)+(offset))=value


void write_program_memory(unsigned long address, unsigned char * dataptr, unsigned char count) wparam;


#endif // __TOOLS_H__
