/**
  * Copyright (c) 2004, Telecooperation Office (TecO),
  * Universitaet Karlsruhe (TH), Germany.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
  *
  *   * Redistributions of source code must retain the above copyright
  *     notice, this list of conditions and the following disclaimer.
  *   * Redistributions in binary form must reproduce the above
  *     copyright notice, this list of conditions and the following
  *     disclaimer in the documentation and/or other materials provided
  *     with the distribution.
  *   * Neither the name of the Universitaet Karlsruhe (TH) nor the names
  *     of its contributors may be used to endorse or promote products
  *     derived from this software without specific prior written
  *     permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
  * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
  * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
  * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
  * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 **/

/*
*
* Date:
*  2005_02_01 Daniel & Raphael
*  CCS compatibility functions
*
*/

#include "tools.h"

void write_program_memory(unsigned long address, unsigned char * dataptr, unsigned char count) wparam
{
  uint8_t i = 0;
  uint8_t j = 0;
  uint8_t parts = 0;
  unsigned char tsa[64];
  unsigned long waddr;
  unsigned long bcounter = 0;
  unsigned char offset;

  if (count == 0) return;

  waddr = address & 0xFFFFC0;
  offset = address & 0x3F;

  for (parts = 0; parts < ((count - 1 + (address & 0x3F)) / 64)+1  ; parts ++) {
    
    TBLPTRU = (waddr >> 16);
    TBLPTRH = ((waddr & 0xFF00) >> 8);
    TBLPTRL = (waddr & 0xC0);
    
    /* copy */
    for (i = 0; i < 64; i++) {
      _asm TBLRD *+ _endasm;
      tsa[i] = TABLAT;
    }
    
    /* modify */
    for (i = 0; (i < 64-offset) && (bcounter < count); i ++) {
      bcounter ++;
      tsa[i+offset] = dataptr[i];
    }
    
    TBLPTRU = (waddr >> 16);
    TBLPTRH = ((waddr & 0xFF00) >> 8);
    TBLPTRL = (waddr & 0xC0);
    
    /* erase */
    _asm
      BSF     _EECON1, 7
      BCF     _EECON1, 6
      BSF     _EECON1, 2
      BSF     _EECON1, 4
      BCF     _INTCON, 7
      MOVLW   0x55
      MOVWF   _EECON2
      MOVLW   0xAA
      MOVWF   _EECON2
      BSF     _EECON1, 1
      NOP
      BSF     _INTCON, 7
    _endasm;

    for (i = 0; i < 8; i++) {
      for (j = 0; j < 7; j++) {
	TABLAT = tsa[i*8+j];
	_asm
	  TBLWT *+
	_endasm;
      }

      TABLAT = tsa[i*8+7];
      _asm
	TBLWT * // pointer has to point on the last byte to write
      _endasm;

      _asm 
	BSF    _EECON1, 7
	BCF    _EECON1, 6
	BSF    _EECON1, 2
	BCF    _INTCON, 7 
	MOVLW  0x55
	MOVWF  _EECON2      
	MOVLW  0xAA
	MOVWF  _EECON2      
	BSF    _EECON1, 1  
	NOP
	BSF    _INTCON, 7
	TBLRD *+ // dummy read to increment pointer
      _endasm;
    }

    _asm
       BCF    _EECON1, 2
    _endasm;

    waddr += 64;
    offset = 0;
  }
  return;
}
