## Copyright (c) 2004, Telecooperation Office (TecO),
## Universitaet Karlsruhe (TH), Germany.
## All rights reserved.
##
## Redistribution and use in source and binary forms, with or without
## modification, are permitted provided that the following conditions
## are met:
##
##   * Redistributions of source code must retain the above copyright
##     notice, this list of conditions and the following disclaimer.
##   * Redistributions in binary form must reproduce the above
##     copyright notice, this list of conditions and the following
##     disclaimer in the documentation and/or other materials provided
##     with the distribution.
##   * Neither the name of the Universitaet Karlsruhe (TH) nor the names
##     of its contributors may be used to endorse or promote products
##     derived from this software without specific prior written
##     permission.
##
## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
## "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
## LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
## A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
## OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
## SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
## LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
## DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
## THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
## (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
## OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

## Date
##   2005_02_01 Daniel & Raphael
##   initial Makefile for the Particle Base System



## DIRECTORIES #########################################################
#
# Directories containing sources to compile. This does NOT mean that all
# sources in these directories will be compiled! This variable is only
# used to create the corresponding build directories.

BUILD_DIRS := \
	system \
	boards \
	sensors \
	libtools


## COMMUNICATION #######################################################
#
# (Nearly) board independent parts of the communication stack

COMM_SOURCES := \
	system/rf.c \
	system/ll.c \
	system/acl.c \
	system/app.c \
	boards/wdt18fxx20.c

COMM_OBJECTS = $(COMM_SOURCES:.c=.o)



## BOARD SOURCES #######################################################
#
# Board specific sources should be added to a BOARD[nr]_SOURCES variable,
# the corresponding loader should be added to LOADER[nr]
# The sources are selected via the BOARD variable in the
# application's Makefile or via command line parameter.
#
# You also have to provide appropriate linker scripts for the application
# and the loader. These scripts differ from the default scripts
# (e.g. 18f6720.lkr) because some code regions have to be protected:
# - the application must not place code where the loader is placed
# - the loader must not place code where the application might be



## BOARD 210 SOURCES, LINKER AND LOADER ################################

BOARD210_SOURCES := \
	boards/pci2c.c \
	boards/pceeprom.c \
	system/eeprom_otap.c \
	boards/board210.c \
	boards/18f6720.c

APPLKR210 := app\#210.lkr

LOADER210 := system/eeprom_loader.c
LOADERLKR229 := loader\#210.lkr


## BOARD 229 SOURCES, LINKER AND LOADER ################################

BOARD229_SOURCES := \
	boards/at45db041.c \
	system/flash_otap.c \
	boards/ballswitch.c \
	boards/ds2431.c \
	boards/owmb.c \
	boards/pci2c.c \
	boards/board229.c \
	boards/18f6720.c

APPLKR229 := app\#229.lkr

LOADER229 := system/flash_loader.c
LOADERLKR229 := loader\#229.lkr



BOARD_SOURCES = $(BOARD$(BOARD)_SOURCES)
BOARD_OBJECTS = $(BOARD_SOURCES:.c=.o)
APPLKR       := $(APPLKR$(BOARD))
LOADER       := $(LOADER$(BOARD))
LOADERLKR     = $(LOADERLKR$(BOARD))



## SENSORBOARD SOURCES #################################################
#
# Sensorboard specific sources (sensorboard, drivers, sensori2c if
# needed)should be added to a [BOARDNAME]_SOURCES variable.
# The sources are selected via the SENSORBOARD variable in the
# application's Makefile or via command line parameter.


## SSIMP192 SOURCES ####################################################

SSIMP192_SOURCES := \
	sensors/adxl210.c \
	sensors/adxl210_xy.c \
	sensors/adxl210_z.c \
	sensors/fsr.c \
	sensors/sp0101nc1.c \
	sensors/tc74.c \
	sensors/tsl250_generic.c \
	sensors/tsl2550.c \
	sensors/voltage.c \
	boards/sensori2c.c \
	boards/ssimp192.c 




SENSORS_SOURCES = $($(SENSORBOARD)_SOURCES)
SENSORS_OBJECTS = $(SENSORS_SOURCES:.c=.o)




## TOOLS ###############################################################

TOOLS_SOURCES := \
	libtools/delaycyclesloop.c \
	libtools/divcomp.c \
	libtools/rotate.c \
	libtools/shift.c \
	libtools/writeprogrammem.c

TOOLS_OBJECTS = $(TOOLS_SOURCES:.c=.o)


## STACK MODULES #######################################################
#
# The following modules affect the stack. Therefore they are passed to the
# sources via -D__OPT_....__
# If a module was selected in the application Makefile the corresponding
# sources are added to the appropriate file lists. Some modules may depend
# on others (like SDJS depends on ACK), so ensure that all dependencies
# are solved.


# SDJS module
ifeq (SDJS,$(findstring SDJS,$(STACKOPTIONS)))
COMM_SOURCES+= system/sdjs.c
# ACK is needed by SDJS, so compile it and add it to the options if not present
ifneq (ACK,$(findstring ACK,$(STACKOPTIONS)))
STACKOPTIONS+= ACK
endif
endif

# ACK module
ifeq (ACK,$(findstring ACK,$(STACKOPTIONS)))
COMM_SOURCES+= system/ack.c
endif


# pass the options to the preprocessor
OPTCMDLINE = $(STACKOPTIONS:%=-D__OPT_%__)



## OTHER MODULES #######################################################
#
# The following modules do not affect the stack. Therefore nothing is
# passed to the stack sources. (compare to the stack modules above)
# If a module was selected in the application Makefile the corresponding
# sources are added to the OTHER_SOURCES file list. Some modules may
# depend on others so ensure that all dependencies are solved.


OTHER_SOURCES = 

# RTC module
ifeq (RTC,$(findstring RTC,$(OPTIONS)))
OTHER_SOURCES+= system/rtc.c
endif


OTHER_OBJECTS = $(OTHER_SOURCES:.c=.o)





## RULES ###############################################################

.PHONY: default loader

ifdef SENSORBOARD
PARTICLEBASE = sensors.lib
endif

PARTICLEBASE+= awarecon.lib libtools.lib $(OTHER_OBJECTS) 


default: all


libtools.lib: $(TOOLS_OBJECTS)
	$(GPLIB) -c libtools.lib $(TOOLS_OBJECTS)


awarecon.lib: $(COMM_OBJECTS) $(BOARD_OBJECTS)
	$(GPLIB) -c awarecon.lib $(COMM_OBJECTS) $(BOARD_OBJECTS)


sensors.lib: $(SENSORS_OBJECTS)
	$(GPLIB) -c sensors.lib $(SENSORS_OBJECTS)



# The following rules are made dependent from the particle.mk itself
# and other user defined files (the user's Makefile should be contained).
# This is important to propagate compile flag changes to all sources,
# especially if the board was changed. This might be annoying but leave
# it for safety reasons.

# create dependency files
%.d: %.c particle.mk $(SPECIAL_DEPEND)
	@for dir in $(BUILD_DIRS) "`dirname $@`"; do \
	  [ -d $$dir ] || mkdir -p $$dir; \
	done
	@-set -e; rm -f $@; \
	echo "Checking dependencies for $<"; \
	$(CC) -M $(CPPFLAGS) -Dpic$(TARGET) $< | sed -e 's,\(.*\)\.rel[ :]*,$*.o $@ : ,g'  > $@; 


# create the object files
%.o: %.c particle.mk $(SPECIAL_DEPEND)
	$(CC) $(CFLAGS) $(CPPFLAGS) -o $@ $<


# build the separate loader
loader: $(LOADER)
	$(CC) $(CPPFLAGS) $(CFLAGS_LOADER) $< -o $(notdir $(LOADER:.c=.hex))



# include the dependency files if we do not want to clean up
ifneq ($(MAKECMDGOALS),clean)
ifneq ($(MAKECMDGOALS),realclean)
-include $(COMM_SOURCES:.c=.d)
-include $(BOARD_SOURCES:.c=.d)
-include $(TOOLS_SOURCES:.c=.d)

ifdef SENSORBOARD
-include $(SENSORS_SOURCES:.c=.d)
endif

ifdef OTHER_SOURCES
-include $(OTHER_SOURCES:.c=.d)
endif

ifdef MY_SRC
-include $(MY_SRC:.c=.d)
endif
endif
endif
