/**
  * Copyright (c) 2004, Telecooperation Office (TecO),
  * Universitaet Karlsruhe (TH), Germany.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
  *
  *   * Redistributions of source code must retain the above copyright
  *     notice, this list of conditions and the following disclaimer.
  *   * Redistributions in binary form must reproduce the above
  *     copyright notice, this list of conditions and the following
  *     disclaimer in the documentation and/or other materials provided
  *     with the distribution.
  *   * Neither the name of the Universitaet Karlsruhe (TH) nor the names
  *     of its contributors may be used to endorse or promote products
  *     derived from this software without specific prior written
  *     permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
  * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
  * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
  * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
  * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 **/


#ifndef __ADXL210_Z_H__
#define __ADXL210_Z_H__
/**
* Sensor driver for adxl210
*
* Provides low and high level functions for initializing and sampling of the
* adxl acceleration sensor
*
* Author: Albert Krohn  (krohn@teco.edu)
* Date: 2003-06-11 (yyyy-mm-dd)
*
* version 001 (2003-12-03, cdecker)
*   - driver on new software architecure
*
*
* Date:
*  2005_02_01 Daniel & Raphael
*  new Particle Base System organization
*
*/


#define ACCL_Z_SENSOR			001

#include <tools.h>

/**
* Switches the ACCL on
*/
#define AcclZSensorOn()	output_high(PIN_ACCL_ADXL_POWER_Z)

/**
* Switches the Adxl off
*/
#define AcclZSensorOff()	output_low(PIN_ACCL_ADXL_POWER_Z)


/**
* Initializes the Adxl
* Sets power to output and sampling pins to input.
*
* @return returns 0 if no error
*/
uint8_t AcclZSensorInit();


/**
* Setups the microcontroller for sampling
* @return returns 0 if no error
*/
uint8_t AcclZSensorPrepare();



/**
* Returns the acceleration in g scale
* Unit: for one g (scale=10 means +-1g is +-10 digits)
* @param g the accl value in g
* @return returns 0 if no error
*/
uint8_t AcclZSensorGetGravity(int16_t* g) wparam;


/**
* Packs the accl sensor value for the Z axis directly in the sendbuffer
* as an ACL packet which is ready to be sent.
* @param value temperature sensor value
* @return error code, 0 if no error
*
* Note: The format is described in acltypes.h
*/
uint8_t AcclZSensorPackInACL(int16_t value) wparam;

#endif //__ADXL210_Z_H__
