/**
  * Copyright (c) 2004, Telecooperation Office (TecO),
  * Universitaet Karlsruhe (TH), Germany.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
  *
  *   * Redistributions of source code must retain the above copyright
  *     notice, this list of conditions and the following disclaimer.
  *   * Redistributions in binary form must reproduce the above
  *     copyright notice, this list of conditions and the following
  *     disclaimer in the documentation and/or other materials provided
  *     with the distribution.
  *   * Neither the name of the Universitaet Karlsruhe (TH) nor the names
  *     of its contributors may be used to endorse or promote products
  *     derived from this software without specific prior written
  *     permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
  * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
  * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
  * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
  * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 **/


#ifndef __FSR_H__
#define __FSR_H__
/**
* Sensor driver for force sensor fsr
*
* Provides low and high level functions for initializing and sampling of the
* force sensor.
*
* Author: Christian Decker (cdecker@teco.edu)
* Date: 2003-11-26 (yyyy-mm-dd)
*
* version 001 (2003-11-28, cdecker)
*   - changed pin naming from old driver design
*
* version 002 (2003-12-02, cdecker)
*   - sensor runs on new software architecture
*
* version 003 (2003-12-03, cdecker)
*   - add function to pack sensor value directly into packet
*
*
* Date:
*  2005_02_01 Daniel & Raphael
*  new Particle Base System organization
*
*/


#define FORCE_SENSOR			003

#include <tools.h>

/**
* Switches the force sensor on
*/
#define ForceSensorOn()	output_high(PIN_FORCE_FSR_POWER)

/**
* Switches the force sensor off
*/
#define ForceSensorOff()	output_low(PIN_FORCE_FSR_POWER)


/**
* Initializes the FSR force sensor.
* Sets power to output and sampling pin to input.
*
* @return returns 0 if no error
*/
uint8_t ForceSensorInit();


/**
* Setups the microcontroller for sampling
* Sets the correct AD channel, if neccessary
* @return returns 0 if no error
*/
uint8_t ForceSensorPrepare();


/**
* Gets the sensor value
* @param value the sensor value
* @return returns 0 if no error
*/
uint8_t ForceSensorGet(uint8_t* value) wparam;

/**
* Packs the force sensor value directly in the sendbuffer
* as an ACL packet which is ready to be sent.
* @param value force sensor value
* @return error code, 0 if no error
*
* Note: The format is described in acltypes.h
*/
uint8_t ForceSensorPackInACL(uint8_t value) wparam;

#endif //__FSR_H__
