/**
  * Copyright (c) 2004, Telecooperation Office (TecO),
  * Universitaet Karlsruhe (TH), Germany.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
  *
  *   * Redistributions of source code must retain the above copyright
  *     notice, this list of conditions and the following disclaimer.
  *   * Redistributions in binary form must reproduce the above
  *     copyright notice, this list of conditions and the following
  *     disclaimer in the documentation and/or other materials provided
  *     with the distribution.
  *   * Neither the name of the Universitaet Karlsruhe (TH) nor the names
  *     of its contributors may be used to endorse or promote products
  *     derived from this software without specific prior written
  *     permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
  * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
  * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
  * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
  * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 **/


/**
* Sensor template driver example for a user sensor
*
* Provides low and high level functions for initializing and sampling of the example sensor
* the sensor has a power line on conn2 and an analogue value on conn11.
*
* Author: Al krohn@teco.edu
* Date: 2004-04-13 (yyyy-mm-dd)
*
*/

#include "mysensor.h"
#include "../boards/coreboard.h"
#include "../boards/sensorboard.h"
#include "../system/acl.h"

/**
* Initializes my sensor
* Sets power to output and sampling pin to input.
*
* @return returns 0 if no error
*/

uint8_t MySensorInit()
{
	bit_clear(TRIS_CONN_02);					// set power pin to output
	bit_set(TRIS_CONN_03);					// set sample pin to input

	MySensorOff();							// power off

	return 0;								// indicate no error
}




/**
* Gets the sensor value
* @param value the sensor value
* @return returns 0 if no error
*/
uint8_t MySensorGet(uint8_t* value) wparam
{

	*value = ADCRead8();
	return 0;
}


/**
* Setups the microcontroller for sampling
* Sets the correct AD channel, if neccessary
* @return returns 0 if no error
*/
uint8_t MySensorPrepare()
{
	// set channel
	if ( !ADCIsChSet(ADCH_CONN_11) ) {
		ADCSetCh(ADCH_CONN_11);
	}

	// set conversion clock
	ADCSetClock(0);

	// start ADC
	ADCEnable();

	return 0;
}


/**
* Packs the sensor value directly in the sendbuffer
* as an ACL packet which is ready to be sent.
* @return error code, 0 if no error
*
* Note: The format is described in acltypes.h
*/
uint8_t MySensorPackInACL(uint16_t value) wparam
{

	if ( ACLGetRemainingPayloadSpace() > 4 )
	{
		ACLAddNewType(ACL_TYPE_SVC_HI,ACL_TYPE_SVC_LO); // SVC
		ACLAddData((uint8_t)(value>>8));					// value
		ACLAddData((uint8_t)(value&0x00FF));				// value
	}
	else
	{
		return 1; 	// not enough space left
	}

	return 0; // no error
}

