/**
  * Copyright (c) 2004, Telecooperation Office (TecO),
  * Universitaet Karlsruhe (TH), Germany.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
  *
  *   * Redistributions of source code must retain the above copyright
  *     notice, this list of conditions and the following disclaimer.
  *   * Redistributions in binary form must reproduce the above
  *     copyright notice, this list of conditions and the following
  *     disclaimer in the documentation and/or other materials provided
  *     with the distribution.
  *   * Neither the name of the Universitaet Karlsruhe (TH) nor the names
  *     of its contributors may be used to endorse or promote products
  *     derived from this software without specific prior written
  *     permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
  * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
  * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
  * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
  * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 **/


#ifndef __MYSENSOR_H__
#define __MYSENSOR_H__
/**
* Sensor template driver example for a user sensor
*
* Provides low and high level functions for initializing and sampling of the example sensor
* the sensor has a power line on conn2 and an analogue value on conn11.
*
* Author: Al krohn@teco.edu
* Date: 2004-04-13 (yyyy-mm-dd)
*
*/

#define MY_SENSOR			001					// define the name and version of your sensor

#include <tools.h>

/**
* Switches the sensor on
*/
#define MySensorOn()	output_high(PIN_CONN_02);

/**
* Switches the sensor off
*/
#define MySensorOff()	output_low(PIN_CONN_02);


/**
* Initializes my sensor
* Sets power to output and sampling pin to input.
*
* @return returns 0 if no error
*/

uint8_t MySensorInit();


/**
* Setups the microcontroller for sampling
* Sets the correct AD channel, if neccessary
* @return returns 0 if no error
*/
uint8_t MySensorPrepare();


/**
* Gets the sensor value
* @param sensor value 
* @return returns 0 if no error
*/
uint8_t MySensorGet(uint16_t* value) wparam;


/**
* Packs the sensor value directly in the sendbuffer
* as an ACL packet which is ready to be sent.
* @param value sensor value
* @return error code, 0 if no error
*
* Note: The format is described in acltypes.h
*/
uint8_t MySensorPackInACL(uint16_t value) wparam

#endif // __MYSENSOR_H__
