/**
  * Copyright (c) 2004, Telecooperation Office (TecO),
  * Universitaet Karlsruhe (TH), Germany.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
  *
  *   * Redistributions of source code must retain the above copyright
  *     notice, this list of conditions and the following disclaimer.
  *   * Redistributions in binary form must reproduce the above
  *     copyright notice, this list of conditions and the following
  *     disclaimer in the documentation and/or other materials provided
  *     with the distribution.
  *   * Neither the name of the Universitaet Karlsruhe (TH) nor the names
  *     of its contributors may be used to endorse or promote products
  *     derived from this software without specific prior written
  *     permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
  * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
  * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
  * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
  * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 **/


#ifndef __SP0101NC1_H__
#define __SP0101NC1_H__
/**
* Sensor driver for microphone SP0101NC1
*
* Provides low and high level functions for initializing and sampling of the
* microphone
*
* Author: Albert Krohn  (krohn@teco.edu)
* Date: 2003-06-11 (yyyy-mm-dd)
*
* version 001 (2003-12-03, cdecker)
*   - driver on new software architecure
*
* version 002 (2003-12-04, cdecker)
*   - error in volume computation which causes infinte loop corrected
*
* Date:
*  2005_02_01 Daniel & Raphael
*  new Particle Base System organization
*
*/


#define AUDIO_SENSOR			002

#include <tools.h>
/**
* Switches the micro amp on
*/
#define AudioSensorOn()	output_high(PIN_AUDIO_MIC_POWER)

/**
* Switches the micro amp off
*/
#define AudioSensorOff()	output_low(PIN_AUDIO_MIC_POWER)


/**
* Initializes the Audio microphone amp
* Sets power to output and sampling pin to input.
*
* @return returns 0 if no error
*/
uint8_t AudioSensorInit();



/**
* Setups the microcontroller for sampling
* Sets the correct AD channel, if neccessary
* @return returns 0 if no error
*/
uint8_t AudioSensorPrepare();

/**
* Computes the average power of the audio signal.
* This power is computed
* @param volume pointer for storing the volume level
* @return returns 0 if no error
*/
uint8_t AudioSensorGetVolume(uint8_t* volume) wparam;

/**
* Packs the audio sensor value directly in the sendbuffer
* as an ACL packet which is ready to be sent.
* @param type type of audio sensor value (e.g. 1 for mean volume)
* @param data pointer to data array of audio value(s)
* @param dataLength length of data array
* @return error code, 0 if no error
*
* Note: The format is described in acltypes.h
*/
uint8_t AudioSensorPackInACL(uint8_t type, uint8_t* __PIC_data, uint8_t dataLength) wparam;

#endif // __SP0101NC1_H__
