/**
  * Copyright (c) 2004, Telecooperation Office (TecO),
  * Universitaet Karlsruhe (TH), Germany.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
  *
  *   * Redistributions of source code must retain the above copyright
  *     notice, this list of conditions and the following disclaimer.
  *   * Redistributions in binary form must reproduce the above
  *     copyright notice, this list of conditions and the following
  *     disclaimer in the documentation and/or other materials provided
  *     with the distribution.
  *   * Neither the name of the Universitaet Karlsruhe (TH) nor the names
  *     of its contributors may be used to endorse or promote products
  *     derived from this software without specific prior written
  *     permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
  * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
  * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
  * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
  * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 **/


#ifndef __TC74_H__
#define __TC74_H__
/**
* Sensor driver for tc74 il2c temp sensor
*
* Provides low level functions for reading the tc74 temperature sensor
* via I2C.
*
* Author: Albert Krohn  (krohn@teco.edu)
* Date: 2003-07-06 (yyyy-mm-dd)
*
* modified by cdecker@teco.edu
*
* version 001:
*   - adapt driver to new i2c wrapper
*
* version 002:
*   - ACL pack function
*
*
* Date:
*  2005_02_01 Daniel & Raphael
*  new Particle Base System organization
*
*/


#define TEMPERATURE_SENSOR		002

#define I2C_TEMPERATURE_ID		0x90

#include <tools.h>

/**
* Switches the temperature sensor on
*/
#define TemperatureSensorOn()	output_high(PIN_TEMP_TC74_POWER)

/**
* Switches the temperature sensor off
*/
#define TemperatureSensorOff()	output_low(PIN_TEMP_TC74_POWER)


/**
* Initializes the temperature sensor
* Uses the Sensor I2C Bus
*
* @return returns 0 if no error
*/

uint8_t TemperatureSensorInit();

/**
* Setups the I2C bus for the temperature sensor
* @return returns 0 if no error
*/
uint8_t TemperatureSensorPrepare();

/**
* Gets the sensor value
* @param value the sensor value
* @return returns 0 if no error
*/
uint8_t TemperatureSensorGet(int8_t* value) wparam;


/**
* Packs the temperature sensor value directly in the sendbuffer
* as an ACL packet which is ready to be sent.
* @param value temperature sensor value
* @return error code, 0 if no error
*
* Note: The format is described in acltypes.h
*/
uint8_t TemperatureSensorPackInACL(int8_t value) wparam;

#endif // __TC74_H__
