/**
  * Copyright (c) 2004, Telecooperation Office (TecO),
  * Universitaet Karlsruhe (TH), Germany.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
  *
  *   * Redistributions of source code must retain the above copyright
  *     notice, this list of conditions and the following disclaimer.
  *   * Redistributions in binary form must reproduce the above
  *     copyright notice, this list of conditions and the following
  *     disclaimer in the documentation and/or other materials provided
  *     with the distribution.
  *   * Neither the name of the Universitaet Karlsruhe (TH) nor the names
  *     of its contributors may be used to endorse or promote products
  *     derived from this software without specific prior written
  *     permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
  * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
  * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
  * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
  * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 **/


#ifndef __VOLTAGE_H__
#define __VOLTAGE_H__
/**
* Sensor driver for measuring the battery voltage
*
* This is a particle core board driver, i.e. the voltage measurement is done
* on the core board and not on a sensor board.
* Provides low and high level functions for initializing and sampling of the
* voltage measurment.
*
* Author: Christian Decker (cdecker@teco.edu)
* Date: 2003-12-11 (yyyy-mm-dd)
*
* version 001 (2003-12-11, cdecker)
*   - init version
*
* version 002 (2004-04-05) by Klaus Schmidt (schmidtk@teco.edu)
*	- SPart & Particle version
*
* version 003 (2004-11-03) by Michael Demel	(demel@teco.edu)
*   - Voltage Sensor adjust to 3,3V (VRef)
*
* Date:
*  2005_02_01 Daniel & Raphael
*  new Particle Base System organization
*
*/


#define VOLTAGE_SENSOR			002

#include <tools.h>
/**
* On/Off functions are not need, because this feature is not supported
*/

/**
* Initializes the Voltage sensor.
* @return returns 0 if no error
*/

uint8_t VoltageSensorInit();


/**
* Gets the voltage in millivolt (mV)
* @param value voltage in millivolt (mV)
* @return returns 0 if no error
*/
uint8_t VoltageSensorGet(uint16_t* value) wparam;

/**
* Packs the voltage directly in the sendbuffer
* as an ACL packet which is ready to be sent.
* @param value voltage measurement [mV]
* @return error code, 0 if no error
*
* Note: The format is described in acltypes.h
*/
uint8_t VoltageSensorPackInACL(uint16_t value) wparam;

/**
* Setups the microcontroller for sampling
* Sets the correct AD channel, if neccessary
* @return returns 0 if no error
*/
uint8_t VoltageSensorPrepare();


#endif //__VOLTAGE_H__
