/**
  * Copyright (c) 2004, Telecooperation Office (TecO),
  * Universitaet Karlsruhe (TH), Germany.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
  *
  *   * Redistributions of source code must retain the above copyright
  *     notice, this list of conditions and the following disclaimer.
  *   * Redistributions in binary form must reproduce the above
  *     copyright notice, this list of conditions and the following
  *     disclaimer in the documentation and/or other materials provided
  *     with the distribution.
  *   * Neither the name of the Universitaet Karlsruhe (TH) nor the names
  *     of its contributors may be used to endorse or promote products
  *     derived from this software without specific prior written
  *     permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
  * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
  * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
  * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
  * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 **/


#ifndef __ACK_H__
#define __ACK_H__
/**
* Functions for acknowledged packets, to be used within ACL.
*
* Author: sabin (wendhack@teco.edu)
* Date: 2003-10-02 (yyyy-mm-dd)
*
* modified by Christian Decker (cdecker@teco.edu)
* Date: 2003-12-04
* - comment out test_pin implementation
*
* modified by sabin (wendhack@teco.edu)
* Date: 2004-03-03
* - inserted timeout to delete ACL_ACK_yet_received_list-seqencenr.
* - added third return value for sendsuccess-functions: 0 failed, 1 success, 2 still in queue.
* - created ACLACKCheckInPacket() as not to have two times the same code in ACLAckedDataIsNew() and ACLAckedDataIsNewNow().
* - fixed: ACLAckedDataIsNew() and ACLAckedDataIsNewNow() now return false when received DATA_ACK!!
* - explicitly switch on ACLACKRun() in ACLACKInit() by ACL_ACK_on.
* - fixed: inserted ACLSetDataToOld() when found ACLAckedDataIsNew()!
*
* Date:
*  2004_05_26 Albert:
*  fixed ACL_ACK_BUFFER_SIZE from 40 to 64(!)  (why wasn't it 25? or 33????? ... argh!!!!)
*
* Date:
*  2004_11_02 sabin
*  fixed: Check packet in yet received packets-list only one time a slot ( new: ACL_ACK_packet_checked_in_this_slot ).
*
* Date:
*  2005_02_01 Daniel & Raphael
*  new Particle Base System organization
*
*/

#include <tools.h>

void	ACLACKInit();
uint8_t 	ACLAckedDataIsNew();
uint8_t 	ACLAckedDataIsNewNow();
uint8_t		ACLSendingBusyAcked();
uint8_t 	ACLSendPacketAcked( uint8_t add1, uint8_t add2, uint8_t add3, uint8_t add4, uint8_t add5, uint8_t add6, uint8_t add7, uint8_t add8, uint8_t timeout ) wparam;
uint8_t 	ACLGetSendSuccessAcked();
uint8_t 	ACLGetSendSuccessAckedNow();

void 	ACLACKRun();
void	ACLACKCheckNewPacket();
void	ACLACKUpdateTimeouts();
void	ACLACKCheckRepeats();
void	ACLACKHandleError();

uint8_t ACLACKCheckInPacket();
void 	ACLACKSendAck();
void 	ACLACKHandleAck();
void 	ACLACKSendPacketAgain();
void 	ACLACKSendReset();
void 	ACLACKReceiveReset();
void 	ACLACKReset();

#endif //  __ACK_H__
