/**
  * Copyright (c) 2004, Telecooperation Office (TecO),
  * Universitaet Karlsruhe (TH), Germany.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
  *
  *   * Redistributions of source code must retain the above copyright
  *     notice, this list of conditions and the following disclaimer.
  *   * Redistributions in binary form must reproduce the above
  *     copyright notice, this list of conditions and the following
  *     disclaimer in the documentation and/or other materials provided
  *     with the distribution.
  *   * Neither the name of the Universitaet Karlsruhe (TH) nor the names
  *     of its contributors may be used to endorse or promote products
  *     derived from this software without specific prior written
  *     permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
  * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
  * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
  * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
  * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 **/

/*
 *
 * Date:
 *  2005_02_01 Daniel & Raphael
 *  new Particle Base System organization
 *
 */

#include "acl.h"
#include "rf.h"
#include "app.h"

#include "otap.h"
#include "../boards/wdt18fxx20.h"

// ACL

boolean ACL_send_buffer_locked;
boolean ACL_ACM_answers;		// answer on ACM messages like helo and so on
uint8_t ACL_write_position;

uint8_t ACL_subscriptions[ACL_SUBSCRIPTIONLIST_LENGTH][2];
boolean ACL_subscribe_all;
boolean ACL_ignore_control_messages;
boolean ACL_pass_control_messages;

uint8_t ACL_payload_send[LL_PAYLOAD_SIZE];
uint8_t ACL_payload_length;

/* not needed
  uint8_t ACL_i;		// dirty workaround because not enough RAM
  uint8_t ACL_res;
*/


// here comes the "ACL"
//---------------------------------------------------------------------------------------------------------------------------------------
//---------------------------------------------------------------------------------------------------------------------------------------
//---------------------------------------------------------------------------------------------------------------------------------------
//------------------------------LL-------------------------------------------------------------------------------------------------------
//---------------------------------------------------------------------------------------------------------------------------------------
//---------------------------------------------------------------------------------------------------------------------------------------
//---------------------------------------------------------------------------------------------------------------------------------------
//---------------------------------------------------------------------------------------------------------------------------------------




void ACLStartUp()
{
	uint8_t result[10];
	uint8_t selftest_result,i;

	selftest_result=AppSelfTest(result);		//runs the selftest


	//!!
	PCLedBlueOff();
	PCLedRedOff();
	DelayMs(100);				// for stabilizing of power etc.

	ACLInit();					// sets the pins in pic init correct (looks at selftest_active

	if(selftest_result==1)
	{
		for(i=0;i<10;i++)			//send result ten times to get it
		{
			//send packet
			ACLAddNewType(ACL_TYPE_CST_HI,ACL_TYPE_CST_LO);		//MST Self test
			ACLAddData(result[0]);
			ACLAddData(result[1]);
			ACLAddData(result[2]);
			ACLAddData(result[3]);
			ACLAddData(result[4]);
			ACLAddData(result[5]);
			ACLAddData(result[9]);

			ACLSendPacket(30);
			while(ACLSendingBusy()) {PCLedRedOn();PCLedRedOff();}

		}

		while(selftest_active)				//wait for selftest board to be removed
		{
			output_high(PIN_D7);
			if (!input(PIN_B7)) selftest_active=false;
			output_low(PIN_D7);
			if (input(PIN_B7)) selftest_active=false;

		}

		ACLInit();					//restart the whole stack once more
	}


}




void ACLInit()
{

	ACLStop();	//hold stack if already running

	PCLedRedOn();

	ACLFlushSubscriptions();
	ACL_send_buffer_locked=false;
 	ACL_write_position=0;
 	ACL_subscribe_all=false;
 	ACL_ACM_answers=true;
 	ACL_ignore_control_messages=false;
 	ACL_pass_control_messages=false;

	ACL_payload_length=0;
	ACL_write_position=0;

	OtapInit();			//reset all global variables for otap

	AppSetLEDBehaviour(LEDS_NORMAL);			//setdefault


	PCLedRedOn();
	DelayMs(200);
	PCLedRedOff();
	DelayMs(200);


	LLInit();
	ACLStart();


}






uint8_t ACLSubscribe(uint8_t LL_type_h,uint8_t LL_type_l) wparam
{
	uint8_t i;
	for (i=0;i<ACL_SUBSCRIPTIONLIST_LENGTH;i++)		// first test if already there
	{
		if ((ACL_subscriptions[i][0]==LL_type_h) && (ACL_subscriptions[i][1]==LL_type_l)) return 1;
	}
	for (i=0;i<ACL_SUBSCRIPTIONLIST_LENGTH;i++)
	{
		if ((ACL_subscriptions[i][0]==0) && (ACL_subscriptions[i][1]==0))
		{
			ACL_subscriptions[i][0]=LL_type_h;
			ACL_subscriptions[i][1]=LL_type_l;
			return 1;								// successful subscribed
		}
	}
	return 0;										// subsriptions failed
}

uint8_t ACLUnsubscribe(uint8_t LL_type_h,uint8_t LL_type_l) wparam
{
	uint8_t i;

	for (i=0;i<ACL_SUBSCRIPTIONLIST_LENGTH;i++)		// if type is there, delete it
	{
		if ((ACL_subscriptions[i][0]==LL_type_h) && (ACL_subscriptions[i][1]==LL_type_l))
		{
			ACL_subscriptions[i][0]=0;
			ACL_subscriptions[i][1]=0;
			return 1;		// successufl unsubscribed
		}
	}
	return 0;		// not found
}

uint8_t ACLFlushSubscriptions()
{
	uint8_t i;

	for (i=0;i<ACL_SUBSCRIPTIONLIST_LENGTH;i++)
	{
		ACL_subscriptions[i][0]=0;
		ACL_subscriptions[i][1]=0;
	}

	ACL_subscribe_all=false;
	ACLSubscribeDefault();
	return 0;
}

void ACLSubscribeAll()
{
	ACL_subscribe_all=true;
}

void ACLAnswerOnACM()
{
 	ACL_ACM_answers=true;
}
void ACLNoAnswerOnACM()
{
	ACL_ACM_answers=false;
}






uint8_t ACLSubscribeDefault()
{
	uint8_t result;

	result=ACLSubscribe(ACL_TYPE_ACM_H,ACL_TYPE_ACM_L);		//165 14 is CMA (Control and Management virtual Artefact)

	return result;
}




uint8_t ACLVerifySubscription(uint8_t type_h,uint8_t type_l) wparam
{
	uint8_t i;

	for (i=0;i<ACL_SUBSCRIPTIONLIST_LENGTH;i++)
	{
		if ((ACL_subscriptions[i][0]==type_h) && (ACL_subscriptions[i][1]==type_l)) return 1;
	}


	return 0;		//subscription not found
}





// retun one if there was a control msg // return not needed anymore
////#pragma inline
uint8_t ACLProcessControlMessages()
{



	if ((LL_payload_receivebuffer[0]==ACL_TYPE_ACM_H) && (LL_payload_receivebuffer[1]==ACL_TYPE_ACM_L))
	// there was a control msg
	{


 		// remote shutdown: NEEDS THE SOFTWARE-WATCHDOG-DRIVER FOR PIC (only exists for 18fxx20 at this time!!!!)
 		// WDT18FXX20 has to be included before this STACK-File
 		if ((LL_payload_receivebuffer[3]==ACL_TYPE_CRR_HI) && (LL_payload_receivebuffer[4]==ACL_TYPE_CRR_LO))	// check a control msg, e.g. "1,1"
 		{
 			#define LONGESTWDT 7	//this is the longest wait-time which is accepted from the Software-WDT (2.304sec)
 			uint8_t oldwdtvalue = 0;
 			uint8_t oldwdttest = 0;
 			uint8_t shutdownminutes = 0;
 			uint8_t shutdownseconds = 0;
 			uint16_t allseconds = 0;
 			uint16_t i = 0;
 			uint16_t timercycles = 0;

 			if (ACLMatchesMyIP(LL_payload_receivebuffer,6))
 			{
 				shutdownminutes=LL_payload_receivebuffer[14];
 				shutdownseconds=LL_payload_receivebuffer[15];

 				// check shutdown-time - if the given time is 0, the device is reseted
 				if ( (shutdownminutes==0) && (shutdownseconds==0) )
 				{
 					reset_cpu();
 				}

 				// check shutdown-time - if the given time is both 255, ShutDown the whole Device, only wakes up after hardware-reset
 				if ( (shutdownminutes==255) && (shutdownseconds==255) )
 				{
 					ACLStop();
 					sleep();
 				}

 				// Check ShutDown-Time - time in minutes in higher byte and time in seconds in lower byte
 				if ( ( (shutdownminutes!=0) || (shutdownseconds!=0) ) && ( (shutdownminutes!=255) || (shutdownseconds!=255) ) )
 				{
 					oldwdttest = WDTTest();
 					oldwdtvalue = WDTReadValue();

 					allseconds = shutdownseconds + ( 60*shutdownminutes );
 					timercycles = (allseconds>>1);	//Division by 2, because the longest WDT is 2.304s and you need an integral value

 					WDTConfig(LONGESTWDT);
 					WDTEnable();

 					PCLedBlueOff();
 					PCLedRedOff();

 					for (i=0;i<timercycles;i++)
 						{
 							sleep();
 							_asm
 							 nop
 							 nop
 							 nop
 							_endasm;
 						}

 					WDTDisable();

 					WDTConfig(oldwdtvalue);

 					if (oldwdttest) {
 						WDTEnable();
 						}
 				}
 			}
 		}

 		// ControlFieldStrength:
 		// give the value to set the fieldstrength
 		if ((LL_payload_receivebuffer[3]==ACL_TYPE_CFS_HI) && (LL_payload_receivebuffer[4]==ACL_TYPE_CFS_LO))
 		{
 			if (ACLMatchesMyIP(LL_payload_receivebuffer,6))
 			{
 				uint8_t fsvalue;
 				fsvalue = LL_payload_receivebuffer[14];
 				ACLSetFieldStrength(fsvalue);
 			}

 		}



		// check syncrate setting
		if ((LL_payload_receivebuffer[3]==198) && (LL_payload_receivebuffer[4]==208))
		{
			RFSetSyncRate(LL_payload_receivebuffer[6]);

		}

		// check initial listen slots setting
		if ((LL_payload_receivebuffer[3]==159) && (LL_payload_receivebuffer[4]==192))
		{
			RFSetInitialListenSlots(LL_payload_receivebuffer[6]);
		}


		// check modeswitch
		if ((LL_payload_receivebuffer[3]==1) && (LL_payload_receivebuffer[4]==1))	// check a control msg, e.g. "1,1"
		{
		}


		//check over the air programming
		if ((LL_payload_receivebuffer[3]==198) && (LL_payload_receivebuffer[4]==88))	// check a control msg, e.g. "1,1"
		{
				OtapReceive();		// call over the airprogramming stuff
		}


		//check set ID
		if ((LL_payload_receivebuffer[3]==ACL_TYPE_CID_HI) && (LL_payload_receivebuffer[4]==ACL_TYPE_CID_LO) &&( selftest_active==true))
		{
 			OtapSetID();
  		}


                //check ledstyle
                if ((LL_payload_receivebuffer[3]==ACL_TYPE_CLE_HI) && (LL_payload_receivebuffer[4]==ACL_TYPE_CLE_LO))
                {
                        if( ACLMatchesMyIP( LL_payload_receivebuffer, 12 ))
                        {
                                AppSetLEDBehaviour( LL_payload_receivebuffer[6] );
                                if( LL_payload_receivebuffer[7] ) PCLedRedOn();
                                else PCLedRedOff();
                                if( LL_payload_receivebuffer[8] ) PCLedBlueOn();
                                else PCLedBlueOff();
                        }
                }
  
  
		//////////////////////////////////////////
		if (ACL_ACM_answers==false) return 0;	// no answering allowed; all messages below here are not allowed, if acl_acm_answer is false





		//check Helo
		if ((LL_payload_receivebuffer[3]==115) && (LL_payload_receivebuffer[4]==216))
		{
			if (ACLMatchesMyIP(LL_payload_receivebuffer,6))		// does sent IP match my IP??
			{

				if (LLSendingBusy()) ACLAbortSending();			// interrupt any waiting stuff



				// now set the whole packet by hand:
				LL_payload_send[0]=ACL_TYPE_ACM_H;
				LL_payload_send[1]=ACL_TYPE_ACM_L;
				LL_payload_send[2]=0;

				LL_payload_send[3]=134;
				LL_payload_send[4]=32;

				LL_payload_send[5]=8;
				LL_payload_send[6]=LL_header_receivebuffer[3];		// put in orogin adress from HELO
				LL_payload_send[7]=LL_header_receivebuffer[4];
				LL_payload_send[8]=LL_header_receivebuffer[5];
				LL_payload_send[9]=LL_header_receivebuffer[6];
				LL_payload_send[10]=LL_header_receivebuffer[7];
				LL_payload_send[11]=LL_header_receivebuffer[8];
				LL_payload_send[12]=LL_header_receivebuffer[9];
				LL_payload_send[13]=LL_header_receivebuffer[10];
				LL_payload_length=14;								// give the correct len

				if (LLSendPacket(ACL_CONTROL_MESSAGES_TIMEOUT))     // NEVER USE ACL FUNCTION here!!!
				{
					rf_status|=RF_STATUS_INSERTED_CTRLMSG;			// this means that this msg if special concerning the update of the states
				}


			}

		}
		// this was helo



		return 1;											// there was a control management msg
	}
	return 0;												// there was no control management msg
}

void ACLSetFieldStrength(uint8_t power) wparam
{
	LLSetFieldStrength(power);
}


uint8_t ACLSendingBusy()
{
	if (LLSendingBusy()) return 1;
	else	return 0;
}

uint8_t ACLGetSendSuccess()
{
	if (LLGetSendSuccess()) return 1;
	else return 0;
}






uint8_t ACLMatchesMyIP(unsigned char *buffer,uint8_t start) wparam
{
	uint8_t res,i;
	res=0;

	// check if was my address		// ll_header_send [3..10] hold the myIP
	for (i=0;i<8;i++)
	 	if (buffer[start+i]==LL_header_send[3+i]) res++;
	if (res==8) return 1;

	res=0;

	// check if  was broadcast adress
	for (i=0;i<8;i++)
	 	if (buffer[start+i]==255) res++;
	if (res==8) return 1;

	return 0;
}





void ACLAbortSending()
{
	ACL_send_buffer_locked=false;
	ACL_write_position=0;
	ACL_payload_length=0;
	LLAbortSending();
}




// returns1 if packet was started
// returns 0 if still sending another or connection is off..
uint8_t ACLSendPacket(uint8_t slot_timeout) wparam
{

	// check if sending busy or layer off

	if (!(rf_status & RF_STATUS_LAYER_ON)) return 0;
	if (rf_status & RF_STATUS_SENDING) return 0;

	// now copy the ACL to the LL payload
	// # copy
	memcpy(LL_payload_send,ACL_payload_send,LL_PAYLOAD_SIZE);	// copies the acl data to ll
	LL_payload_length=ACL_payload_length;
	if (LLSendPacket(slot_timeout))
	{
		// now ACL_payload_send is free again
		ACL_payload_length=0;
		ACL_write_position=0;
		return 1;
	}

	else return 0;

}

uint8_t ACLClearSendData()
{
	if (LLSendingBusy()) return 0;					// dont clear if busy sending
	if (ACL_send_buffer_locked==true) return 0;		// dont clear if reservation is active

	memset(ACL_payload_send,0, LL_PAYLOAD_SIZE);	// clear buffer

	ACL_payload_length=0;							// no data in ACL buffer
	ACL_write_position=0;


	return 1;
}






// return 0 if ok
// returns 1 if full
// returns 2 if locked
uint8_t ACLAddNewType(uint8_t type_h, uint8_t type_l) wparam
{
	if (ACL_send_buffer_locked==true) return 2;		// dont set if reservation is active
	if ((LL_PAYLOAD_SIZE-ACL_payload_length)<3U)
	{
		return 1;		// not enough space
	}
	else
	{
		//place the new type
		ACL_payload_send[ACL_payload_length]=type_h;
		ACL_payload_send[ACL_payload_length+1]=type_l;
		ACL_payload_send[ACL_payload_length+2]=0;	//data length is zero
		ACL_write_position=ACL_payload_length+2;
		ACL_payload_length+=3;
		return 0;
	}
}

uint8_t ACLAddData(uint8_t payload) wparam
{
	if (ACL_send_buffer_locked==true) return 2;		// dont set if reservation is active
	if ((LL_PAYLOAD_SIZE-ACL_payload_length)<1U)
	{
		return 1;		// not enough space
	}
	else
	{
		ACL_payload_send[ACL_write_position]++;			//count the acl tuple len up
		ACL_payload_send[ACL_payload_length]=payload;		//place the data
		ACL_payload_length+=1;
		return 0;
	}

}

uint8_t ACLGetRemainingPayloadSpace()
{
	//return (LL_payload_length-ACL_payload_length);
	return LL_PAYLOAD_SIZE-ACL_payload_length;
}






uint8_t ACLGetReceivedPayloadLength()
{
	return LL_header_received[1];
}




//looks for the length of context data of the given type, return2 -1 if type wasn't found
int8_t ACLGetReceivedDataLength(uint8_t type_h, uint8_t type_l) wparam
{
	uint8_t i;

	for (i=0;i<=LL_header_received[1]-2;i++)
	{
		if ((LL_payload_received[i]==type_h) && (LL_payload_received[i+1]==type_l))
		{
			return (LL_payload_received[i+2]);											//return valid length
		}
	}

	return -1;																			// type not found
}




unsigned char* ACLGetReceivedData(uint8_t type_h, uint8_t type_l) wparam
{
	uint8_t i;

	for (i=0;i<=LL_header_received[1]-2;i++)
	{
		if ((LL_payload_received[i]==type_h) && (LL_payload_received[i+1]==type_l))
		{
			if (LL_payload_received[i+2]!=0)
			{
				return (&(LL_payload_received[i+3]));									// if len is not zero: return valid pointer to data begin
			}

		}
	}

	//attention!! if type wasn't found: return is undetermined!!
	return 0;
}

unsigned char ACLGetReceivedByte(uint8_t type_h, uint8_t type_l, uint8_t position) wparam
{
	uint8_t i;

	for (i=0;i<=LL_header_received[1]-2;i++)
	{
		if ((LL_payload_received[i]==type_h) && (LL_payload_received[i+1]==type_l))
		{
			if (LL_payload_received[i+2]!=0)
			{
				return (LL_payload_received[i+3+position]);									// if len is not zero: return valid pointer to data begin
			}

		}
	}

	//attention!! if type wasn't found: return is undetermined!!
	return 0;

}







uint8_t ACLFoundReceivedType(uint8_t type_h, uint8_t type_l) wparam
{
	if (ACLGetReceivedDataLength(type_h,type_l)!=-1) return 1;	//found!
	else return 0;		//not found
}



void ACLSetControlMessagesBehaviour(boolean ignore, boolean pass) wparam
{
	ACL_ignore_control_messages=ignore;		//all control messages are ignored
	ACL_pass_control_messages=pass;			// all control messages are as well copied to receivebuffer for applicaiton
}



uint8_t ACLSentPacketInThisSlot()
{
	return (LLSentPacketInThisSlot());
}


void ACLStart()
{
	LLStart();
}

void ACLStop()
{
	LLStop();
}


void ACLLockReceiveBuffer()
{
	LLLockReceiveBuffer();
}

uint8_t ACLReceiveBufferLocked()
{
	return LLReceiveBufferLocked();
}


void ACLReleaseReceiveBuffer()
{
	LLReleaseReceiveBuffer();
}


void ACLSetDataToOld()
{
	LLSetDataToOld();
}

void ACLSetDataToNew()
{
	LLSetDataToNew();
}


uint8_t ACLDataIsNew()
{
	return LLDataIsNew();
}

uint8_t ACLDataReceivedInThisSlot()
{
	if (LL_received_packet_in_this_slot==true) return 1;
	return 0;
}

uint8_t ACLDataIsNewNow()
{
	return ACLDataReceivedInThisSlot();
}


//
uint8_t ACLAdressedDataIsNew()
{
	if (!ACLDataIsNew()) return 0;

	if (ACLFoundReceivedType(ACL_TYPE_CAD_HI,ACL_TYPE_CAD_LO))
	{
		if (ACLMatchesMyIP( ACLGetReceivedData( ACL_TYPE_CAD_HI, ACL_TYPE_CAD_LO), 0 )) return 1;
	}
	return 0;
}

uint8_t ACLAdressedDataIsNewNow()
{
	if (!ACLDataIsNewNow()) return 0;

	if (ACLFoundReceivedType(ACL_TYPE_CAD_HI,ACL_TYPE_CAD_LO))
	{
		if (ACLMatchesMyIP( ACLGetReceivedData( ACL_TYPE_CAD_HI, ACL_TYPE_CAD_LO), 0 )) return 1;
	}
	return 0;
}


//sends out the current packet with a target ID
uint8_t ACLSendPacketAdressed(uint8_t add1, uint8_t add2, uint8_t add3, uint8_t add4, uint8_t add5, uint8_t add6, uint8_t add7, uint8_t add8, uint8_t timeout ) wparam
{
	if (ACLGetRemainingPayloadSpace()<11U) return 0;		//not enough space

	ACLAddNewType(ACL_TYPE_CAD_HI,ACL_TYPE_CAD_LO);
	ACLAddData(add1);
	ACLAddData(add2);
	ACLAddData(add3);
	ACLAddData(add4);
	ACLAddData(add5);
	ACLAddData(add6);
	ACLAddData(add7);
	ACLAddData(add8);

	if (ACLSendPacket(timeout)) return 1;
	return 0;
}

