#ifndef __ACL_H__
#define __ACL_H__
/**
  * Copyright (c) 2004, Telecooperation Office (TecO),
  * Universitaet Karlsruhe (TH), Germany.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
  *
  *   * Redistributions of source code must retain the above copyright
  *     notice, this list of conditions and the following disclaimer.
  *   * Redistributions in binary form must reproduce the above
  *     copyright notice, this list of conditions and the following
  *     disclaimer in the documentation and/or other materials provided
  *     with the distribution.
  *   * Neither the name of the Universitaet Karlsruhe (TH) nor the names
  *     of its contributors may be used to endorse or promote products
  *     derived from this software without specific prior written
  *     permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
  * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
  * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
  * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
  * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 **/

#include "acltypes.h"
#include "ll.h"
#include <tools.h>

//ACL
uint8_t ACLSubscribe(uint8_t LL_type_h,uint8_t LL_type_l) wparam;		//adds a type to the subscription list
uint8_t ACLUnsubscribe(uint8_t LL_type_h,uint8_t LL_type_l) wparam;		//deletes a type out of the subscription list
uint8_t ACLFlushSubscriptions();						//deletes all subscription (not the default ones)
uint8_t ACLSubscribeDefault();							//subscribes to the default types (control msgs..)
uint8_t ACLVerifySubscription(uint8_t type_h,uint8_t type_l) wparam;		//checks if a subscription is there
void ACLInit();									//start ACL and lower layers, resets the whole stack

uint8_t ACLProcessControlMessages();						//internal: is called if control msg is there
void ACLSetFieldStrength(uint8_t power) wparam;					//sets the field strength of transmitter signals
uint8_t ACLSendingBusy();							//returns true if LL has Packet in send queue
uint8_t ACLGetSendSuccess();							//returns the result of last Packet transmission
uint8_t ACLSendPacket(uint8_t slot_timeout) wparam;				//starts packet transmission, return values!
uint8_t ACLAddNewType(uint8_t type_h, uint8_t type_l) wparam;			//adds a type into the ACL send buffer, return values!
uint8_t ACLAddData(uint8_t __PIC_data) wparam;					//adds one byte data into the ACL send buffer, return values!
void ACLAbortSending();								//stops a running transmission
void ACLSubscribeAll();								//subscribes to any possible type (all packets are received)
uint8_t ACLMatchesMyIP(unsigned char *buffer,uint8_t start) wparam;		//checks, if buffer holds my IP
unsigned char* ACLGetReceivedData(uint8_t type_h, uint8_t type_l) wparam;	//returns the ACL payload data of the given type in the last received packet
uint8_t ACLGetReceivedPayloadLength();						//returns the number of bytes received
void ACLSetControlMessagesBehaviour(boolean ignore, boolean pass) wparam;	//if ignore==true: don't react on any control msg; if pass==true: give control msgs to ACL as received payload packets
int8_t ACLGetReceivedDataLength(uint8_t type_h, uint8_t type_l) wparam;	        //returns number of bytes received of given type in last msg
uint8_t ACLSentPacketInThisSlot();						//is true until next slot if msg was send in this slot
uint8_t ACLClearSendData();							//deletes the send buffer of ACL
void ACLStart();								//restart ACL after ACLstop()
void ACLStop();									//stops the RF stack. Everything is hold, continues after ACLstart. msg stay in queue
uint8_t ACLGetRemainingPayloadSpace();						//returns the number of free bytes in the transmit buffer
void ACLLockReceiveBuffer();							//locks the receive buffer: no new msgs are received
uint8_t ACLReceiveBufferLocked();						//returns true if receivebuffer is locked
void ACLReleaseReceiveBuffer();							//un-lock the receivebuffer;
void ACLSetDataToOld();								//set received data to "old": means ACLDataIsNew will not return true unless a new packet was received
void ACLSetDataToNew();								//set received data to "new": means ACLDataIsNew will return true every slot
uint8_t ACLDataIsNew();								//returns true if data in receive buffer is "new"
unsigned char ACLGetReceivedByte(uint8_t type_h, uint8_t type_l, uint8_t position) wparam;	//returns a single received byte on the position
void ACLStartup();								// is the first function in any case,  runs selftest and aclinit
uint8_t ACLDataIsNewNow();							//==ACLDataReceivedin this slot
uint8_t ACLFoundReceivedType(uint8_t type_h, uint8_t type_l) wparam;		//returns true if type was found
uint8_t ACLAdressedDataIsNew();							// return1 if adressed data came
uint8_t ACLAdressedDataIsNewNow();						// return 1 if adressed data came in this slot
uint8_t ACLSendPacketAdressed(uint8_t add1, uint8_t add2, uint8_t add3, uint8_t add4, uint8_t add5, uint8_t add6, uint8_t add7, uint8_t add8, uint8_t timeout ) wparam; //sends out the current packet with a target ID


//DEBUG, APP, SERVICE, and all the rest
void DebugBuildTestPacketACL();


#define ACL_TYPE_ACM_H		ACL_TYPE_ACM_HI
#define ACL_TYPE_ACM_L		ACL_TYPE_ACM_LO
#define ACL_SUBSCRIPTIONLIST_LENGTH	8
#define ACL_CONTROL_MESSAGES_TIMEOUT	30

extern uint8_t ACL_subscriptions[ACL_SUBSCRIPTIONLIST_LENGTH][2];
extern boolean ACL_subscribe_all;
extern boolean ACL_ignore_control_messages;
extern boolean ACL_pass_control_messages;

extern uint8_t ACL_payload_send[LL_PAYLOAD_SIZE];
extern uint8_t ACL_payload_length;


#endif // __ACL_H__
