/**
  * Copyright (c) 2004, Telecooperation Office (TecO),
  * Universitaet Karlsruhe (TH), Germany.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
  *
  *   * Redistributions of source code must retain the above copyright
  *     notice, this list of conditions and the following disclaimer.
  *   * Redistributions in binary form must reproduce the above
  *     copyright notice, this list of conditions and the following
  *     disclaimer in the documentation and/or other materials provided
  *     with the distribution.
  *   * Neither the name of the Universitaet Karlsruhe (TH) nor the names
  *     of its contributors may be used to endorse or promote products
  *     derived from this software without specific prior written
  *     permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
  * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
  * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
  * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
  * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */

/**
* This File is the otap loader (placed at the end of the processor flash memory
* HOW TO produce a new otap loader:
* compile the otap_loader.c;
*
* convert it with the hex to rom file and name it flash_loader.rom
*
*
* then find the jump (see flash_loader.lst) to the high area (should be a goto at 0000)
* place that address in the flash_loader.rom file with the following defs (e.g.):
*
* #define OTAP_LOADER_MAIN				0x1F436
* #define OTAP_LOADER_MAIN_U			0x1			//  	Upper byte
* #define OTAP_LOADER_MAIN_H			0xF4		//		High byte
* #define OTAP_LOADER_MAIN_L			0x36		//		Low byte
*
*
* these defs only depend on the processor (these defs must be copied to the flash_loader.rom as well, must be equal to the ones here in flash_loader.c
* #define OTAP_LOADER_START 		0x1E700
* #define OTAP_LOADER_END   		0x1FFFF		// the end of the memory of 18f6720
* #define OTAP_PIC_MEMORY_START 	0x06
* #define OTAP_PIC_MEMORY_END 		0x1FFFF
*
* further, define the 
*
* TecO, University of Karlsruhe, Germany
* http://www.teco.edu
* 2004/08/12
*
*
* version 001 (krohn, 2004-08-12)
*
* Date:
*  2005_02_01 Daniel & Raphael
*  new Particle Base System organization
*
**/


/* 
 * If your loader does not fit into the reserved memory (look at loader.lkr)
 * you may modify the reserved area AND the corresponding entry in the linker
 * script for the applications.
 */



#include "flash_loader.h"	// own pins and stuff


uint8_t at 0x30 ser_data;
uint8_t at 0x31 crch;
uint8_t at 0x32 crcl;
uint8_t at 0x33 cdata[16];


uint8_t at 0x50 i;
uint8_t at 0x51 j;
uint8_t at 0x52 parts;
uint8_t at 0x53 bcounter;
uint8_t at 0x54 offset;


/* IMPORTANT:
 * to reduce program code and runtime the following function is a modified
 * version of write_program_memory found in tools.h.
 * To reduce call overhead we changed the following things:
 * - no parameter for data to write, we take the data from the global
 *   variable cdata
 * - no parameter for the number of bytes to write. It is hardcoded to 16
 */

void write_program_memory_line(uint32_t address) wparam {
  unsigned char tsa[64];
  uint32_t  waddr;

  waddr = address & 0xFFFFC0;
  offset = address & 0x3F;

  for (parts = 0; parts < ((16 -1  + (address & 0x3F)) / 64)+1  ; parts ++) {
    
    TBLPTRU = (waddr >> 16);
    TBLPTRH = ((waddr & 0xFF00) >> 8);
    TBLPTRL = (waddr & 0xC0);
    
    /* copy */
    for (i = 0; i < 64; i++) {
      _asm TBLRD *+ _endasm;
      tsa[i] = TABLAT;
    }
    
    /* modify */
    bcounter = 0;
    for (i = 0; (i < 64-offset) && (bcounter < 16); i ++) {
      bcounter ++;
      tsa[i+offset] = cdata[i];
    }
    
    TBLPTRU = (waddr >> 16);
    TBLPTRH = ((waddr & 0xFF00) >> 8);
    TBLPTRL = (waddr & 0xC0);
    
    /* erase */
    _asm
      BSF     _EECON1, 7
      BCF     _EECON1, 6
      BSF     _EECON1, 2 // WREN
      BSF     _EECON1, 4 // FREE
      BCF     _INTCON, 7
      MOVLW   0x55
      MOVWF   _EECON2
      MOVLW   0xAA
      MOVWF   _EECON2
      BSF     _EECON1, 1
      NOP
      BSF     _INTCON, 7
    _endasm;

    /* program */
    
    for (i = 0; i < 8; i++) {
      for (j = 0; j < 7; j++) {
	TABLAT = tsa[i*8+j];
	_asm
	  TBLWT *+
	_endasm;
      }

      TABLAT = tsa[i*8+7];
      _asm
	TBLWT * // pointer has to point on the last byte to write
      _endasm;


      _asm 
	BSF    _EECON1, 7
	BCF    _EECON1, 6
	BSF    _EECON1, 2
	BCF    _INTCON, 7
	MOVLW  0x55
	MOVWF  _EECON2      
	MOVLW  0xAA
	MOVWF  _EECON2      
	BSF    _EECON1, 1  
	NOP
	BSF    _INTCON, 7
	TBLRD *+ // dummy read to increment pointer
      _endasm;

    }

    _asm
      BCF    _EECON1, 2
    _endasm;
  }
  return;
}


#define ota_readnextline_exteeprom(count) { \
	uint8_t i; \
	for(i=0;i<count;i++) cdata[i]=SPIShift(FLASH_DONT_CARE); }


/* keep in mind that CRC16 uses global variables instead of parameters
 * to reduce call overhead. So this function has SIDE EFFECTS!
 */
void CRC16()
{
  uint8_t tmp;
  tmp = crcl;
  crcl = crch;
  crch = tmp;
  
  crcl ^= ser_data;
  crcl ^= crcl >> 4;
  crch ^= crcl << 4;
  
  crch ^= crcl >> 3;
  crcl ^= (crcl << 4) << 1;

}


void main() _naked
{
        uint8_t ota_NumberOfTables,i,rec_crch,rec_crcl,l;
	uint8_t buff[10];
	uint16_t burnAddr,burnBlock,burnOffset,line,EepromDataAddr,EepromTablesAddr,k,ProgramLines;


	set_pin_to_output(RED);
	PCLedRedOff();
	

	FlashInit();
	
	ProgramLines=0;
	EepromTablesAddr=5;
	EepromDataAddr=64;

	FlashReadSequence(0,buff,10);
	rec_crch=buff[0];
	rec_crcl=buff[1];
	ProgramLines = (buff[2] << 8) | buff[3];
	ota_NumberOfTables=buff[4];

	crch=0;crcl=0;

	for(i=0;i<ota_NumberOfTables;i++)
	  {
	    
	    FlashReadSequence((uint32_t)EepromTablesAddr,buff,4);
	    burnAddr = (buff[0] << 8) | buff[1];
	    line = (buff[2] << 8) | buff[3];
	    
	    
	    burnBlock=burnAddr&0xFFC0;
	    burnOffset=burnAddr&0x3F;

	    EepromTablesAddr+=4;
	    
	    FlashSetReadPointer((uint32_t) EepromDataAddr);
	    
	    for(k=0;k<line;k++)
	      {
		if (bit_test(k,4))
		  {
		    PCLedBlueOn();
		    PCLedRedOn();
		  }
		else
		  {
		    PCLedBlueOff();
		    PCLedRedOff();
		  }
		
		ota_readnextline_exteeprom(16);
		EepromDataAddr+=16;

		write_program_memory_line(burnAddr);
		read_program_memory(burnAddr,cdata,16);

		for(l=0;l<16;l++)
		  {
		    ser_data = cdata[l];
		    CRC16();
		  }
		burnAddr+=16;
	      }
	    FlashDisable();//important, the flash must go back to idle mode
	    
	  }
	
	
	ser_data = rec_crch;
	CRC16();
	ser_data = rec_crcl;
	CRC16();
	
	EepromDataAddr=64;
	EepromTablesAddr=5;


	if (crch != 0 && crcl !=0) { // in case of ERROR
	  PCLedBlueOff();
	  PCLedRedOn();
	  while(1);
	}



	PCLedRedOff();
	PCLedBlueOff();


	reset_cpu();
}
