/**
  * Copyright (c) 2004, Telecooperation Office (TecO),
  * Universitaet Karlsruhe (TH), Germany.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
  *
  *   * Redistributions of source code must retain the above copyright
  *     notice, this list of conditions and the following disclaimer.
  *   * Redistributions in binary form must reproduce the above
  *     copyright notice, this list of conditions and the following
  *     disclaimer in the documentation and/or other materials provided
  *     with the distribution.
  *   * Neither the name of the Universitaet Karlsruhe (TH) nor the names
  *     of its contributors may be used to endorse or promote products
  *     derived from this software without specific prior written
  *     permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
  * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
  * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
  * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
  * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 **/

/*
 *
 * Date:
 *  2005_02_01 Daniel & Raphael
 *  new Particle Base System organization
 *
 */

#include "ll.h"
#include "rf.h"
#include "acl.h"

uint8_t LL_slots_left;
uint8_t LL_sequence_no;
boolean LL_receive_buffer_locked;
uint16_t LL_longbuffer;

ll_data_in_last_slot	LL_last_data;
boolean LL_received_packet_in_this_slot;	//the reincarnation of the good old mac_msg_recieved_in_last_slot
uint8_t LL_payload_send[LL_PAYLOAD_SIZE];
uint8_t LL_payload_receivebuffer[LL_PAYLOAD_SIZE];
uint8_t LL_payload_received[LL_PAYLOAD_SIZE];
uint8_t LL_header_send[LL_HEADER_SIZE];
uint8_t LL_header_receivebuffer[LL_HEADER_SIZE];
uint8_t LL_header_received[LL_HEADER_SIZE];
uint8_t LL_tail_send[LL_TAIL_SIZE];
uint8_t LL_tail_receivebuffer[LL_TAIL_SIZE];
uint8_t LL_tail_received[LL_TAIL_SIZE];
uint8_t LL_payload_length;			// hold the actual size of LL frame (not includes ll header, lltail)



uint8_t LL_payload_write_pos;



// here comes the Link Layer
//---------------------------------------------------------------------------------------------------------------------------------------
//---------------------------------------------------------------------------------------------------------------------------------------
//---------------------------------------------------------------------------------------------------------------------------------------
//------------------------------LL-------------------------------------------------------------------------------------------------------
//---------------------------------------------------------------------------------------------------------------------------------------
//---------------------------------------------------------------------------------------------------------------------------------------
//---------------------------------------------------------------------------------------------------------------------------------------
//---------------------------------------------------------------------------------------------------------------------------------------





uint8_t LLGetIDFromHardware()
{

	uint8_t buff;
	//return;
	//FlashInit();
	//RFPowerModuleOn();
	//RFSetModeSleep();

	//check if valid ID in Eeprom:

        // the new stuff: get the id from the 1-wire
  
  
        buff=PCGetID(&LL_header_send[3]);
        return buff;
  
        /* storage of id is discontinued at the moment...
        //
 


	result=0;
	for (i=0;i<=7;i++)
	{
		buff=ReadIntEeprom(EEPROM_ADDRESS_ID+i);
		LL_header_send[3+i]=buff;
		if (buff==255) result++;
	}
	if (result<8)
	{
		//ID in internal EEProm was valid!
		//now write it to external eeprom and exit


		FlashWriteSequence(FLASH_ID_ADDRESS, &LL_header_send[3],8);
		FlashFlush();
		return 1;
	}


	// id in internal EEPROM was invalid; try external eeprom

		FlashReadSequence(FLASH_ID_ADDRESS,&LL_header_send[3],8);
		result=0;
		for(i=0;i<=7;i++)
		{
			buff=LL_header_send[3+i];
			if (buff==255) result++;
		}
		if (result<8)
		{
			//external was ok, write it to internal
			for(i=0;i<=7;i++)
			{
				WriteIntEeprom(EEPROM_ADDRESS_ID+i,LL_header_send[i+3]);
			}
			return 1;
		}



	LL_header_send[3] = 10;
	LL_header_send[4] = 1;
	LL_header_send[5] = 0;
	LL_header_send[6] = 1;
	LL_header_send[7] = 10;
	LL_header_send[8] = 1;
	LL_header_send[9] = 0;
	LL_header_send[10] = 216;

	return 1;		//both were invalid: there's no valid id
	*/
}






void LLInit()
{
	LL_sequence_no=0;


	//prepare standard header with Ids



	LL_header_send[0]=VERSION;



	/*
	LL_header_send[3]=ReadIntEeprom(EEPROM_ADDRESS_ID+0);
	LL_header_send[4]=ReadIntEeprom(EEPROM_ADDRESS_ID+1);
	LL_header_send[5]=ReadIntEeprom(EEPROM_ADDRESS_ID+2);
	LL_header_send[6]=ReadIntEeprom(EEPROM_ADDRESS_ID+3);
	LL_header_send[7]=ReadIntEeprom(EEPROM_ADDRESS_ID+4);
	LL_header_send[8]=ReadIntEeprom(EEPROM_ADDRESS_ID+5);
	LL_header_send[9]=ReadIntEeprom(EEPROM_ADDRESS_ID+6);
	LL_header_send[10]=ReadIntEeprom(EEPROM_ADDRESS_ID+7);
	*/


	LL_payload_length=0;	// set used ll payload to zero
	LL_receive_buffer_locked=false;
	LL_received_packet_in_this_slot=false;
	LL_last_data=LL_NONE;

	RFInit();

	LLGetIDFromHardware();	//take id from internal or external eeprom and set ll_header_send
}




void LLStart()
{
	RFStart();
}


void LLStop()
{
	RFStop();	// handles the states
}



void LLSlotEnd()
{
	// check if timeout for waiting packet
	if (LLSendingBusy())
	{
		LL_slots_left--;
		if (LL_slots_left==0)
		{
			rf_status&=~RF_STATUS_SENDING;					// busy is over

			if (rf_status & RF_STATUS_INSERTED_CTRLMSG)
			{
				rf_status&=~RF_STATUS_INSERTED_CTRLMSG;		// in case it was switched on, switch it off and do NOT update the success state
			}
			else
			{
				rf_status&=~RF_STATUS_SEND_SUCCESS;
			}

			LL_payload_length=0;							// set used payload to zero
		}
	}

	// if msg received in last slot set appropr. bit

}



void LLAbortSending()
{
	if (LLSendingBusy())
	{
		if (rf_status & RF_STATUS_INSERTED_CTRLMSG)
		{
			rf_status&=~RF_STATUS_INSERTED_CTRLMSG;
			rf_status&=~RF_STATUS_SENDING;
		}
		else
		{
			rf_status&=~RF_STATUS_SEND_SUCCESS;
			rf_status&=~RF_STATUS_SENDING;
		}
		LL_payload_length=0;
	}
}


uint8_t LLSetSendingSuccess()
{
	//update the states
	if (rf_status & RF_STATUS_INSERTED_CTRLMSG)
	{
		// no update of success flag
		rf_status&= ~RF_STATUS_INSERTED_CTRLMSG;	// clear the control management bit
		rf_status&= ~RF_STATUS_SENDING;				// busy is over
		LL_payload_length=0;						// set used payload to zero
	}
	else
	{
		rf_status|= RF_STATUS_SEND_SUCCESS;			// succesfull sent
		rf_status&= ~RF_STATUS_SENDING;				// busy is over
		LL_payload_length=0;						// set used payload to zero
	}
	return 0;
}



uint8_t LLSendingBusy()
{
	if (rf_status & RF_STATUS_SENDING) return 1;
	return 0;
}

// !! this returns the last(!) result, also if sending is active at the moment
uint8_t LLGetSendSuccess()
{
	if (rf_status & RF_STATUS_SEND_SUCCESS) return 1;
	else return 0;
}

uint8_t LLIsActive()
{
	if (rf_status& RF_STATUS_LAYER_ON) return 1;
	else return 0;
}


void LLLockReceiveBuffer()
{
	LL_receive_buffer_locked=true;

	LLSetDataToOld();
}

uint8_t LLReceiveBufferLocked()
{
	if (LL_receive_buffer_locked==true) return 1;
	return 0;
}


void LLReleaseReceiveBuffer()
{
	LL_receive_buffer_locked=false;
	LLSetDataToOld();
}


void LLSetDataToOld()
{
	rf_status&=~RF_STATUS_NEW_DATA;
}

void LLSetDataToNew()
{
	rf_status|=RF_STATUS_NEW_DATA;
}


uint8_t LLDataIsNew()
{
	if (rf_status & RF_STATUS_NEW_DATA) return 1;
	return 0;
}



void DebugBuildTestPacketACL()
{

	// this is only testcode

	ACLAddNewType(ACL_TYPE_ACM_H,ACL_TYPE_ACM_L);
	ACLAddNewType(115,216);	//helo

	ACLAddData(255);
	ACLAddData(255);
	ACLAddData(255);
	ACLAddData(255);

	ACLAddData(255);
	ACLAddData(255);
	ACLAddData(255);
	ACLAddData(255);


}




uint8_t LLGetFieldStrength()
{
	return rf_field_strength;
}

void LLSetFieldStrength(uint8_t value) wparam
{
	RFSetFieldStrength(value);
}


uint16_t LLCalcCRC16(uint8_t *header_data, uint8_t *payload_data, uint8_t payload_size) wparam
{
	//calcs a crc on LL_header, ACL_data, LL_tail
	uint8_t hb,lb,i,tmp;
	lb=0;
	hb=0;

	for(i=0;i<LL_HEADER_SIZE;i++)
	{
		tmp = lb;
		lb = hb;
		hb = tmp;
		lb ^= header_data[i];
		lb ^= lb >> 4;
		hb ^= lb << 4;
		hb ^= lb >> 3;
		lb ^= (lb << 4) << 1;
	}

	//len=header_data[1]-LL_HEADER_SIZE-LL_TAIL_SIZE;
	for(i=0;i<payload_size;i++)
	{
		tmp = lb;
		lb = hb;
		hb = tmp;
		lb ^= payload_data[i];
		lb ^= lb >> 4;
		hb ^= lb << 4;
		hb ^= lb >> 3;
		lb ^= (lb << 4) << 1;
	}


	return hb*256+lb;
}







// prepares the header and sends the packet out
uint8_t LLSendPacket(uint8_t slot_limit) wparam
{
	//uint16_t crc16; // makes RAM too full if local var is active



	if (slot_limit==0) slot_limit=1;	// zero makes no sense

	if (!(rf_status & RF_STATUS_LAYER_ON)) return 0;
	if (rf_status & RF_STATUS_SENDING) return 0;


	// finalize the LLPacket

	//DebugGiveOut(LL_payload_length);


	LL_header_send[1]=LL_payload_length;
	LL_header_send[2]=rf_field_strength;
	LL_header_send[11]=LL_sequence_no;


	LL_longbuffer=LLCalcCRC16(LL_header_send,LL_payload_send, LL_payload_length);

	LL_tail_send[0]=(LL_longbuffer>>8);	//CRCh (implicit typecast; takes low byte from uint16_t)
	LL_tail_send[1]=LL_longbuffer;		//CRCl

	LL_slots_left=slot_limit;
	LL_sequence_no++;

	rf_status|= RF_STATUS_SENDING;

	return 1;
}


uint8_t LLGetRemainingPayloadSpace()
{

	return (LL_PAYLOAD_SIZE-LL_payload_length);

}


uint8_t LLSentPacketInThisSlot()
{
	if (rf_status & RF_STATUS_JUST_SENT) return 1;
	return 0;
}


