#ifndef __LL_H__
#define __LL_H__

/**
  * Copyright (c) 2004, Telecooperation Office (TecO),
  * Universitaet Karlsruhe (TH), Germany.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
  *
  *   * Redistributions of source code must retain the above copyright
  *     notice, this list of conditions and the following disclaimer.
  *   * Redistributions in binary form must reproduce the above
  *     copyright notice, this list of conditions and the following
  *     disclaimer in the documentation and/or other materials provided
  *     with the distribution.
  *   * Neither the name of the Universitaet Karlsruhe (TH) nor the names
  *     of its contributors may be used to endorse or promote products
  *     derived from this software without specific prior written
  *     permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
  * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
  * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
  * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
  * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 **/

#include <tools.h>

#define LL_HEADER_SIZE			12
#define LL_PAYLOAD_SIZE			64
#define LL_TAIL_SIZE			2
#define LL_FRAME_MAX_SIZE		(LL_HEADER_SIZE+LL_PAYLOAD_SIZE+LL_TAIL_SIZE)

enum ll_data_in_last_slot{LL_CRC_ERROR,LL_ACM,LL_GOOD,LL_NONE};
typedef enum ll_data_in_last_slot ll_data_in_last_slot; 

void LLStart();
void LLStop();
uint8_t LLSendPacket(uint8_t slot_limit) wparam;
uint8_t LLSendingBusy();
uint8_t LLGetSendSuccess();
uint8_t LLIsActive();
void LLLockReceiveBuffer();
void LLReleaseReceiveBuffer();
uint8_t LLReceiveBufferLocked();
void LLSlotEnd();
void LLSetDataToOld();
void LLSetDataToNew();
void LLInit();
uint8_t  LLGetFieldStrength();
void LLSetFieldStrength(uint8_t value) wparam;
uint16_t LLCalcCRC16(uint8_t *header_data,uint8_t *payload_data, uint8_t payload_size) wparam;//,uint8_t payload_size);
void LLAbortSending();
uint8_t LLSetSendingSuccess();
uint8_t LLDataIsNew();
uint8_t LLSentPacketInThisSlot();
uint8_t LLGetIDFromHardware();								// takes id either from internal or external eeprom

extern ll_data_in_last_slot	LL_last_data;
extern boolean LL_received_packet_in_this_slot;	//the reincarnation of the good old mac_msg_recieved_in_last_slot
extern uint8_t LL_payload_send[LL_PAYLOAD_SIZE];
extern uint8_t LL_payload_receivebuffer[LL_PAYLOAD_SIZE];
extern uint8_t LL_payload_received[LL_PAYLOAD_SIZE];
extern uint8_t LL_header_send[LL_HEADER_SIZE];
extern uint8_t LL_header_receivebuffer[LL_HEADER_SIZE];
extern uint8_t LL_header_received[LL_HEADER_SIZE];
extern uint8_t LL_tail_send[LL_TAIL_SIZE];
extern uint8_t LL_tail_receivebuffer[LL_TAIL_SIZE];
extern uint8_t LL_tail_received[LL_TAIL_SIZE];
extern uint8_t LL_payload_length;											// hold the actual size of LL frame (not includes ll header, lltail)


#endif // __LL_H__
