#ifndef __RF_H__
#define __RF_H__

/**
  * Copyright (c) 2004, Telecooperation Office (TecO),
  * Universitaet Karlsruhe (TH), Germany.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
  *
  *   * Redistributions of source code must retain the above copyright
  *     notice, this list of conditions and the following disclaimer.
  *   * Redistributions in binary form must reproduce the above
  *     copyright notice, this list of conditions and the following
  *     disclaimer in the documentation and/or other materials provided
  *     with the distribution.
  *   * Neither the name of the Universitaet Karlsruhe (TH) nor the names
  *     of its contributors may be used to endorse or promote products
  *     derived from this software without specific prior written
  *     permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
  * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
  * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
  * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
  * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 **/

#include "../boards/coreboard.h"



#define VERSION			5
#define MY_MAC_ID                141

// this is debug stuff
#define SENDER			0
#define DEBUG			0
#define SYNC_DEBUG		0		// set this to one: on pin 4 is the sync time visible to compare with other devices


void RFInit();


void RFSpiWait() _naked;
void RFDelaySlots(uint8_t slots) wparam;
void RFWaitForTimer1Exact(uint8_t timeout) wparam;
void RFWaitForNextBitTimeExact() _naked;
void RFNewRandom();
void RFStop();
void RFSetFieldStrength(uint8_t power) wparam;		// unimplemented
void RFReceiveByte();
uint16_t RFMicroSecondsUntilNextSlot();


#define RFSetSyncRate(rate) rf_sync_rate=rate

#define RFSetInitialListenSlots(slots) rf_initial_listen_slots=slots

#define RFSetModeSleep()    {output_low(PIN_RF_CTR0); output_low(PIN_RF_CTR1);}

#define RFSetModeAsk()      {output_low(PIN_RF_CTR0); output_high(PIN_RF_CTR1);}

#define RFSetModeOok()      {output_high(PIN_RF_CTR0); output_low(PIN_RF_CTR1);}

#define RFSetModeReceive()  {output_high(PIN_RF_CTR0); output_high(PIN_RF_CTR1); output_high(PIN_RF_SENSITIVE);}

#define RFSetModeSensitive() {output_high(PIN_RF_CTR0); output_high(PIN_RF_CTR1); output_low(PIN_RF_SENSITIVE);}

#define RFSpiClear() 	_asm MOVF _SSPBUF, W _endasm

#define RFSpiSendByte() {_asm MOVF _SSPBUF, W _endasm; _asm MOVWF _rf_rx1 _endasm; _asm MOVF _rf_tx, W _endasm; _asm MOVWF _SSPBUF _endasm;}

#define RFDeleteTimer1Overflow() bit_clear(RF_TIMER1_IF) /* clear TMR1IF i */


#define RFUartStartContinuous() {_asm bcf _RCSTA2,5 _endasm; _asm bcf _RCSTA2,4 _endasm; _asm bsf _RCSTA2,4 _endasm;}

#define RFUartClearFifo() {_asm btfsc _PIR3,5 _endasm; _asm movf _RCREG2,W _endasm; _asm btfsc _PIR3,5 _endasm; _asm movf _RCREG2,W _endasm;}
#define RFUartOff() _asm bcf _RCSTA2,7 _endasm

#define RFUartOn() _asm bsf _RCSTA2,7 _endasm

#define RFSpiOff() {output_low(PIN_RF_TX); _asm bcf _SSPCON1, 5 _endasm;}

#define RFSpiOn() _asm bsf _SSPCON1,5 _endasm 

#define RFScramble() {_asm clrf _rf_transition1 _endasm; _asm BTFSC _rf_transition0, 3 _endasm; _asm incf _rf_transition1, F _endasm; _asm btfsc _rf_transition0, 6 _endasm; _asm incf _rf_transition1, F _endasm; _asm bcf _STATUS,0 _endasm; _asm rlcf _rf_transition0, F _endasm; _asm btfsc _rf_transition1,0 _endasm; _asm bsf _rf_transition0,0 _endasm; }

#define RFStart() {rf_sync_state=SYNC_STATE_ALONE; if (DEBUG) rf_sync_state=SYNC_STATE_DEBUG; RFSetModeSleep(); RFPowerModuleOn(); DelayMs(90); RFSetFieldStrength(rf_field_strength); PCLedRedOn(); DelayMs(200); PCLedRedOff(); DelayMs(200); rf_status|=RF_STATUS_LAYER_ON; RFDeleteTimer1Overflow(); set_timer1(65000); set_timer2(0); enable_interrupts(INT_TIMER1);}


void SlotEndCallBack();



// RF stati
#define RF_STATUS_SENDING			0x01
#define	RF_STATUS_SEND_SUCCESS		0x02
#define RF_STATUS_NEW_PACKET		0x04
#define RF_STATUS_RECEIVE_ERROR		0x08
#define RF_STATUS_LAYER_ON			0x80		// one if stack is running
#define RF_STATUS_INSERTED_CTRLMSG	0x40		// is being set if waiting normal msg was interrupted due t incoming control/management msg
#define RF_STATUS_NEW_DATA			0x10		// new data arrived
#define RF_STATUS_JUST_SENT			0x20		// data send in this slot

// Sync states
#define SYNC_STATE_SYNCED		1
#define SYNC_STATE_ALONE		2
#define SYNC_MAX_MISSED_SYNCS 	7		// after these empty sync=> state=alone
#define SYNC_RECOVER_SYNCS		4		// after these syncs it's recover state (only sync receive)
#define SYNC_STATE_DEBUG		99
//sync modes
#define RF_SYNC_MODE_ACTIVE		1		// searches for partners and syncs
#define RF_SYNC_MODE_PASSIVE	2		// does not search for partners but syncs

#define RF_SYNC_RATE_DEFAULT	80		// normal = 30% sync rate


extern volatile sfr at 0x40 rf_rx1;
extern volatile sfr at 0x41 rf_rx2;
extern volatile sfr at 0x42 rf_buff;
extern volatile sfr at 0x43 rf_thisbyte;
extern volatile sfr at 0x44 rf_pos;
extern volatile sfr at 0x45 rf_period;
extern volatile sfr at 0x46 rf_duty_cycle;
extern volatile sfr at 0x47 rf_bit_count;
extern volatile sfr at 0x48 rf_shift;
extern volatile sfr at 0x49 rf_transition0;
extern volatile sfr at 0x4a rf_transition1;
extern volatile sfr at 0x4b rf_transition2;
extern volatile sfr at 0x4c rf_transition3;
extern volatile sfr at 0x4d rf_limit;
extern volatile sfr at 0x4e rf_random_l;
extern volatile sfr at 0x4f rf_random_h;
extern volatile sfr at 0x50 rf_last_err;
extern volatile sfr at 0x51 rf_timeout;	// current random number. used for generating "random" numbers in RFGetRandom()
extern volatile sfr at 0x52 rf_tx;


extern uint8_t rf_status;		// holds the state of rf layer like busy, receiving succes , err, etc
extern uint8_t rf_sync_state;
extern uint8_t rf_field_strength;
extern uint8_t rf_LED_style;
extern uint16_t rf_slotcounter;
extern uint8_t rf_sync_rate;		// 0 means never send a sync; 255 means always send a sync
extern uint8_t rf_initial_listen_slots;									// the number of listenslots before a lonely note sends a sync


extern boolean selftest_active;



#define timer1_handler fsm
void fsm();

#endif // __RF_H__
