/**
  * Copyright (c) 2004, Telecooperation Office (TecO),
  * Universitaet Karlsruhe (TH), Germany.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
  *
  *   * Redistributions of source code must retain the above copyright
  *     notice, this list of conditions and the following disclaimer.
  *   * Redistributions in binary form must reproduce the above
  *     copyright notice, this list of conditions and the following
  *     disclaimer in the documentation and/or other materials provided
  *     with the distribution.
  *   * Neither the name of the Universitaet Karlsruhe (TH) nor the names
  *     of its contributors may be used to endorse or promote products
  *     derived from this software without specific prior written
  *     permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
  * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
  * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
  * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
  * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 **/


/**
* Driver for real time clock in the new hardware layout (external timer3)
*
* Author: Christian Decker (cdecker@teco.edu)
* Date: 2003-06-10 (yyyy-mm-dd)
*
* Current state: template
*
* Date:
*  2005_02_01 Daniel & Raphael
*  new Particle Base System organization
*
*/

#include "rtc.h"

#define RTC_SLOTS_TO_WAIT_FOR_TIMESERVER	100

uint8_t RTC_second;
uint8_t RTC_minute;
uint8_t RTC_hour;
uint8_t RTC_day;
uint8_t RTC_month_year;
uint8_t	RTC_got_time = 0;

uint8_t	RTC_updated;

/**
* Initializes real time clock.
* @return returns 0 if no error
*/
void RTCInit()
{
	ClockTimerInit(CLOCK_32kHz);
	//bit_set( PIC_T1CON, 3 );
	//PIC_T3CON = 0x77;
	DelayUs( 50 );
	RTCSetClock( 0, 0, 0, 0 );
	RTCSetDate( 0, 0, 3 );
	ACLSubscribe( ACL_TYPE_CTA_HI, ACL_TYPE_CTA_LO );
	if( RTCGetTimeFromServer()) RTC_got_time = 1;
}

void RTCSetClock( uint8_t hour, uint8_t minute, uint8_t second, uint16_t millisecond ) wparam
{
	RTC_hour = hour;
	RTC_minute = minute;
	RTC_second = second;
	RTCSetMillisecond( millisecond );
}

void RTCSetDate( uint8_t day, uint8_t month, uint8_t year ) wparam
{
	RTC_day = day;
	RTCSetMonth( month );
	RTCSetYear( year );
}

void RTCGetClock( uint8_t* hour, uint8_t* minute, uint8_t* second, uint16_t* millisecond ) wparam
{
	*hour = RTC_hour;
	*minute = RTC_minute;
	*second = RTC_second;
	*millisecond = RTCGetMillisecond();
}

void RTCGetDate( uint8_t* day, uint8_t* month, uint8_t* year ) wparam
{
	*day = RTC_day;
	*month = RTC_month_year & 0x0F;
	*year = ( RTC_month_year & 0xF0 ) >> 4;
}

uint8_t RTCGetTimeFromServer()
{
	unsigned char* buff;
	uint16_t wait;
	uint8_t result = 0;

	wait = RTC_SLOTS_TO_WAIT_FOR_TIMESERVER;

	RTCSetClock( 0, 0, 0, 0 );
	RTCSetDate( 0, 0, 0 );

	while ( wait )
	{
		if( !ACLSendingBusy())
		{
			ACLClearSendData();
			ACLAddNewType( ACL_TYPE_CTR_HI, ACL_TYPE_CTR_LO );
			ACLSendPacket( 20 );
			wait--;
		}
		if ( ACLDataIsNew() )
		{
			if( ACLFoundReceivedType( ACL_TYPE_CTA_HI, ACL_TYPE_CTA_LO ))
			{
				buff = ACLGetReceivedData( ACL_TYPE_CTA_HI, ACL_TYPE_CTA_LO );
				RTCSetClock( buff[3], buff[4], buff[5], 0 );
				RTCSetDate( buff[2], buff[1], buff[0] );
				result = 1;
				break;	// got time from server;
			}
		}
	}
	return result;
}

void RTCAddTimeStampACL()
{
	uint8_t hour;
	uint8_t minute;
	uint8_t second;
	uint16_t millisecond;
	uint8_t day;
	uint8_t month;
	uint8_t year;

	RTCGetClock( &hour, &minute, &second, &millisecond );
	RTCGetDate( &day, &month, &year );

	if( ACLGetRemainingPayloadSpace() >= 11 )
	{
		ACLAddNewType( ACL_TYPE_CTS_HI, ACL_TYPE_CTS_LO );
		ACLAddData( year );
		ACLAddData( month );
		ACLAddData( day );
		ACLAddData( hour );
		ACLAddData( minute );
		ACLAddData( second );
		ACLAddData( hi( millisecond ));
		ACLAddData( lo( millisecond ));
	}
}

void RTCAddTimeStampSpecificACL( uint8_t chose_data ) wparam
{

// chose_data: bit 	0 set: 		year 		required in time stamp.
//					1		 	month
//					2			day
//					3			hour
//					4			minute
//					5			second
//					6			milisecond

	uint8_t hour;
	uint8_t minute;
	uint8_t second;
	uint16_t millisecond;
	uint8_t day;
	uint8_t month;
	uint8_t year;
	uint16_t sum = 3;		// bytes for type & length.
	uint8_t i;

	RTCGetClock( &hour, &minute, &second, &millisecond );
	RTCGetDate( &day, &month, &year );

	for( i = 0; i < 7; ++i )
	{
		if( bit_test( chose_data, i )) ++sum;
	}

	if( bit_test( chose_data, 6 )) sum += 2;

	if( ACLGetRemainingPayloadSpace() >= sum )
	{
		ACLAddNewType( ACL_TYPE_CTS_HI, ACL_TYPE_CTS_LO );
		if( bit_test( chose_data, 0 )) ACLAddData( year );
		if( bit_test( chose_data, 1 )) ACLAddData( month );
		if( bit_test( chose_data, 2 )) ACLAddData( day );
		if( bit_test( chose_data, 3 )) ACLAddData( hour );
		if( bit_test( chose_data, 4 )) ACLAddData( minute );
		if( bit_test( chose_data, 5 )) ACLAddData( second );
		if( bit_test( chose_data, 6 ))
		{
			ACLAddData( hi( millisecond ));
			ACLAddData( lo( millisecond ));
		}
	}
}

/**
* Gets real time clock value
* @param value
* @return returns 0 if no error
*/
void RTCUpdate()
{
	uint8_t day;
	uint8_t month;
	uint8_t year;
	uint8_t* buff;

		RTC_updated = 0;
		while( ClockTimerGetHi() >= 16 )
		{
			RTC_updated = 1;
			ClockTimerSetHi(ClockTimerGetHi() - 16);
			++RTC_second;
			if( RTC_second == 60 )
			{
				RTC_second = 0;
				++RTC_minute;
				if( RTC_minute == 60 )
				{
					RTC_minute = 0;
					++RTC_hour;
					if( RTC_hour == 24 )
					{
						RTC_hour = 0;
						++RTC_day;
						if( RTC_day > 28 )
						{
							if( RTC_day > 31 )
							{
								RTC_day = 1;
								++RTC_month_year;
							}
							if(( RTC_day > 30 ) && ((( RTC_month_year & 0x0F ) == 4 ) || (( RTC_month_year & 0x0F ) == 6 ) || (( RTC_month_year & 0x0F ) == 9 ) || (( RTC_month_year & 0x0F ) == 11 )))
							{
								RTC_day = 1;
								++RTC_month_year;
							}
							if( RTC_month_year == 2 )
							{
								if( !( RTC_month_year & 0x30 ) && ( RTC_month_year & 0xF0 ))
								{
									if( day > 29 )
									{
										RTC_day = 1;
										++RTC_month_year;
									}
								}
								else if( day > 28 )
								{
									RTC_day = 1;
									++RTC_month_year;
								}
							}
							if(( RTC_month_year & 0x0F ) > 12 )
							{
								RTC_month_year &= 0xF0;
								RTC_month_year += 17;
							}
						}
					}
				}
			}
		}

		if( ACLDataIsNewNow())
		{
			if( ACLFoundReceivedType( ACL_TYPE_CTA_HI, ACL_TYPE_CTA_LO ))
			{
				buff = ACLGetReceivedData( ACL_TYPE_CTA_HI, ACL_TYPE_CTA_LO );
				RTCSetClock( buff[3], buff[4], buff[5], 0 );
				RTCSetDate( buff[2], buff[1], buff[0] );
				RTC_got_time = 1;
			}
		}
}

uint16_t RTCGetMillisecond()
{
	uint8_t image_TMR3L;
	uint16_t result;

		image_TMR3L = ClockTimerGetLo();
		result = ( uint16_t )( ClockTimerGetHi() ) * 625;
		result += (( uint16_t )( image_TMR3L ) * 24 / 10 );	// ungenau... 244, irgendwas m
		return( result / 10 );
}

uint8_t RTCGetMonth()
{
	return( RTC_month_year & 0x0F );
}

uint8_t RTCGetYear()
{
	return(( RTC_month_year & 0xF0 ) >> 4 );
}

void RTCSetMillisecond( uint16_t millisec ) wparam
{
	uint16_t image_TMR3H;
	uint16_t image_TMR3L;

		if( millisec > 999 ) return;
		image_TMR3H = millisec * 10 / 625;
		image_TMR3L = millisec - ( image_TMR3H * 625 / 10 );
		image_TMR3L = image_TMR3L * 100 / 25;				// ungenau... 244, irgendwas m

		ClockTimerSetHi( ( uint8_t )image_TMR3H);
		ClockTimerSetLo( ( uint8_t )image_TMR3L);
}

void RTCSetMonth( uint8_t mon ) wparam
{
	if( mon > 12 ) return;
	RTC_month_year &= 0xF0;
	RTC_month_year |= mon;
}

void RTCSetYear( uint8_t yea ) wparam
{
	if( yea > 15 ) return;
	RTC_month_year &= 0x0F;
	RTC_month_year |= ( yea << 4 );
}

void RTCIncYear()
{
	RTC_month_year += 16;
}
